<?php //

/*
 * Prevent Direct Access
 */
defined( 'ABSPATH' ) or die( "Sorry, Can not access page directly." );

/**
 * Class for functions not directly related to handling video campaign
 */
if ( ! class_exists( 'Autovid_Profit_Core' ) ) {

    class Autovid_Profit_Core {

	public $display_msgs = FALSE;	// internal: sends msgs to error log or display screen
	private $messages = array();            // level of messages to display

	protected $plugin = null;
	public $logger = null;


	public function __construct( $params = array() ) {
		$this->plugin = $params;
		$this->get_options_to_vars();

		#SINGLETON $this->logger = Autovid_Profit_Log::getInstance();
		$this->logger = new Autovid_Profit_Log();
		$this->logger ->init( $params );
                
                
	}

	/**
	 * Add a Settings shortcut link to the description for the plugin on the Plugins dashboard page
	 */
	public function main_settings_link( $links ) {
		$settings_link = '<a href="admin.php?page=' . $this->get_my_plugin( 'id' ) . '">Settings</a>';
		array_unshift( $links, $settings_link );
		return $links;
	}

	/*
	 * Add these menus to the admin dashboard
	 */
	public function add_menus() {
		// create top level menu on dashboard sidebar
        	add_menu_page( 
			$this->get_my_plugin( 'short_name' ) . ': Main Settings',	// page title
			$this->get_my_plugin( 'short_name' ),				// menu title
			'manage_options',				// capability required
			$this->get_my_plugin( 'id' ),				// menu id
			array( $this, 'main_settings' ),		// function to call
			'dashicons-video-alt3'				// icon
		);

        	//create submenu items for our menu
        	add_submenu_page( 
			$this->get_my_plugin( 'id' ), 
			$this->get_my_plugin( 'short_name' ) . ': Main Settings', 
			'Main Settings', 
			'manage_options', 
			$this->get_my_plugin( 'id' ), 
			array( $this, 'main_settings' ) 
		);
        	add_submenu_page( 
			$this->get_my_plugin( 'id' ),
			$this->get_my_plugin( 'short_name' ) . ': Make Video Campaign',
			'Make Video Campaign',
			'manage_options',
			$this->get_my_plugin( 'prefix' ) . 'video_campaign',
			array( $this, 'make_video_campaign' )
		);
        	add_submenu_page( 
			$this->get_my_plugin( 'id' ),
			$this->get_my_plugin( 'short_name' ) . ': Show Video Campaign',
			'Show Video Campaign', 
			'manage_options', 
			$this->get_my_plugin( 'prefix' ) . 'show_video_campaign', 
			array( $this, 'show_video_campaign' ) 
		);
        	add_submenu_page( 
			$this->get_my_plugin( 'id' ), 
			$this->get_my_plugin( 'short_name' ) . ': Scheduler', 
			'Scheduler', 
			'manage_options', 
			$this->get_my_plugin( 'prefix' ) . 'scheduler', 
			array( $this->scheduler, 'show_scheduler') 
		);

                do_action('avp_extra_menus');
                
                add_submenu_page( 
			$this->get_my_plugin( 'id' ), 
			$this->get_my_plugin( 'short_name' ) . ': Help', 
			'Help', 
			'manage_options', 
			$this->get_my_plugin( 'prefix' ) . 'help', 
			array( $this, 'avp_help_cb') 
		);
	} 
        
                /*
        * Help Video Section
        */ 

        public function avp_help_cb() {

            $youtubevideo_id = ''; 
            $help_video_bottom_description = '';
            $help_lastline = '';
            ?>
        <?php include( sprintf( "%sautovid-profit-help.php", AVP_PLUGIN_DIR ) ); ?>	
        <div class='wrap autovid-profit-section'>
                <h2 class='autovid-profit-title'><?php echo AVP_PLUGIN_NAME . " v" . AVP_PLUGIN_VERSION; ?></h2>
                <?php include( sprintf( "%stemplates/buttons.php", AVP_PLUGIN_DIR ) ); ?>	
        </div>
           <div class="wrap">
               <h3>AutoVid Profit - Help</h3>
               <?php if($help_videos)  { ?>  

               <ul class="help_training_videos">

               <?php foreach($help_videos as $video)  { ?>     
                   <li>

                       <h3><?php echo $video['title']; ?></h3>
                       <iframe width="560" height="315" src="https://www.youtube.com/embed/<?php echo $video['youtube_video_id']; ?>?rel=0&controls=0&&showinfo=0&loop=1    " frameborder="0" allowfullscreen></iframe>

                   </li> 
               <?php } ?>     
               </ul>
               <?php } ?> 
               
               <?php if($help_video_bottom_description!='') { ?>
               <div class="bottom_descrption"><?php echo $help_video_bottom_description; ?></div>
               <?php } ?> 
               
               <p><?php echo $help_lastline; ?></p>

           </div>



            <?php

        } 
        

	/**
	 * Verify database is up to date and set plugin option defaults
	 */
        public function activate() {

		// check for required versions of WordPress and PHP
		global $wp_version;
		$checks_passed = FALSE;
		$wp = "3.0";
		$php = "5.2";
		if ( version_compare( PHP_VERSION, $php, '<' ) ) {
			$flag = 'PHP';
		}
		elseif ( version_compare( $wp_version, $wp, '<' ) ) {
			$flag = 'WordPress';
		}
		else { // everything passes check
			$checks_passed = TRUE;
		}
		if ( FALSE === $checks_passed ) {
			$version = 'PHP' == $flag ? $php : $wp;
			$current_version = 'PHP' == $flag ? PHP_VERSION : $wp_version;
			deactivate_plugins( basename( $this->get_my_plugin( 'file' ) ) );
			wp_die('<p>Notice! The <strong>' . $this->get_my_plugin( 'name' ) . '</strong> plugin requires '.$flag.'  version '.$version.' or greater. <br />(Currently installed: '.$flag.' version '.$current_version.')</p>','Plugin Activation Error',  array( 'response'=>200, 'back_link'=>TRUE ) );
		}

		if ( get_option( $this->get_my_plugin( 'prefix' ) . 'db_version' ) != $this->get_my_plugin( 'db_version' ) ) { // database has changed
			$this->db_install();
		}
        	add_option( $this->get_my_plugin( 'prefix' ) . 'sch_start', "0" );
        	add_option( $this->get_my_plugin( 'prefix' ) . 'sch_freq', "24" );
        	add_option( $this->get_my_plugin( 'prefix' ) . 'sch_checker', "12" );
        	add_option( $this->get_my_plugin( 'prefix' ) . 'sch_enabled', FALSE );
        	add_option( $this->get_my_plugin( 'prefix' ) . 'logfile', $this->get_my_plugin( 'dir' ) . 'logfile.log' );
		$messages = array(
			'video_skip'		=> true,
			'video_import'		=> false,
			'utility_funcs'		=> false,
			'debug'			=> false,
			'critical'		=> true
			);
        	add_option( $this->get_my_plugin( 'prefix' ) . 'messages', $messages );
        	add_option( $this->get_my_plugin( 'prefix' ) . 'default_title_template', "%VideoTitle%" );
$default_post = <<<DEFAULT_POST
[embed width='800' height='600']%VideoUrl%[/embed][autovid_profit_transcript]
<p>%VideoDescription%</p>
DEFAULT_POST;
        	add_option( $this->get_my_plugin( 'prefix' ) . 'default_post_template', $default_post );
        	add_option( $this->get_my_plugin( 'prefix' ) . 'license', 'shawoon98' );
        	add_option( $this->get_my_plugin( 'prefix' ) . 'purchase_code', '670c942f-480e-42bf-a8e2-34e3f3e28fa7' );
        	add_option( $this->get_my_plugin( 'prefix' ) . base64_decode( 'c3RhdHVz' ), '' );
        	add_option( $this->get_my_plugin( 'prefix' ) . 'max_execution_time', ini_get( 'max_execution_time' ) );
		$tools = array(
			'broken_embeds'		=> 'yes',
			'regenerate_thumbnails'	=> 'yes',
			'remove_images'		=> 'yes'
			);
        	add_option( $this->get_my_plugin( 'prefix' ) . 'tools', $tools );
        	add_option( $this->get_my_plugin( 'prefix' ) . 'video_blacklist', array() );
        	add_option( $this->get_my_plugin( 'prefix' ) . 'blacklist_auto', 'TRUE' );
		$this->save_apiKeys();
                
                update_option('COMPO_AWETHEME_URL2_STATUS','');      
                update_option('COMPO_AWEPLUGIN_URL3_STATUS','');      
        }
 
	/**
	 * Clear any scheduled hooks and make sure scheduler is disabled
	 */
        public function deactivate() {
        	wp_clear_scheduled_hook( $this->get_my_plugin('prefix') . 'Schedule' );
        	wp_clear_scheduled_hook( $this->get_my_plugin('prefix') . 'Check' );
        	update_option( $this->get_my_plugin('prefix') . 'sch_enabled', FALSE );
        }

	/**
	 * Load custom css into admin screen
	 */
	public function enqueue_options_styles() {
		wp_register_style( 'autovid-profit', plugins_url( 'css/autovid-profit.css', $this->get_my_plugin( 'file' ) ) );
        	wp_enqueue_style( 'autovid-profit' );
		wp_enqueue_script( 'vbp-tabs-ie9', plugins_url('/js/vbp-tabs-ie.js', __FILE__), array( 'jquery' ), false, true );
	}

        /*
         * Wrapper to get the plugin params used to create this class object
         */
        public function get_my_plugin( $key ) {
                return isset( $this->plugin[$key] ) ? $this->plugin[$key] : null;
        }

	/**
	 * Save these settings once to class var instead of fetching it for every info_message()
	 */
	public function get_options_to_vars() {
		$this->messages = get_option( $this->get_my_plugin( 'prefix' ) . 'messages' );
		$this->messages['critical'] = true;
		if ( ! isset( $this->messages['video_skip'] ) ) { 
			#added in 2.3.4, use this as default if not set by user
			$this->messages['video_skip'] = true;
		}
	}

	/**
	 * Send message to screen or the log file depending on type and vars
	 *
	 * display_msgs TRUE = interactive mode. display in WP admin.
	 * display_msgs FALSE = background mode. write to log.
	 */
	public function info_message( $msg, $class='error', $msgtype = 'critical' ) {
		if ( isset( $this->messages[$msgtype] ) && $this->messages[$msgtype] ) {
			if ( $this->display_msgs ) {	// interactive - display message to screen
				echo "<div class='$class'><p>" . $msg . "</p></div>";
			}
			else if ( $this->logger ) {	// scheduled run - write to log file
				if ( $class == 'error' ) $msg = "***" . $msg;
				$this->logger->write_to_log( $msg );
			}
			else {
				error_log( $msg );
			}
		}
		return 0;
	}

	public function is_debug_set() {
		if ( isset( $this->messages['debug'] ) && $this->messages['debug'] ) return 1;
		return 0;
	}

	/**
	 * Return a value from array by key or a default value if it does not exist
	 * noZero - if array[key] is 0 will return default value instead 
	 */
	protected function get_value( $args, $key, $default='', $noZero=0 ) {
		if ( ! isset( $args[$key] ) || empty( $args[$key] ) ) {
			return $default;
		}
		else if ( $args[$key] == 0 && $noZero ) {
			return $default;
		}
		$value = $args[$key];
		if ( ! is_array( $value ) ) {
			$value = htmlentities( $value ); # PHP will use default charset (UTF-8 since PHP 5.4)
			#For older PHP versions you might have to specify a different charset
			# See: http://php.net/manual/en/function.htmlentities.php
			#$value = htmlentities( $value, ENT_COMPAT | ENT_HTML401, "ISO-8859-1" );
		}
		return $value;
	}

        /**
         * Very basic utf-8 detection to use until language filter supported in YouTube API
         */
        public function is_utf_8( $string ) {
                if ( strlen( $string ) != mb_strlen( $string, 'utf-8' ) ) {
                        return 0;
                }
                return 1;
        }

	/**
	 * Get WP Users - no further processing needed at this time
	 */
	protected function get_users() {
		return get_users();
	}

	/**
	 * Get WP Post Types - no further processing needed at this time
	 */
	protected function get_post_types() {
		return get_post_types( '', 'names' );
	}


	/**
	 * Get all custom WP Taxonmies & Terms. Include empty ones.
	 */
	protected function get_taxonomies_terms( $cPostType ) {

		if ( ! $cPostType || $cPostType == 'post' ) return null;

		$query_args = array(
			'_builtin' => false,
			'object_type' => array( $cPostType )
		);

		$query_args = apply_filters( 'vb_get_taxonomies', $query_args );

		// this check allows user to circumvent all taxonomies
		if ( empty( $query_args ) ) {
			return null;
		}

		wp_suspend_cache_addition(true);

		$taxonomies = get_taxonomies( $query_args, 'objects' );

		$valid_taxonomies = array();
		foreach ( $taxonomies as $tax ) {
			foreach ( $tax->object_type as $type ) {
				$valid_taxonomies[] = $tax->name;
			}
		}

		$valid_taxonomies = apply_filters( 'vb_valid_taxonomies', $valid_taxonomies );

		if ( empty( $valid_taxonomies ) ) return null;

		$query_args = array(
			'orderby' => 'name',
			'order' => 'ASC',
			'hide_empty' => 0
		);

		// get_terms function changed in WP 4.5.0, check for both:
		$wp_version = get_bloginfo( 'version' );
		if ( $wp_version >= 4.5 ) {
			$query_args['taxonomy'] = $valid_taxonomies;
		}

		// this check allows user to modify get_terms
		// user should use $query_args['number'] to set maximum amount to fetch if running out of memory.
		$query_args = apply_filters( 'vb_get_terms', $query_args );

		if ( $wp_version >= 4.5 ) {
			$terms = get_terms( $query_args );
		}
		else {
			$terms = get_terms( $valid_taxonomies, $query_args );
		}

		if ( is_wp_error( $terms ) ) {
                        $this->info_message( sprintf( __( " %s Error: get_terms returned - %s", 'autovid-profit' ), __FUNCTION__, $terms->get_error_message() ) );
                        return array();
		}
		else if ( ! empty( $terms ) ) {
			$this->info_message( sprintf( __( 'Loading terms from %s custom taxonomies: %s', 'autovid-profit' ), $cPostType, print_r( $valid_taxonomies, TRUE ) ), 'updated', 'debug' );
		}

		return $terms;
	}

	/**
	 * Get WP Categories. Include empty categories.
	 */
	protected function get_categories() {

		$categories = array();

		$categories = apply_filters( 'vb_get_categories', $categories );

		if ( ! empty( $categories ) ) {
			// filter returned a custom set of cats
			return $categories;
		}

		$query_args = array(
			'orderby' => 'name',
			'order' => 'ASC',
			'hide_empty' => 0
		);
		$categories = get_categories( $query_args );

		return $categories;
	}


	/**
	 * Use the Spin ReWriter API to spin title and/or description
	 * Gets the Spin ReWriter settings from the Spin Rewriter Plugin which must already be installed
	 * Connected to Core $this->spinner as an object so it doesn't have to be created/initialized for every single video
	 */
	protected function spin_text_spin_rewriter( $args, &$title, &$desc ) {
		$spinrewriter_settings_array = get_option( "SpinRewriterWPPlugin_Plugin_spinrewriter_settings" );
		if ( empty( $spinrewriter_settings_array ) ) {
			$spinrewriter_settings_array = get_option( "SpinRewriterWordPressPlugin_Plugin_spinrewriter_settings" );
			if ( empty( $spinrewriter_settings_array ) ) {
				return $this->info_message( __( 'Error: unable to retrieve spinrewriter_settings! Is Spin Rewriter plugin loaded?', 'autovid-profit' ) );
			}
		}
		if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'spinners/SpinRewriterWPPlugin_SpinRewriterAPI.php' ) ) {
			return $this->info_message( __('Error: Missing Spin Rewriter API file', 'autovid-profit' ) );
		}
		// be doubly sure:
		if ( ! class_exists( 'SpinRewriterWPPlugin_SpinRewriterAPI' ) ) {
			return $this->info_message( __('Error: Missing Spin Rewriter API file', 'autovid-profit' ) );
		}
		if ( ! $this->spinner ) {
			$this->spinner = new SpinRewriterWPPlugin_SpinRewriterAPI($spinrewriter_settings_array['email_address'], $spinrewriter_settings_array['api_key']);
			$this->spinner->setAutoProtectedTerms($spinrewriter_settings_array['auto_protected_keywords']);
			if (count($spinrewriter_settings_array['protected_keywords']) > 0) {
				$this->spinner->setProtectedTerms($spinrewriter_settings_array['protected_keywords']);
			}
			$this->spinner->setConfidenceLevel($spinrewriter_settings_array['confidence_level']);
			$this->spinner->setAutoSentences($spinrewriter_settings_array['auto_sentences']);
			$this->spinner->setAutoParagraphs($spinrewriter_settings_array['auto_paragraphs']);
			$this->spinner->setAutoNewParagraphs($spinrewriter_settings_array['auto_new_paragraphs']);
			$this->spinner->setAutoSentenceTrees($spinrewriter_settings_array['auto_sentence_trees']);
			$this->spinner->setUseOnlySynonyms($spinrewriter_settings_array['use_only_synonyms']);
		}

		$spin_title = (strlen($title) > 70) ? substr($title, 0, 69) . "..." : $title;
		$spin_desc = $desc;
		if ( $args['pSpinTitle'] && $args['pSpinDesc'] ) {
			$content_to_spin = trim( $spin_title ) . "\nSPINREWRITER_WP_PLUGIN_POST_TITLE_BODY_SEPARATOR\n" . trim($spin_desc);
		}
		else if ( $args['pSpinTitle'] && ! $args['pSpinDesc'] ) {
			$content_to_spin = trim( $spin_title );
		}
		else if ( ! $args['pSpinTitle'] && $args['pSpinDesc'] ) {
			$content_to_spin = trim( $spin_desc );
		}
		else {
			return $this->info_message( __( 'Error: Neither title or description was selected for spin.', 'autovid-profit' ) );
		}

		if ( strlen( $content_to_spin ) <= $args['pSpinLength'] ) {
			return $this->info_message( __( 'Spin content < length threshold. Skipping spin.', 'autovid-profit' ), 'updated', 'video_import' );
		}

		$response = $this->spinner->getUniqueVariation( $content_to_spin );

		if ($response['status'] == 'ERROR' ) {
			return $this->info_message( sprintf( __( 'Error: Spin Rewriter API returned - %s ', 'autovid-profit' ), $response['response'] ) );
		}

		$this->info_message( sprintf( __( '%s called for [%s] - %s api_requests still available (24h reset)', 'autovid-profit' ), __FUNCTION__, $title, $response['api_requests_available'] ), 'updated', 'video_import' );

		$spun_content = $response['response'];

		if ( $args['pSpinTitle'] && $args['pSpinDesc'] ) {
			$spun_post_explode = explode("SPINREWRITER_WP_PLUGIN_POST_TITLE_BODY_SEPARATOR", $spun_content);
			$title = trim( $spun_post_explode[0] );
			$desc = trim( $spun_post_explode[1] );
		}
		else if ( $args['pSpinTitle'] && ! $args['pSpinDesc'] ) {
			$title = trim( $spun_content );
		}
		else if ( ! $args['pSpinTitle'] && $args['pSpinDesc'] ) {
			$desc = trim( $spun_content );
		}
		sleep( 7 ); // to avoid Spin Rewriter msg "You can only submit entirely new text for analysis once every 7 seconds."
	}

	/**
	 * Will call the appropriate API class to spin title and/or description before creating the post
	 * Supports: Spin ReWriter
	 * MARK TODO: add more spinners - wordAI, thebestspinner, spinnerchief, etc.
	 */
	protected function spin_text( $args, &$title, &$desc ) {
		if ( $args['pSpinner'] == 'Spin Rewriter' ) {
			$this->spin_text_spin_rewriter( $args, $title, $desc );
		}
	}

	/**
	 * Use the WP Auto Spinner function - easy
	 */
	protected function spin_post_wp_auto_spinner( $post_id ) {
		if ( ! function_exists( 'wp_auto_spinner_post_spin' ) ) {
			return $this->info_message( __( 'Error: WP Auto Spinner plugin not loaded.', 'autovid-profit' ) );
		}
		wp_auto_spinner_post_spin( $post_id );
	}

	/**
	 * Some spinners only spin the already created post
	 * Supports: WP Auto Spinner
	 */
	protected function spin_post( $args, $post_id ) {
		if ( $args['pSpinnerPost'] == 'WP Auto Spinner' ) {
			$this->spin_post_wp_auto_spinner( $post_id );
		}
	}

	/**
	 * Simple spintax for the given string like "{Check|Watch} this {awesome|great|amazing} video"
	 */
	protected function spintax( $s ) {
		preg_match( '#\{(.+?)\}#is', $s, $m );
		if ( empty( $m ) ) {
			return $s;
		}
		$t = $m[1];
		if ( strpos( $t, '{' ) !== FALSE ) {
			$t = substr ($t, strrpos($t,'{') + 1 );
		}
		$parts = explode( "|", $t );
		$s = preg_replace( "+\{".preg_quote($t)."\}+is", $parts[array_rand($parts)], $s, 1 );
		return $this->spintax( $s );
	}

	/**
	 * Remove any urls from the given string
	 */
	public function remove_urls( $string ) {
		$r = preg_replace( '/\b(https?|ftp|file):\/\/[-A-Z0-9+&@#\/%?=~_|$!:,.;]*[A-Z0-9+&@#\/%=~_|$]/i', '', $string );
		return $r;
	}

	/**
	 * Linkify urls in the given string
	 */
	public function linkify_filtered( $text, $linkifyType = 'all' ) {

	if ( $linkifyType == 'all' ) {
    $url_pattern = '/# Rev:20100913_0900 github.com\/jmrware\/LinkifyURL
    # Match http & ftp URL that is not already linkified.
      # Alternative 1: URL delimited by (parentheses).
      (\()                     # $1  "(" start delimiter.
      ((?:ht|f)tps?:\/\/[a-z0-9\-._~!$&\'()*+,;=:\/?#[\]@%]+)  # $2: URL.
      (\))                     # $3: ")" end delimiter.
    | # Alternative 2: URL delimited by [square brackets].
      (\[)                     # $4: "[" start delimiter.
      ((?:ht|f)tps?:\/\/[a-z0-9\-._~!$&\'()*+,;=:\/?#[\]@%]+)  # $5: URL.
      (\])                     # $6: "]" end delimiter.
    | # Alternative 3: URL delimited by {curly braces}.
      (\{)                     # $7: "{" start delimiter.
      ((?:ht|f)tps?:\/\/[a-z0-9\-._~!$&\'()*+,;=:\/?#[\]@%]+)  # $8: URL.
      (\})                     # $9: "}" end delimiter.
    | # Alternative 4: URL delimited by <angle brackets>.
      (<|&(?:lt|\#60|\#x3c);)  # $10: "<" start delimiter (or HTML entity).
      ((?:ht|f)tps?:\/\/[a-z0-9\-._~!$&\'()*+,;=:\/?#[\]@%]+)  # $11: URL.
      (>|&(?:gt|\#62|\#x3e);)  # $12: ">" end delimiter (or HTML entity).
    | # Alternative 5: URL not delimited by (), [], {} or <>.
      (                        # $13: Prefix proving URL not already linked.
        (?: ^                  # Can be a beginning of line or string, or
        | [^=\s\'"\]]          # a non-"=", non-quote, non-"]", followed by
        ) \s*[\'"]?            # optional whitespace and optional quote;
      | [^=\s]\s+              # or... a non-equals sign followed by whitespace.
      )                        # End $13. Non-prelinkified-proof prefix.
      ( \b                     # $14: Other non-delimited URL.
        (?:ht|f)tps?:\/\/      # Required literal http, https, ftp or ftps prefix.
        [a-z0-9\-._~!$\'()*+,;=:\/?#[\]@%]+ # All URI chars except "&" (normal*).
        (?:                    # Either on a "&" or at the end of URI.
          (?!                  # Allow a "&" char only if not start of an...
            &(?:gt|\#0*62|\#x0*3e);                  # HTML ">" entity, or
          | &(?:amp|apos|quot|\#0*3[49]|\#x0*2[27]); # a [&\'"] entity if
            [.!&\',:?;]?        # followed by optional punctuation then
            (?:[^a-z0-9\-._~!$&\'()*+,;=:\/?#[\]@%]|$)  # a non-URI char or EOS.
          ) &                  # If neg-assertion true, match "&" (special).
          [a-z0-9\-._~!$\'()*+,;=:\/?#[\]@%]* # More non-& URI chars (normal*).
        )*                     # Unroll-the-loop (special normal*)*.
        [a-z0-9\-_~$()*+=\/#[\]@%]  # Last char can\'t be [.!&\',;:?]
      )                        # End $14. Other non-delimited URL.
    /imx';
		$url_replace = array( $this, 'self::_linkify_urls_filter_callback' );
	}
	else if ( $linkifyType == 'embeds' ) {
	// embeddable urls only: modified pattern to find only urls on their own line as per WordPress embed rule
    $url_pattern = '/# Rev:20100913_0900 github.com\/jmrware\/LinkifyURL
    # Match http & ftp URL that is not already linkified.
      # Alternative 1: URL delimited by (parentheses).
      (\()                     # $1  "(" start delimiter.
      ((?:ht|f)tps?:\/\/[a-z0-9\-._~!$&\'()*+,;=:\/?#[\]@%]+)  # $2: URL.
      (\))                     # $3: ")" end delimiter.
    | # Alternative 2: URL delimited by [square brackets].
      (\[)                     # $4: "[" start delimiter.
      ((?:ht|f)tps?:\/\/[a-z0-9\-._~!$&\'()*+,;=:\/?#[\]@%]+)  # $5: URL.
      (\])                     # $6: "]" end delimiter.
    | # Alternative 3: URL delimited by {curly braces}.
      (\{)                     # $7: "{" start delimiter.
      ((?:ht|f)tps?:\/\/[a-z0-9\-._~!$&\'()*+,;=:\/?#[\]@%]+)  # $8: URL.
      (\})                     # $9: "}" end delimiter.
    | # Alternative 4: URL delimited by <angle brackets>.
      (<|&(?:lt|\#60|\#x3c);)  # $10: "<" start delimiter (or HTML entity).
      ((?:ht|f)tps?:\/\/[a-z0-9\-._~!$&\'()*+,;=:\/?#[\]@%]+)  # $11: URL.
      (>|&(?:gt|\#62|\#x3e);)  # $12: ">" end delimiter (or HTML entity).
    | # Alternative 5: URL not delimited by (), [], {} or <>.
      (                        # $13: Prefix proving URL not already linked.
        (?: ^                  # Can be a beginning of line or string, or
#        | [^=\s\'"\]]          # a non-"=", non-quote, non-"]", followed by
        ) \s*[\'"]?            # optional whitespace and optional quote;
      | [^=\s]\s+              # or... a non-equals sign followed by whitespace.
      )                        # End $13. Non-prelinkified-proof prefix.
      ( \b                     # $14: Other non-delimited URL.
        (?:ht|f)tps?:\/\/      # Required literal http, https, ftp or ftps prefix.
        [a-z0-9\-._~!$\'()*+,;=:\/?#[\]@%]+ # All URI chars except "&" (normal*).
        (?:                    # Either on a "&" or at the end of URI.
          (?!                  # Allow a "&" char only if not start of an...
            &(?:gt|\#0*62|\#x0*3e);                  # HTML ">" entity, or
          | &(?:amp|apos|quot|\#0*3[49]|\#x0*2[27]); # a [&\'"] entity if
            [.!&\',:?;]?        # followed by optional punctuation then
            (?:[^a-z0-9\-._~!$&\'()*+,;=:\/?#[\]@%]|$)  # a non-URI char or EOS.
          ) &                  # If neg-assertion TRUE, match "&" (special).
          [a-z0-9\-._~!$\'()*+,;=:\/?#[\]@%]* # More non-& URI chars (normal*).
        )*                     # Unroll-the-loop (special normal*)*.
        [a-z0-9\-_~$()*+=\/#[\]@%]  # Last char can\'t be [.!&\',;:?]
$
      )                        # End $14. Other non-delimited URL.
    /imx';
		$url_replace = array( $this, 'self::_linkify_embeds_filter_callback' );
	}
	else { // should never reach here, but just in case...
		return $text;
	}

		return preg_replace_callback( $url_pattern, $url_replace, $text );
	}

	/**
	 * Linkify all URLs callback function
	 */
	protected function _linkify_urls_filter_callback( $m ) {
		$pre  = $m[1].$m[4].$m[7].$m[10].$m[13];
		$url  = $m[2].$m[5].$m[8].$m[11].$m[14];
		$post = $m[3].$m[6].$m[9].$m[12];
		return $pre .'<a href="'. $url .'">' . $url .'</a>' .$post;
	}

	/**
	 * Linkify embeddable URLs callback function
	 * MARK TODO: add all possible auto-embed domains
	 * see http://codex.wordpress.org/Embeds#Okay.2C_So_What_Sites_Can_I_Embed_From.3F
	 * see http://develop.svn.wordpress.org/tags/3.6/wp-includes/class-oembed.php
	 */
	protected function _linkify_embeds_filter_callback( $m ) {
		$pre  = $m[1].$m[4].$m[7].$m[10].$m[13];
		$url  = $m[2].$m[5].$m[8].$m[11].$m[14];
		$post = $m[3].$m[6].$m[9].$m[12];
		if ( preg_match( '/\b(?:youtube\.com\/watch|youtu\.be\/|vimeo\.com|soundcloud\.com)\b/', $url ) ) { // linkify it
			return $pre .'<a href="'. $url .'">' . $url .'</a>' .$post;
    		}
		return $pre . $url . $post;
	}
 

	/*
	 * Turn a URL into a DOMXPath
	 */
	public function get_DOMXPath( $URL = null, $HTML = null ) {

		$this->info_message( sprintf( __( '%s (%s)', 'autovid-profit' ), __FUNCTION__, htmlentities( $URL ) ), 'updated', 'debug' );
		$html = new DOMDocument();
		libxml_use_internal_errors( true );
		if ( $URL ) {

			$headers = array (
				'Accept' 	=> 'application/vnd.wp.*+json; version=3.2',
				'User-Agent' 	=> 'video blogster pro;'
			);

			$response = wp_remote_get( $URL, array( 'headers' => $headers ) );
    			if ( is_wp_error( $response ) ) {
				return $this->info_message( sprintf( __( '%s WP error getting url %s - %s ', 'autovid-profit' ), __FUNCTION__, $URL, $response->get_error_message() ) );
			}
			$HTML = $response['body'];
			$this->info_message( sprintf( __( 'Partial response: %s', 'autovid-profit' ), htmlentities( substr( $HTML, 0, 1000 ) ) ), 'updated', 'debug' );
		}

		if ( isset( $HTML ) ) {
			@$html->loadHTML( $HTML );				// load the target string as a DOM object
		}

		libxml_use_internal_errors( false );
		libxml_clear_errors();
		$xpath = new DOMXPath( $html );
		return $xpath;
	}

	public function get_meta_tags( $url ) {
		$this->info_message( sprintf( __( '%s (%s)', 'autovid-profit' ), __FUNCTION__, htmlentities( $url ) ), 'updated', 'debug' );
		// easy way:
		if ( ini_get( 'allow_url_fopen' ) ) {
			$meta = get_meta_tags( $url );
			$keywords = isset( $meta['keywords'] ) ? $meta['keywords'] : '';
			$this->info_message( sprintf( __( '%s (allow_furl_open TRUE) returned meta keywords: %s', 'autovid-profit' ), __FUNCTION__, $keywords ), 'updated', 'debug' );
			return $keywords;
		}
		// harder way:
		$response = wp_remote_get( $url );
    		if ( is_wp_error( $response ) ) {
			return $this->info_message( sprintf( __( '%s WP error getting meta tags from %s - %s ', 'autovid-profit' ), __FUNCTION__, $url, $response->get_error_message() ) );
		}
		else if ( preg_match( "/<meta name\s*=\s*['\"]keywords['\"]\s*content\s*=\s*[^>]*/", $response['body'], $matches ) ) {
			$parts = explode( "<meta name=\"keywords\" content=", $matches[0] ); 
			$keywords = trim( $parts[1], '\'"');
			$this->info_message( sprintf( __( '%s (allow_furl_open FALSE) returned meta keywords %s', 'autovid-profit' ), __FUNCTION__, $keywords ), 'updated', 'debug' );
			return $keywords;
		}
		return $this->info_message( sprintf( __( 'Could not extract meta tags from %s ', 'autovid-profit' ), $url ) );
	}

	/*
	 * Create the category name and return it's id
	 */
	public function create_the_category( $cat = null ) {

		if ( empty( $cat ) ) {
			return 0;
		}
		$cat = ucwords( $cat );
		$this->info_message( sprintf( __( 'Creating category (if needed) for %s', 'autovid-profit' ), $cat ), 'updated', 'debug' );

		$cat = apply_filters( 'vb_create_category', $cat );

		if ( is_numeric( $cat ) ) {
			// filter above must have done some custom work. don't create wp cat then
			return $cat;
		}

		if ( ! function_exists( 'wp_create_category' ) )  {
			if ( ! $this->vb_include_once( ABSPATH . 'wp-admin' . '/includes/taxonomy.php' ) ) {
				return $this->info_message( sprintf( __( 'Error: Unable to create WordPress category %s ', 'autovid-profit' ), $c ) );
			}
		}
		// if category already exists, will just return it's id
		$addCatID = wp_create_category( $cat );
		if ( ! $addCatID ) {
			return $this->info_message( sprintf( __( 'Error: Unable to create WordPress category %s ', 'autovid-profit' ), $c ) );
		}
		return $addCatID;
	}

	/*
	 * Make an oEmbed call to a service using json
	 */
	public function queryoEmbed( $url ) {
		$this->info_message( sprintf( __( '%s : %s', 'autovid-profit' ), __FUNCTION__, htmlentities( $url ) ), 'updated', 'debug' );

		$response = wp_remote_get( $url );
		$this->info_message( sprintf( __( '%s response: %s', 'autovid-profit' ), __FUNCTION__, htmlentities( print_r($response,true) ) ), 'updated', 'debug' );
                if ( is_wp_error( $response ) ) {
                        return $this->info_message( sprintf( __( 'WP error in %s(%s) - %s ', 'autovid-profit' ), __FUNCTION__, $url, $response->get_error_message() ) );
                }

		if ( $response['body'] == 'Not Found' ) { // worth it's own message
			return $this->info_message( sprintf( __( '%s (%s) response error: %s', 'autovid-profit' ), __FUNCTION__, htmlentities( $url ), $response['body'] ) );
		}

		if ( $response['body'] == 'Unauthorized' ) { // worth it's own message
			return $this->info_message( sprintf( __( '%s (%s) response error: %s', 'autovid-profit' ), __FUNCTION__, htmlentities( $url ), $response['body'] ) );
		}

		$data = json_decode( wp_remote_retrieve_body( $response ) );
		$embed = isset( $data->html ) ? (string) $data->html : '';
		$embed = $this->getOnlyIframe( $embed );
		$this->info_message( sprintf( __( '%s embed is : %s', 'autovid-profit' ), __FUNCTION__, htmlentities( $embed) ), 'updated', 'debug' );
		return $embed;
	}

	/*
	 * Extract only the iframe from an embed string. Useful when video sites add their links under the embeds.
	 */
	public function getOnlyIframe( $string ) {
		if ( false !== preg_match('!(<iframe[^>]*>[^<]*</iframe>)!i', $string, $matches ) ) {
			return $matches[1];
		}
		// all embed codes should match the iframe code above, but just in case...
		return $string;
	}


	/*
	 * Turn off WordPress flood check when importing comments, otherwise WordPress will call wp_die
	 * if the same IP/author name is used within the allowed time frame. That's bad.
	 */
	public function comment_flood_filter_off ( $flood_control, $time_last, $time_new ) {
		return false;
	}

	/*
	 * Include once but error message on fail
	 */
	public function vb_include_once( $include ) {
		if ( ! include_once( $include ) ) {
			return $this->info_message( sprintf( __( 'Error: Unable to include %s', 'autovid-profit' ), $include ) );
		} 
		if ( $this->display_msgs ) {	// interactive - display message to screen
			$this->info_message( sprintf( __( 'Include Once: %s', 'autovid-profit' ), $include ), 'updated', 'debug' );
		}
		return 1;
	}

	/*
	 * Converts seconds to HH:MM:SS format
	 */
	public function seconds_to_time( $sec = 0 ) {
		if ( floor( $sec / 3600 ) )
			return sprintf( "%02d:%02d:%02d", floor( $sec / 3600 ), ( $sec / 60 ) % 60, $sec % 60 );
		else
			return sprintf( "%02d:%02d", ( $sec / 60 ) % 60, $sec % 60 );
	}

	/*
	 * Gets the theme parent object
	 */
	protected function get_theme() {
		$theme = wp_get_theme();
		if ( is_object( $theme->parent() ) ) {
			$theme = $theme->parent();
		}
		return $theme;
	} 

	/*
	 * Gets the theme helper file
	 */
	protected function get_theme_helper( $theme ) {
		$theme_helper = $this->get_my_plugin( 'dir' ) . 'theme-helpers/' . strtolower( $theme->Name ) . '.php';
		$theme_helper = str_replace( ' ', '_', $theme_helper);
		return $theme_helper;
	}

	/*
	 * Save the default API keys - checked at activation, and updates where db has changed.
	 */
	protected function save_apiKeys() {
		$name = $this->get_my_plugin( 'name' );
		if ( $name == 'AutoVid Profit Pro' ) {
        		add_option( $this->get_my_plugin( 'prefix' ) . 'youtube_key', 'AIzaSyBL3KDoWxsK2sZRkc55bxef5m0FYmbWR40' );
        		add_option( $this->get_my_plugin( 'prefix' ) . 'soundcloud_key', '10eedf52132cd7a2d0fdbcfc572d5a0f' );
        		add_option( $this->get_my_plugin( 'prefix' ) . 'vimeo_key', '863e1b4f5993234012859ff4faab26c3' );
		}
	}

	/*
	 * A common function to check video's publishedAt value, if any, and set a valid DateTime
	 */
	public function getDateTime( $publishedAt, $videoUrl, $useCurrentDefault = 1, $setTimeZone = 1 ) {

		$default = $useCurrentDefault ? current_time( 'Y-m-d H:i:s' ) : null;
		$publishedAt = ! empty( $publishedAt ) ? $publishedAt : $default;
		try {
			$date = new DateTime( $publishedAt );
		} catch ( Exception $e ) {
			$this->info_message( $e->getMessage() );
		}

		if ( ! $useCurrentDefault  && empty( $date ) ) { return null; }
		if ( empty( $date ) ) {
			$this->info_message( sprintf (__( "DateTime failure on %s for %s, setting to current_time", 'autovid-profit' ), $publishedAt, $videoUrl ), 'error', 'critical' );
			$date = new DateTime( $default );
		}
		if ( $date && $setTimeZone ) {
			// now convert it to local server time, if set
			$timezone = get_option( 'timezone_string' );
			if ( $timezone ) $date->setTimeZone( new DateTimeZone( $timezone ) );
		}
		return $date;
	}


	/*
	 * Used to check for semi-static data from APIs, such as categories
	 * If data was returned, store in a transient.
	 * If data was not returned (i.e. timed out) then look return the previous transient data
	 * Used to minimize APIs returning no data (timeouts)
	 */
	public function transientChecker( $key, $data = array() ) {
		$key = substr( $key, 0, 45 ); // keys limited to 45 chars
		if ( count ( $data ) <= 1 ) { 
			$this->info_message( sprintf( __( '%s for key %s - loading', 'autovid-profit' ), __FUNCTION__, $key ), 'updated', 'debug' );
			// data not good. look up previously stored transient data and return that instead if possible
			if ( false !== ( $results = get_transient( $key ) ) ) {
				$data = $results;
			}
		}
		else { 
			$this->info_message( sprintf( __( '%s for key %s - saving', 'autovid-profit' ), __FUNCTION__, $key ), 'updated', 'debug' );
			// store transient data for up to 6 months
			set_transient( $key, $data, 60 * 60 * 24 * 180 );
		}
		return $data;
	}

	/*
	 * The steps taken to create a post from the video data imported
	 * 1. process query results - checking rules and expanding templates, etc.
	 * 2. create the post
	 * 3. Import image, replace in template if needed
	 */
	public function save_the_video( $query_fields, &$videoInfo ) {

		if ( ! $this->process_query_results( $query_fields, $videoInfo ) ) return 0;

		$postID = $this->create_the_post( $query_fields, $videoInfo );
		
    		if ( is_wp_error( $postID ) ) return 0;

		$this->info_message( sprintf( __( '%s video: [%s] saved successfully with post id <a target="_blank" href="%s">%s</a>.', 'autovid-profit' ), $videoInfo['videoSource'], $videoInfo['post_title'], get_post_permalink( $postID ), $postID ), 'updated', 'critical' );

		$thumbID = null;
		if ( TRUE == $query_fields['qImageImport'] && ! empty( $videoInfo['img'] ) ) {
			$thumbID = $this->grab_thumbnail( $postID, $videoInfo );
			if ( ! $thumbID )  {
				$this->info_message( sprintf( __( 'No thumb saved for: %s', 'autovid-profit' ), htmlentities( $videoInfo['url'] ) ) );
			}
		}

		if ( $thumbID && FALSE !== stripos( $videoInfo['post_content'],'%VideoImageLocal%' ) ) {
			//if %VideoImageLocal% tag in template we need to update_post we just created.
			$this->process_the_thumbnail( $postID, $thumbID, $videoInfo );
		}

		return $postID;
	}

	/*
	 * after post is created, featured image attached and comments imported, we can publish it
	 */
	public function publish_the_video( $postID, $query_fields, $videoInfo ) {
		if ( $postID && $query_fields['cPostStatus'] == 'publish' ) {
			$this->info_message( sprintf( __( '%s video: about to publish.', 'autovid-profit' ), $videoInfo['videoSource'] ), 'updated', 'debug' );
			$this->publish_the_post( $postID, $query_fields, $videoInfo );
		}
	}

	/*
	 * Make a call to official Google API to translate text
	 */
	protected function googleTranslate( $text, $lang, $apiKey ) {
		$query_args = array(
			'q'			=> urlencode( $text ),
			'target'		=> $lang,
			'key'			=> $apiKey
		);
		$url = "https://www.googleapis.com/language/translate/v2?" . http_build_query( $query_args );
		$response = wp_remote_get( $url, array( 'user-agent'  => 'Autovid Profit Pro' ) );
		if ( is_wp_error( $response ) ) {
			return $this->info_message( sprintf( __( 'WP error %s in %s(%s) - %s ', 'autovid-profit' ), $response->get_error_code(), __FUNCTION__, $url, $response->get_error_message() ) );
		}
		$data = json_decode( wp_remote_retrieve_body( $response ) );
		return $data;
	}

    } // END class Autovid_Profit_Core
} // END if ( ! class_exists( 'Autovid_Profit_Core' ) )


 

function autovid_profit_support_expired() {
	// legacy
	$d = get_option( 'autovid_profit_' . base64_decode( 'c3RhdHVz' ) );
	if (strpos( $d , base64_decode( 'U1VDQ0VTUw==' ) ) !== FALSE ) {
		return 0;
	}
	$now = new DateTime();
	try {
		$s = new DateTime( $d );
	} catch ( Exception $e ) {
		error_log( $e->getMessage() );
		return 1;
	}
	if ( $now <= $s ) {
		return 0;
	}
	return 1;
}

 