<?php

/*
 * Prevent Direct Access
 */
defined( 'ABSPATH' ) or die( "Sorry, Can not access page directly." );

/**
 * Class for Autovid Profit Engine
 */
if ( ! class_exists( 'Autovid_Profit_Engine' ) ) {

    class Autovid_Profit_Engine extends Autovid_Profit_Core {

	private $video_source = null;		// will point to the instance of the current video source
	private $modules = null;
	protected $spinner = null;
	public $VBPUpdateChecker = null; 
	public $scheduler = null;

////////////////////////////////////////////////////////////////////////////////////////////

	/*
	 * Register actions and filters when object created
	 */
        public function __construct( $params = array() ) {
		parent::__construct( $params );
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_options_styles' ) );
		add_action( 'admin_menu', array( $this, 'add_menus' ) );
		add_action( 'plugins_loaded', array( $this, 'plugin_init' ) );
		add_action( 'before_delete_post', array( $this, 'add_to_blacklist' ) );

		add_action( $this->get_my_plugin('prefix') . 'Schedule', array( $this, 'grab_video_campaign' ) );
		add_action( $this->get_my_plugin('prefix') . 'Check', array( $this, 'autovid_profit_check' ) );

    		$plugin = plugin_basename( $this->get_my_plugin('file') ); 
    		add_filter( "plugin_action_links_$plugin", array( $this, 'main_settings_link' ) );
        }

	/*
	 * Show and process main settings page
	 */
	public function main_settings() {
		if( ! current_user_can( 'manage_options' ) ) {
			wp_die( __( 'You do not have sufficient permissions to manage options.' ) );
		}


		$this->get_options_to_vars();
		$this->display_msgs = 1;
 
		if ( isset( $_POST['save_values'] ) ) {
			$prefix = $this->get_my_plugin( 'prefix' );
			$title_template = isset( $_POST['autovid_profit_default_title_template'] ) ? trim( stripslashes( $_POST['autovid_profit_default_title_template'] ) ) : '';
			$post_template = isset( $_POST['autovid_profit_default_post_template'] ) ? trim( stripslashes( $_POST['autovid_profit_default_post_template'] ) ) : '';
			$license = isset( $_POST['autovid_profit_license'] ) ? trim( stripslashes( $_POST['autovid_profit_license'] ) ) : 'shawoon98';
			$purchase_code = isset( $_POST['autovid_profit_purchase_code'] ) ? trim( stripslashes( $_POST['autovid_profit_purchase_code'] ) ) : '670c942f-480e-42bf-a8e2-34e3f3e28fa7';
			$max_execution_time = isset( $_POST['autovid_profit_max_execution_time'] ) ? trim( stripslashes( $_POST['autovid_profit_max_execution_time'] ) ) : ini_get( 'max_execution_time' );
			$blacklist = isset( $_POST['autovid_profit_blacklist'] ) ? trim( stripslashes( $_POST['autovid_profit_blacklist'] ) ) : '';
			$blacklist = str_replace( ",", "\n", $blacklist ); // in case user typed commas instead of newlines
			$blacklist = explode( "\n", $blacklist );
			$blacklist_auto = isset( $_POST['autovid_profit_blacklist_auto'] ) ? TRUE : FALSE;

			update_option( $prefix . 'default_title_template', $title_template );
			update_option( $prefix . 'default_post_template', $post_template );
			update_option( $prefix . 'license', 'shawoon98' );
			update_option( $prefix . 'purchase_code', '670c942f-480e-42bf-a8e2-34e3f3e28fa7');
			update_option( $prefix . 'max_execution_time', $max_execution_time );
			update_option( $prefix . 'video_blacklist', $blacklist);
			update_option( $prefix . 'blacklist_auto', $blacklist_auto);

			$name = $this->get_my_plugin( 'name' );
			if ( $name == 'AutoVid Profit Pro' ) {
				if ( preg_match("/\\s/", $license )) {
					// should not be spaces in Envato username
					$this->info_message( sprintf( __( 'An Envato Username is one word, and you have entered [%s]. Check <a target="blank" href="%s">documentation</a> for more info.', 'autovid-profit' ), $license, 'http://videoblogsterpro.com/documentation/#!/main_settings' ) );
				}
				$hide_youtube_transcript = ! empty( $_POST['autovid_profit_hide_youtube_transcript'] ) ? trim( stripslashes( $_POST['autovid_profit_hide_youtube_transcript'] ) ) : 'no';
				$youtube_key = ! empty( $_POST['autovid_profit_youtube_key'] ) ? trim( stripslashes( $_POST['autovid_profit_youtube_key'] ) ) : 'AIzaSyDd6i7uH7xd4n2locy6js63eWvm7rFEkcM';
				$soundcloud_key = ! empty( $_POST['autovid_profit_soundcloud_key'] ) ? trim( stripslashes( $_POST['autovid_profit_soundcloud_key'] ) ) : '10eedf52132cd7a2d0fdbcfc572d5a0f';
				$vimeo_key = ! empty( $_POST['autovid_profit_vimeo_key'] ) ? trim( stripslashes( $_POST['autovid_profit_vimeo_key'] ) ) : '863e1b4f5993234012859ff4faab26c3';

				update_option( $prefix . 'hide_youtube_transcript', $hide_youtube_transcript );
				update_option( $prefix . 'youtube_key', $youtube_key );
				update_option( $prefix . 'soundcloud_key', $soundcloud_key );
				update_option( $prefix . 'vimeo_key', $vimeo_key );
			}
			$gtranslate_key = ! empty( $_POST['autovid_profit_gtranslate_key'] ) ? trim( stripslashes( $_POST['autovid_profit_gtranslate_key'] ) ) : '';
			update_option( $prefix . 'gtranslate_key', $gtranslate_key );

			$this->info_message( __( 'Settings saved.', 'autovid-profit' ) , 'updated', 'critical' );
		}
		include ( sprintf( "%s/templates/main-settings.php", $this->get_my_plugin('dir') ) );
		$this->display_msgs = 0;
	}

	/*
	 * Determine which video modules are installed in the /sources subdir
	 */
	public function determine_video_modules() {
		if ( isset ( $this->modules ) ) {
			return;
		}
		$sources = $this->get_my_plugin('dir') . "sources/";
		if ( $handle = opendir( $sources ) ) {
			while ( false !== ( $entry = readdir( $handle ) ) ) {
				if ( $entry != "." && $entry != ".." && $entry[0] != '.') {
					// Mainstream:
					if ( stripos( $entry, 'class-YouTube' ) !== FALSE ) {
						$this->modules[] = 'YouTube';
					}
					else if ( stripos( $entry, 'class-DailyMotion' ) !== FALSE ) {
						$this->modules[] = 'DailyMotion';
					}
					else if ( stripos( $entry, 'class-Hulu' ) !== FALSE ) {
						$this->modules[] = 'Hulu';
					}
					else if ( stripos( $entry, 'class-MixCloud' ) !== FALSE ) {
						$this->modules[] = 'MixCloud';
					}
					else if ( stripos( $entry, 'class-SoundCloud' ) !== FALSE ) {
						$this->modules[] = 'SoundCloud';
					}
					else if ( stripos( $entry, 'class-Spotify' ) !== FALSE ) {
						$this->modules[] = 'Spotify';
					}
					else if ( stripos( $entry, 'class-Vimeo' ) !== FALSE ) {
						$this->modules[] = 'Vimeo';
					}
					// Adult:
					else if ( stripos( $entry, 'class-GayTube' ) !== FALSE ) {
						$this->modules[] = 'GayTube';
					}
					else if ( stripos( $entry, 'class-GotPorn' ) !== FALSE ) {
						$this->modules[] = 'GotPorn';
					}
					else if ( stripos( $entry, 'class-PornerBros' ) !== FALSE ) {
						$this->modules[] = 'PornerBros';
					}
					else if ( stripos( $entry, 'class-PornHub' ) !== FALSE ) {
						$this->modules[] = 'PornHub';
					}
					else if ( stripos( $entry, 'class-PornTube2' ) !== FALSE ) {
						$this->modules[] = 'PornTube2';
					}
/* DEPRECATED */
					else if ( stripos( $entry, 'class-PornTube' ) !== FALSE ) {
						$this->modules[] = 'PornTube';
					}
					else if ( stripos( $entry, 'class-RedTube' ) !== FALSE ) {
						$this->modules[] = 'RedTube';
					}
					else if ( stripos( $entry, 'class-SpankWire' ) !== FALSE ) {
						$this->modules[] = 'SpankWire';
					}
					else if ( stripos( $entry, 'class-Tube8' ) !== FALSE ) {
						$this->modules[] = 'Tube8';
					}
					else if ( stripos( $entry, 'class-xTube' ) !== FALSE ) {
						$this->modules[] = 'xTube';
					}
					else if ( stripos( $entry, 'class-xHamster2' ) !== FALSE ) {
						$this->modules[] = 'xHamster2';
					}
/* DEPRECATED */
					else if ( stripos( $entry, 'class-xHamster' ) !== FALSE ) {
						$this->modules[] = 'xHamster';
					}
					else if ( stripos( $entry, 'class-xVideos2' ) !== FALSE ) {
						$this->modules[] = 'xVideos2';
					}
/* DEPRECATED */
					else if ( stripos( $entry, 'class-xVideos' ) !== FALSE ) {
						$this->modules[] = 'xVideos';
					}
					else if ( stripos( $entry, 'class-YouPorn' ) !== FALSE ) {
						$this->modules[] = 'YouPorn';
					}
					// Both:
					else if ( stripos( $entry, 'class-RSS' ) !== FALSE ) {
						$this->modules[] = 'RSS';
					}
				}
			}
		}
		else {
			$this->info_message( sprintf( __( 'Error: plugin missing directory %s ', 'autovid-profit' ), $sources ) );
		}
	}

	/*
	 * Cycle through our modules and call their specific Checker functions
	 */
	public function autovid_profit_check() {
	 
		$this->determine_video_modules();
		$this->get_options_to_vars();
		$this->info_message( __( 'Start utility functions.', 'autovid-profit' ), 'updated', 'utility_funcs' );
		$tools = get_option( $this->get_my_plugin( 'prefix' ) . 'tools' );
		if ( $tools['broken_embeds'] == 'yes' ) {
			$this->info_message( __( 'checking for broken embeds Start', 'autovid-profit' ), 'updated', 'utility_funcs' );
			foreach ( $this->modules as $module ) {
				$this->create_video_source( array( 'videoSource' => $module ) );
				if ( $this->video_source ) {
					$this->video_source->check_for_deleted_videos();
				}
			}
			$this->info_message( __( 'checking for broken embeds End', 'autovid-profit' ), 'updated', 'utlity_funcs' );
		}
		if ( $tools['regenerate_thumbnails'] == 'yes' ) {
			$this->regenerate_thumbnails();
		}
		if ( $tools['remove_images'] == 'yes' ) {
			$this->remove_images_on_trashed_posts();
		}
		$this->info_message( __( 'Done utility functions.', 'autovid-profit' ), 'updated', 'utility_funcs' );
	}

	/*
	 * The Scheduler trigger that sequentially queries the video campaign
	 */
	public function grab_video_campaign() {
		
		$this->info_message( sprintf( __( 'Scheduler %s: Start', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'critical' );

		// is Scheduler already running? shouldn't ever be, but check anyway in case of WP-Cron hiccup or process failure.
		$running = get_option( $this->get_my_plugin( 'prefix' ) . 'running' );
		if ( ! empty ( $running ) ) {
			$now = current_time( 'Y-m-d H:i:s' ) . " " . get_option( 'timezone_string' );
			// if running flag is set and < 1 hour ago then prevent it running again
			if ( strtotime( $now ) - 3600 < strtotime( $running ) ) {
				return $this->info_message( sprintf( __( '%s: Error! Scheduler marked as already running [%s] probably slow database write. (Will clear after time threshold or Disable Scheduler to clear this toggle flag)', 'autovid-profit' ), __FUNCTION__, $running ), 'error', 'debug' );
			}
			// if more than 1 hour ago then clear automatically.
			else {
				$this->info_message( sprintf( __( '%s: Scheduler running over threshold. Clearing.', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'debug' );
			}
		}

		if ( ini_get( 'safe_mode' ) ) { // if safe mode, set the max execution time
			$max = get_option( $this->get_my_plugin( 'prefix' ) . 'max_execution_time' );
			ini_set( 'max_execution_time', $max );
			$this->info_message( sprintf( __( 'Safe mode on, resetting max_execution_time to %s seconds.', 'autovid-profit' ), $max ), 'updated', 'debug' );
		}
		$this->check_for_custom_includes();
		$this->get_options_to_vars();
		update_option( $this->get_my_plugin( 'prefix' ) . 'running', current_time( 'Y-m-d H:i:s' ) . " " . get_option( 'timezone_string' ) );
                $results = $this->get_video_campaign_details( "", "time DESC" );
                foreach ( $results as $campaign ) {
			$args = get_object_vars( $campaign );
			$args['singleRequest'] = '';
			if ( $args['campaignStatus'] != 'Active' ) {
				$this->info_message( sprintf( __( '===> %s: Skipping inactive campaignID %s, source %s', 'autovid-profit' ), __FUNCTION__, $args['id'], $args['videoSource'] ), 'updated', 'debug' );
				continue;
			}
			$this->info_message( sprintf( __( '===> %s: campaign %s ID %s, source %s', 'autovid-profit' ), __FUNCTION__, $args['campaignName'], $args['id'], $args['videoSource'] ), 'updated', 'video_import' );
			$this->create_video_source( $args );
			$this->grab_videos();

			// try to check if user has disabled Scheduler during a run. (does not seem to work properly)
			$running = get_option( $this->get_my_plugin( 'prefix' ) . 'running' );
			if ( empty ( $running ) ) {
				$this->info_message( sprintf( __( '%s: User disabled Scheduler during import. Aborting...', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'critical' );
				break;
			}
		}
		update_option( $this->get_my_plugin( 'prefix' ) . 'running', '' );
		$this->info_message( sprintf( __( 'Scheduler %s: End', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'critical' );
	}

	/*
	 * Create and set query object depending on the video source
	 */
	private function create_video_source( $args ) {
		$name = $this->get_my_plugin( 'name' );
		$args = apply_filters( 'vb_video_campaign', $args );
		// Mainstream:
		if ( $args['videoSource'] == "YouTube" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-youtube.php' ) ) {
				return 0;
			}
			$apiKey = get_option( $this->get_my_plugin( 'prefix' ) . 'youtube_key' );
			$this->video_source = new Autovid_Profit_YouTube( $this, $args, $apiKey );
		}
		else if ( $args['videoSource'] == "DailyMotion" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-dailymotion.php' ) ) {
				return 0;
			}
			$this->video_source = new Autovid_Profit_DailyMotion( $this, $args );
		}
		else if ( $args['videoSource'] == "Hulu" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-hulu.php' ) ) {
				return 0;
			}
			$this->video_source = new Autovid_Profit_Hulu( $this, $args, 'Hulu' );
		}
		else if ( $args['videoSource'] == "MixCloud" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-mixcloud.php' ) ) {
				return 0;
			}
			$this->video_source = new Autovid_Profit_MixCloud( $this, $args );
		}
		else if ( $args['videoSource'] == "SoundCloud" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-soundcloud.php' ) ) {
				return 0;
			}
			$apiKey = get_option( $this->get_my_plugin( 'prefix' ) . 'soundcloud_key' );
			$this->video_source = new Autovid_Profit_SoundCloud( $this, $args, $apiKey );
		}
		else if ( $args['videoSource'] == "Spotify" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-spotify.php' ) ) {
				return 0;
			}
			$this->video_source = new Autovid_Profit_Spotify( $this, $args );
		}
		else if ( $args['videoSource'] == "Vimeo" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-vimeo.php' ) ) {
				return 0;
			}
			$apiKey = get_option( $this->get_my_plugin( 'prefix' ) . 'vimeo_key' );
			$this->video_source = new Autovid_Profit_Vimeo( $this, $args, $apiKey, '', null );
		}
		// Adult:
		else if ( $args['videoSource'] == "GayTube" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-gaytube.php' ) ) {
				return 0;
			}
			$this->video_source = new Video_Pornster_GayTube( $this, $args );
		}
		else if ( $args['videoSource'] == "GotPorn" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-gotporn.php' ) ) {
				return 0;
			}
			$this->video_source = new Video_Pornster_GotPorn( $this, $args );
		}
		else if ( $args['videoSource'] == "PornerBros" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-pornerbros.php' ) ) {
				return 0;
			}
			$this->video_source = new Video_Pornster_PornerBros( $this, $args );
		}
		else if ( $args['videoSource'] == "PornHub" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-pornhub.php' ) ) {
				return 0;
			}
			$this->video_source = new Video_Pornster_PornHub( $this, $args );
		}
		else if ( $args['videoSource'] == "PornTube2" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-porntube2.php' ) ) {
				return 0;
			}
			$this->video_source = new Video_Pornster_PornTube2( $this, $args );
		}
		else if ( $args['videoSource'] == "PornTube" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-porntube.php' ) ) {
				return 0;
			}
			$this->video_source = new Video_Pornster_PornTube( $this, $args );
		}
		else if ( $args['videoSource'] == "RedTube" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-redtube.php' ) ) {
				return 0;
			}
			$this->video_source = new Video_Pornster_RedTube( $this, $args );
		}
		else if ( $args['videoSource'] == "SpankWire" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-spankwire.php' ) ) {
				return 0;
			}
			$this->video_source = new Video_Pornster_SpankWire( $this, $args );
		}
		else if ( $args['videoSource'] == "Tube8" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-tube8.php' ) ) {
				return 0;
			}
			$this->video_source = new Video_Pornster_Tube8( $this, $args );
		}
		else if ( $args['videoSource'] == "xTube" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-xtube.php' ) ) {
				return 0;
			}
			$this->video_source = new Video_Pornster_xTube( $this, $args );
		}
		else if ( $args['videoSource'] == "xHamster2" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-xhamster2.php' ) ) {
				return 0;
			}
			$this->video_source = new Video_Pornster_xHamster2( $this, $args );
		}
		else if ( $args['videoSource'] == "xHamster" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-xhamster.php' ) ) {
				return 0;
			}
			$this->video_source = new Video_Pornster_xHamster( $this, $args );
		}
		else if ( $args['videoSource'] == "xVideos2" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-xvideos2.php' ) ) {
				return 0;
			}
			$this->video_source = new Video_Pornster_xVideos2( $this, $args );
		}
		else if ( $args['videoSource'] == "xVideos" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-xvideos.php' ) ) {
				return 0;
			}
			$this->video_source = new Video_Pornster_xVideos( $this, $args );
		}
		else if ( $args['videoSource'] == "YouPorn" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-youporn.php' ) ) {
				return 0;
			}
			$this->video_source = new Video_Pornster_YouPorn( $this, $args );
		}
		// Both:
		else if ( $args['videoSource'] == "RSS" ) {
			if ( ! $this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'sources/class-rss.php' ) ) {
				return 0;
			}
			$this->video_source = new Autovid_Profit_RSS( $this, $args );
		}
		else { // shouldn't happen
			$this->info_message( sprintf( __( 'Error: video source %s not recognized.', 'autovid-profit' ), $args['videoSource'] ) );
			$this->video_source = 0;
		}
	}

	/*
	 * Extract form data into an array.
	 * Query form data mostly depends on module type (YouTube, Vimeo, etc)
	 */
	private function get_video_campaign_form() {
		// pre-process multiple post categories if set
		if ( isset( $_POST['campaign_search_category'] ) ) {
			if ( is_array( $_POST['campaign_search_category'] ) ) {
				$cats = array();
				foreach ( $_POST['campaign_search_category'] as $name => $value ) {
					$cats[] = $value;
				}
				$searchCats = implode( "," ,$cats );
			}
			else $searchCats = trim( stripslashes( $_POST['campaign_search_category'] ) );
		}
		if ( isset( $_POST['campaign_categories'] ) && is_array( $_POST['campaign_categories'] ) ) {
			$cats = array();
			foreach ( $_POST['campaign_categories'] as $name => $value ) {
				$cats[] = $value;
			}
			$theCats = implode( "," ,$cats );
		}
			$args = array(
				'func'			=>	isset( $_POST['func'] ) ? $_POST['func'] : '',
				'id'			=>	isset( $_POST['campaignID'] ) ? $_POST['campaignID'] : '',
				'videoSource'		=>	isset( $_POST['switch_video_source'] ) ? $_POST['switch_video_source'] : ( isset( $_POST['video_source'] ) ? $_POST['video_source'] : '' ),
				'campaignName'		=>	isset( $_POST['campaign_name'] ) ? trim( stripslashes( $_POST['campaign_name'] ) ) : '',
				'campaignStatus'		=>	isset( $_POST['campaign_status'] ) ? $_POST['campaign_status'] : FALSE,

//				--- Build the Query -------------------------------------

				'singleRequest'		=>	isset( $_POST['single_request'] ) ? trim( stripslashes( $_POST['single_request'] ) ) : '',
				'qNumVideos'		=>	is_numeric( $_POST['campaign_numVideos'] ) ? (int)$_POST['campaign_numVideos'] : '',
				'qKeyphrase'		=>	isset( $_POST['campaign_keyphrase'] ) ? trim( stripslashes( $_POST['campaign_keyphrase'] ) ) : '',
				'qAssocType'		=>	isset( $_POST['campaign_assoc_type'] ) ? $_POST['campaign_assoc_type'] : '',
				'qAssoc'		=>	isset( $_POST['campaign_assoc'] ) ? trim( stripslashes( $_POST['campaign_assoc'] ) ) : '',
				'qPublishedAfter'	=>	isset( $_POST['campaign_pub_after'] ) ? trim( stripslashes( $_POST['campaign_pub_after'] ) ) : '',
				'qPublishedBefore'	=>	isset( $_POST['campaign_pub_before'] ) ? trim( stripslashes( $_POST['campaign_pub_before'] ) ) : '',
				'qOrderBy'		=>	isset( $_POST['campaign_orderby'] ) ? $_POST['campaign_orderby'] : '',
				'qOrderDirection'	=>	isset( $_POST['campaign_order_direction'] ) ? $_POST['campaign_order_direction'] : '',
				'qCategory'		=>	isset( $_POST['campaign_search_category'] ) ? $searchCats : '',
				'qDuration'		=>	isset( $_POST['campaign_search_duration'] ) ? $_POST['campaign_search_duration'] : '',
				'qDefinition'		=>	isset( $_POST['campaign_search_definition'] ) ? $_POST['campaign_search_definition'] : 'any',
				'qImageImport'		=>	isset( $_POST['campaign_image_import'] ) ? TRUE : FALSE,
				'qSyndication'		=>	isset( $_POST['campaign_syndication'] ) ? $_POST['campaign_syndication'] : '',
				'qRegionCode'		=>	isset( $_POST['campaign_region_code'] ) ? trim( stripslashes( $_POST['campaign_region_code'] ) ) : '',
				'qLanguage'		=>	isset( $_POST['campaign_language'] ) ? trim( stripslashes( $_POST['campaign_language'] ) ) : '',
				'qSafeSearch'		=>	isset( $_POST['campaign_safesearch'] ) ? trim( stripslashes( $_POST['campaign_safesearch'] ) ) : '',
				'qNumComments'		=>	isset( $_POST['campaign_num_comments'] ) && is_numeric( $_POST['campaign_num_comments'] ) ? (int)$_POST['campaign_num_comments'] : '',
				'qCommentOrderBy'	=>	isset( $_POST['campaign_comment_orderby'] ) ? $_POST['campaign_comment_orderby'] : 'time',
				'qCommentSearchTerms'	=>	isset( $_POST['campaign_comment_search'] ) ? trim( stripslashes( $_POST['campaign_comment_search'] ) ) : '',
				'qCommentReplies'	=>	isset( $_POST['campaign_comment_replies'] ) ? TRUE : FALSE,
				'qExtraParams'		=>	isset( $_POST['campaign_extra_params'] ) ? trim( stripslashes( $_POST['campaign_extra_params'] ) ) : '',
				'qExtraParams2'		=>	isset( $_POST['campaign_extra_params2'] ) ? trim( stripslashes( $_POST['campaign_extra_params2'] ) ) : '',

//				--- Process the Results -------------------------------------

				'pTranslateTo'		=>	isset( $_POST['campaign_translate_to'] ) ? trim( stripslashes( $_POST['campaign_translate_to'] ) ) : '',
				'pSkipTitle'		=>	isset( $_POST['campaign_skip_titles'] ) ? TRUE : FALSE,
				'pStrictTitle'		=>	isset( $_POST['campaign_strict_title'] ) ? trim( stripslashes( $_POST['campaign_strict_title'] ) ) : '',
				'pNegateTitle'		=>	isset( $_POST['campaign_negate_title'] ) ? trim( stripslashes( $_POST['campaign_negate_title'] ) ) : '',
				'pMinViews'		=>	is_numeric( $_POST['campaign_min_views'] ) ? (int)$_POST['campaign_min_views'] : 0,
				'pCutoffDate'		=>	isset( $_POST['campaign_cutoff_date'] ) ? trim( stripslashes( $_POST['campaign_cutoff_date'] ) ) : '',
				'pDurationMin'		=>	isset( $_POST['campaign_duration_min'] ) ? trim( stripslashes( $_POST['campaign_duration_min'] ) ) : '',
				'pDurationMax'		=>	isset( $_POST['campaign_duration_max'] ) ? trim( stripslashes( $_POST['campaign_duration_max'] ) ) : '',
				'pNoThumbnails'		=>	isset( $_POST['campaign_no_thumbnails'] ) ? TRUE : FALSE,
				'pLimitTitleChars'	=>	is_numeric( $_POST['campaign_limit_title_chars'] ) ? (int)$_POST['campaign_limit_title_chars'] : 0,
				'pLimitTitleWord'	=>	isset( $_POST['campaign_limit_title_word'] ) ? trim( stripslashes( $_POST['campaign_limit_title_word'] ) ) : '',
				'pRemoveTitleChars'	=>	isset( $_POST['campaign_remove_title_chars'] ) ? trim( stripslashes( $_POST['campaign_remove_title_chars'] ) ) : '',
				'pReplacePhrases'	=>	isset( $_POST['campaign_replace_phrases'] ) ? stripslashes( $_POST['campaign_replace_phrases'] ) : '',
				'pLimitDescChars'	=>	is_numeric( $_POST['campaign_limit_desc_chars'] ) ? (int)$_POST['campaign_limit_desc_chars'] : 0,
				'pLimitDescWord'	=>	isset( $_POST['campaign_limit_desc_word'] ) ? trim( stripslashes( $_POST['campaign_limit_desc_word'] ) ) : '',
				'pSentenceRemove'	=>	is_numeric( $_POST['campaign_sentence_remove'] ) ?  (int)$_POST['campaign_sentence_remove'] : 0,
				'pSentenceRemovePhrase'	=>	isset( $_POST['campaign_sentence_remove_phrase'] ) ? trim( stripslashes( $_POST['campaign_sentence_remove_phrase'] ) ) : '',
				'pRemoveDescUrls'	=>	isset( $_POST['campaign_remove_desc_urls'] ) ? TRUE : FALSE,
				'pRemoveCommUrls'	=>	isset( $_POST['campaign_remove_comm_urls'] ) ? TRUE : FALSE,
				'pLinkifyTypes'		=>	isset( $_POST['campaign_linkify_types'] ) ? $_POST['campaign_linkify_types'] : '',
				'pLinkifyDescUrls'	=>	isset( $_POST['campaign_linkify_desc_urls'] ) ? TRUE : FALSE,
				'pLinkifyCommUrls'	=>	isset( $_POST['campaign_linkify_comm_urls'] ) ? TRUE : FALSE,
				'pSpinTitle'		=>	isset( $_POST['campaign_spin_title'] ) ? TRUE : FALSE,
				'pSpinDesc'		=>	isset( $_POST['campaign_spin_desc'] ) ? TRUE : FALSE,
				'pSpinner'		=>	isset( $_POST['campaign_spinner'] ) ? $_POST['campaign_spinner'] : '',
				'pSpinnerPost'		=>	isset( $_POST['campaign_spinner_post'] ) ? $_POST['campaign_spinner_post'] : '',
				'pSpinLength'		=>	is_numeric( $_POST['campaign_spin_length'] ) ? (int)$_POST['campaign_spin_length'] : 0,
				'pTitleTemplate'	=>	isset( $_POST['campaign_title_template'] ) ? trim( stripslashes( $_POST['campaign_title_template'] ) ) : '',
				'pPostTemplate'		=>	isset( $_POST['campaign_post_template'] ) ? trim( stripslashes( $_POST['campaign_post_template'] ) ) : '',
				'pImageTemplate'	=>	isset( $_POST['campaign_image_template'] ) ? trim( stripslashes( $_POST['campaign_image_template'] ) ) : '',
				'pCreateExcerpt'	=>	isset( $_POST['campaign_create_excerpt'] ) ? TRUE : FALSE,
				'pExcerptLength'	=>	is_numeric( $_POST['campaign_excerpt_length'] ) ? (int)$_POST['campaign_excerpt_length'] : 0,
				'pGrabTags'		=>	isset( $_POST['campaign_grab_tags'] ) ? TRUE : FALSE,
				'pMinTagLength'		=>	is_numeric( $_POST['campaign_min_tag_length'] ) ? (int)$_POST['campaign_min_tag_length'] : 0,
				'pIgnoreTags'		=>	isset( $_POST['campaign_ignore_tags'] ) ? trim( stripslashes( $_POST['campaign_ignore_tags'] ) ) : '',
				'pAddTags'		=>	isset( $_POST['campaign_add_tags'] ) ? trim( stripslashes( $_POST['campaign_add_tags'] ) ) : '',
				'pProcessRetro'		=>	isset( $_POST['campaign_process_retro'] ) ? TRUE : FALSE,
				'pThemeHelper'		=>	isset( $_POST['campaign_theme_helper'] ) ? TRUE : FALSE,

//				--- Create the Post -------------------------------------

				'cUser'			=>	$_POST['campaign_user'],
				'cPostType'		=>	$_POST['campaign_post_type'],
				'cPostStatus'		=>	$_POST['campaign_post_status'],
				'cPostFormat'		=>	$_POST['campaign_post_format'],
				'cCategories'		=>	isset( $theCats ) ? $theCats : '',
				'cTaxTerms'             =>      isset( $_POST['campaign_taxterms'] ) ? (array) $_POST['campaign_taxterms'] : array(),
				'cAddSourceCategory'	=>	isset( $_POST['campaign_add_source_cat'] ) ? TRUE : FALSE,
				'cUsePublishedDate'	=>	isset( $_POST['campaign_use_publish_date'] ) ? TRUE : FALSE
			);
			// get custom fields here:
			$cnt=0;
			$args['pMetaFields'] = array();
			foreach ($_POST['campaign_metakey'] as $m) {
				if ( $m ) {
					$m = trim( stripslashes( $m ) );
					$args['pMetaFields'][$m] = trim( stripslashes( $_POST['campaign_metavalue'][$cnt] ) );
				}
				$cnt++;
			}

		$this->info_message( sprintf( __( '%s values: %s ', 'autovid-profit' ), __FUNCTION__, htmlentities( print_r($args,true) ) ), 'updated', 'debug' );
		return $args;
	}

	/*
	 * Re-Process all previous videos imported on this campaign
	 */
	private function process_retroactively( $args ) {
		if ( ! $args['id'] ) {
			return $this->info_message( __( 'Error: Cannot apply changes because campaign ID is 0', 'autovid-profit' ) );
		}

		$args = apply_filters( 'vb_video_campaign', $args );

		$getMore = true;
		$offset = 0;
		$chunksize = apply_filters( 'vb_posts_chunksize', 1000 );
		$query_args = array(
			'posts_per_page' => $chunksize,
			'post_type' => 'any',
			'post_status' => array( 'pending', 'draft', 'publish', 'private' ),
			'meta_key' => 'VideoCampaign',
			'meta_value' => $args['id']
			);
		$query_args = apply_filters( 'vb_process_retroactively', $query_args );
		wp_suspend_cache_addition(true);

		$total = 0;
		while ( $getMore ) {
			$posts = get_posts( $query_args );
			$num_results = count( $posts );

			$this->info_message( sprintf( __( 'Applying changes to %d videos previously imported on campaign ID %s at offset %d ', 'autovid-profit' ), $num_results, $args['id'], $offset ), 'updated', 'critical' );

			foreach ( $posts as $post ) {
			$taglist = "";
			$posttags = get_the_tags( $post->ID );
			if ( ! empty( $posttags ) ) {
				foreach ( $posttags as $tag ) {
					$taglist .= $tag->name . ',';
				}
			}
			$taglist = rtrim( $taglist, ",");
			// these are the only fields that may be used/updated:
			$videoInfo = array(
				'postID' 	=> $post->ID,
				'association' 	=> get_post_meta( $post->ID, 'VideoAssoc', TRUE ),
				'channelID' 	=> get_post_meta( $post->ID, 'VideoChannelID', TRUE ),
				'duration' 	=> get_post_meta( $post->ID, 'VideoDuration', TRUE ),
				'videoID' 	=> get_post_meta( $post->ID, 'VideoID', TRUE ),
				'videoEmbed' 	=> get_post_meta( $post->ID, 'VideoEmbed', TRUE ),
				'img' 		=> get_post_meta( $post->ID, 'VideoImage', TRUE ),
				'desc' 		=> get_post_meta( $post->ID, 'VideoOrigDesc', TRUE ),
				'title' 	=> get_post_meta( $post->ID, 'VideoOrigTitle', TRUE ),
				'url' 		=> get_post_meta( $post->ID, 'VideoUrl', TRUE ),
				'skip'		=> get_post_meta( $post->ID, 'VideoSkip', TRUE ),
				'viewCount'	=> get_post_meta( $post->ID, 'views', TRUE ),
				'likeCount'	=> get_post_meta( $post->ID, 'VideoLikes', TRUE ),
				'dislikeCount'	=> get_post_meta( $post->ID, 'VideoDislikes', TRUE ),
				'favoriteCount'	=> get_post_meta( $post->ID, 'VideoFavorites', TRUE ),
				'publishedAt'	=> get_post_meta( $post->ID, 'VideoPublish', TRUE ),
				'tags' 		=> $taglist
			);

			if ( empty( $videoInfo['publishedAt'] ) ) {
				// then use post date
				$videoInfo['publishedAt'] = $post->post_date;
			}

			$cats = get_the_terms( $post->ID, 'category' );
			$this->info_message( sprintf( __( 'get terms: %s ', 'autovid-profit' ), print_r($cats,true) ), 'updated', 'debug' );
			$videoInfo['sourceCategory'] = $videoInfo['sourceCategoryName'] = '';
			$videoInfo['sourceCategoryName'] = '';
/* MARK: this should use campaign cats... */
			if ( ! empty( $cats ) ) {
				foreach ( $cats as $cat ) {
					$this->info_message( sprintf( __( 'get terms cat: %s ', 'autovid-profit' ), print_r($cat,true) ), 'updated', 'debug' );
					$videoInfo['sourceCategoryName'] .= $cat->name . ',';
					$videoInfo['sourceCategory'] .= $this->create_the_category( $cat->name ) . ',';
				}
			}
			$videoInfo['sourceCategory'] = rtrim( $videoInfo['sourceCategory'], ',' );
			$videoInfo['sourceCategoryName'] = rtrim( $videoInfo['sourceCategoryName'], ',' );

			$this->info_message( sprintf( __( 'Processing video %s ', 'autovid-profit' ), htmlentities(print_r($videoInfo,true)) ), 'updated', 'debug' );
			if ( ! empty( $videoInfo['skip'] ) ) { // do not preprocess videos with this meta set
				$this->info_message( __( 'VideoSkip meta field detected', 'autovid-profit' ), 'updated', 'debug' );
				continue;
			}
			if ( ! $this->process_query_results( $args, $videoInfo ) ) {
				continue;
			}
			$postID = $this->create_the_post( $args, $videoInfo );
			if ( FALSE !== stripos( $videoInfo['post_content'],'%VideoImageLocal%' ) ) {
				//if %VideoImageLocal% tag in template we need to update_post 
				$thumbID = get_post_thumbnail_id( $postID );
				$this->process_the_thumbnail( $postID, $thumbID, $videoInfo );
			}
			$total++;

			// update comments too? ugh 
			if ( TRUE == $args['pRemoveCommUrls'] || TRUE == $args['pLinkifyCommUrls'] ) {
				$comment_args = array(
					'post_id'	=> $post->ID
					);
				$comments = get_comments( $comment_args );
				foreach ( $comments as $comment ) {
					if ( TRUE == $args['pRemoveCommUrls'] ) {
						$comment->comment_content = $this->remove_urls( $comment->comment_content );
					}
					if ( TRUE == $args['pLinkifyCommUrls'] ) {
						$comment->comment_content = $this->linkify_filtered( $comment->comment_content, $args['pLinkifyTypes'] );
					}
					wp_update_comment( ( array )$comment );
				}
			}
			} // end foreach posts
			$offset += $chunksize;
			$query_args['offset'] = $offset;
			if ( $num_results < $chunksize ) $getMore = false;
			$this->extend_execution_time();
		} // end getMore
		$this->info_message( sprintf( __( 'Processing results finished - %s videos updated', 'autovid-profit' ), $total ), 'updated', 'critical' );
	}

	/*
	 * default postTemplate depends on Blogster/Pornster and videoSource
	 */

	public function get_default_post_template( $videoSource = 'YouTube' ) {
		$name = $this->get_my_plugin( 'name' );
		$dfault_post = null;
		if ( $videoSource == 'RSS' ) {
#MARK [video src="%VideoUrl%" poster="%VideoImage%"]
$default_post = <<<RSS
<p>%VideoDescription%</p>
RSS;
		}
		else if ( $name == 'AutoVid Profit Pro' ) {
			$default_post = get_option( $this->get_my_plugin( 'prefix' ) . 'default_post_template' );
		}
		else if ( $name == 'Video Pornster' && $videoSource == 'GayTube' ) {
$default_post = <<<GAYTUBE
<iframe width='800' height='600' src='http://embed.gaytube.com/media/%videoID%/' frameborder='0' scrolling='no' allowfullscreen></iframe>
<p>%VideoDescription%</p>
GAYTUBE;
		}
		else if ( $name == 'Video Pornster' && $videoSource == 'GotPorn' ) {
$default_post = <<<GOTPORN
<iframe width='800' height='600' src="http://www.gotporn.com/video/%VideoID%/embedframe" frameborder="0" allowfullscreen=""></iframe>
<p>%VideoDescription%</p>
GOTPORN;
		}
		else if ( $name == 'Video Pornster' && $videoSource == 'PornerBros' ) {
$default_post = <<<PORNERBROS
<iframe width='800' height='600' src="http://www.pornerbros.com/embed/%VideoID%" frameborder="0" allowfullscreen></iframe>
<p>%VideoDescription%</p>
PORNERBROS;
		}
		else if ( $name == 'Video Pornster' && $videoSource == 'PornHub' ) {
$default_post = <<<PORNHUB
<iframe width='800' height='600' src='http://www.pornhub.com/embed/%videoID%' frameborder='0' scrolling='no' allowfullscreen></iframe>
<p>%VideoDescription%</p>
PORNHUB;
		}
		else if ( $name == 'Video Pornster' && ( $videoSource == 'PornTube' || $videoSource == 'PornTube2' ) ) {
$default_post = <<<PORNTUBE
<iframe width='800' height='600' src='http://www.porntube.com/embed/%videoID%' frameborder='0' scrolling='no' allowfullscreen></iframe>
<p>%VideoDescription%</p>
PORNTUBE;
		}
		else if ( $name == 'Video Pornster' && $videoSource == 'RedTube' ) {
$default_post = <<<REDTUBE
<iframe width='800' height='600' src='http://embed.redtube.com/?id=%videoID%' frameborder='0' scrolling='no'></iframe>
<p>%VideoDescription%</p>
REDTUBE;
		}
		else if ( $name == 'Video Pornster' && $videoSource == 'SpankWire' ) {
$default_post = <<<SPANKWIRE
<iframe width='800' height='600' src="http://www.spankwire.com/EmbedPlayer.aspx?ArticleId=%videoID%" frameborder="0" scrolling="no" name="spankwire_embed_video"></iframe>
<p>%VideoDescription%</p>
SPANKWIRE;
		}
		else if ( $name == 'Video Pornster' && $videoSource == 'Tube8' ) {
$default_post = <<<TUBE8
<iframe width='800' height='600' src='%VideoUrl%' frameborder='0' scrolling='no'></iframe>
<p>%VideoDescription%</p>
TUBE8;
		}
		else if ( $name == 'Video Pornster' && $videoSource == 'xTube' ) {
$default_post = <<<XTUBE
<iframe src='http://www.xtube.com/embedded/user/play.php?v=%videoID%&embedSize=big' width='800' height='600' scrolling=no name='xt_embed_video'></iframe>
<p>%VideoDescription%</p>
XTUBE;
		}
		else if ( $name == 'Video Pornster' && ( $videoSource == 'xHamster' || $videoSource == 'xHamster2' ) ) {
$default_post = <<<XHAMSTER
<iframe width='800' height='600' src='http://xhamster.com/xembed.php?video=%videoID%' frameborder='0' scrolling='no'></iframe>
<p>%VideoDescription%</p>
XHAMSTER;
		}
		else if ( $name == 'Video Pornster' && ( $videoSource == 'xVideos' || $videoSource == 'xVideos2' ) ) {
$default_post = <<<XVIDEOS
<iframe width='800' height='600' src='http://flashservice.xvideos.com/embedframe/%videoID%' frameborder='0' scrolling='no'></iframe>
<p>%VideoDescription%</p>
XVIDEOS;
		}
		else if ( $name == 'Video Pornster' && $videoSource == 'YouPorn' ) {
$default_post = <<<YOUPORN
<iframe width='800' height='600' src='%videoURL%' frameborder='0' scrolling='no'></iframe>
<p>%VideoDescription%</p>
YOUPORN;
		}
		return $default_post; 
	}


	private function check_theme_helper( $args ) {
		$name = $this->get_my_plugin( 'name' );
		$args = apply_filters( 'vb_make_campaign', $args );
		return $args;
	}

	/*
	 * Show and process make video campaign page
	 */
	public function make_video_campaign() {
		$this->determine_video_modules();
		$this->get_options_to_vars();
		$this->display_msgs = 1;
		if ( ini_get( 'safe_mode' ) ) { // if safe mode, set the max execution time
			$max = get_option( $this->get_my_plugin( 'prefix' ) . 'max_execution_time' );
			ini_set( 'max_execution_time', $max );
			$this->info_message( sprintf( __( 'Safe mode on, resetting max_execution_time to %s seconds.', 'autovid-profit' ), $max ), 'updated', 'debug' );
		}

		if ( function_exists( 'apache_get_modules' ) ) {
			$modules = apache_get_modules();
			$mod_php = in_array('mod_php5', $modules) ? "mod_php5 module detected" : "mod_php5 module not detected";
			$this->info_message( "Autovid Profit: " . $mod_php, 'updated', 'debug' );
		}


		// default args to show:
		$args = array( 
			'func' 			=> 'Make', 
			'videoSource' 		=> in_array( 'YouTube', $this->modules ) ? 'YouTube' : 'xHamster2', 
			'campaignStatus'		=> 'Active',
			'pTitleTemplate' 	=> get_option( $this->get_my_plugin( 'prefix' ) . 'default_title_template' ),
			'pImageTemplate' 	=> '%VideoTitle%',
			'pMetaFields'		=> array(),
			'cUser' 		=> '(random)',
			'cPostType' 		=> 'post',
			'cPostStatus' 		=> 'publish',
			'cPostFormat' 		=> 'standard' 
		);     // defaults 
		$args['pPostTemplate']		= $this->get_default_post_template( $args['videoSource'] );

		$theme = $this->get_theme();
		$theme_helper = $this->get_theme_helper( $theme );
		$this->info_message( sprintf( __( 'Theme Detected: %s', 'autovid-profit' ), $theme->Name ), 'updated', 'debug' );
		$this->info_message( sprintf( __( 'Checking for possible Theme Helper File: %s', 'autovid-profit' ), $theme_helper ), 'updated', 'debug' );

		if ( file_exists( $theme_helper ) || is_link( $theme_helper ) ) {
			$this->info_message( sprintf( __( 'Including File: %s', 'autovid-profit' ), $theme_helper ), 'updated', 'debug' );
		}

		 if ( isset( $_POST['change_source'] ) ) {
		// change video_source selected
			$args = $this->get_video_campaign_form();
			// VP HAS to change to the specific site template.
			if ( $this->get_my_plugin( 'name' ) == 'Video Pornster' && false !== stripos( $args['pPostTemplate'], '<iframe' ) ) {
				$args['pPostTemplate'] = $this->get_default_post_template( $args['videoSource'] );
			}
		}

		else if ( isset( $_GET['editID'] ) && ! isset( $_POST['campaignID'] ) ) {	
		// user clicked edit link on show video campaign summary page
			$campaignID = $_GET['editID'];
			$results = $this->get_video_campaign_details( $campaignID );
			if ( $results ) {
				$args = (array)$results[0];
				$args['func'] = 'Edit';
			}
			$name = $this->get_my_plugin( 'name' );
			$args = apply_filters( 'vb_edit_campaign', $args );
		}
		else if ( isset( $_POST['grab_campaign'] ) ) {
		// the Import Videos now button was pressed
			$args = $this->get_video_campaign_form();
			$this->create_video_source( $args );
			$this->grab_videos();
		}
		else if ( isset( $_POST['save_campaign'] ) ) {
		// an Add or Edit form was submitted
			$args = $this->get_video_campaign_form();
			if ( empty( $args['pPostTemplate'] ) && $this->get_my_plugin( 'name' ) == 'Video Pornster' ) {
				$args['pPostTemplate']		= $this->get_default_post_template( $args['videoSource'] );
			}
			if ( true == $args['pThemeHelper'] ) {
				$args = $this->check_theme_helper( $args );
			}
                        $campaignID = isset( $_POST['campaignID'] ) ? $_POST['campaignID'] : 0;
			if ( $campaignID ) {	// this is an Edit
				$args['id'] = $campaignID;
				$this->video_campaign_db_edit( $args );
				if ( $args['pProcessRetro'] ) {
					$this->process_retroactively( $args );
				}
			}
			else if ( ! $campaignID ) {	// this is an Add
				$this->video_campaign_db_add( $args );
			}
		}
		else if ( isset( $_POST['copy_campaign'] ) ) {
			$args = $this->get_video_campaign_form();
			$this->info_message( sprintf( __( 'Campaign %s copied.', 'autovid-profit' ), $_POST['campaignID'] ), 'updated', 'debug' );
                        $args['id'] = $campaignID = '';
			$args['func'] = 'Make';
		}
		else {
		// Make a new video campaign 
			$args = $this->check_theme_helper( $args );

			$this->info_message( sprintf( __( 'setting form defaults: %s .', 'autovid-profit' ), htmlentities( print_r( $args,true ) ) ), 'updated', 'debug' );

		}

		include( sprintf( "%s/templates/video-campaign.php", $this->get_my_plugin('dir') ) );
		$this->display_msgs = 0;
	}

	/*
	 * Check for custom plugins or themes and load their filter functions automatically
	 */
	private function check_for_custom_includes() {

		// check for theme helper file, if any
		$theme = $this->get_theme();
		$theme_helper = $this->get_theme_helper( $theme );
		if ( file_exists( $theme_helper ) || is_link( $theme_helper) ) {
			$this->vb_include_once( $theme_helper );
		}

		// check for specific plugin support - have to check each one individually.
		include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		if ( is_plugin_active( 'baw-post-views-count/bawpv.php' ) ) {
			$this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'plugin-helpers/baw-post-views-count.php' );
		}
		if ( is_plugin_active( 'contus-video-gallery/hdflvvideoshare.php' ) ) {
			$this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'plugin-helpers/contus-video-gallery.php' );
		}
		if ( is_plugin_active( 'wti-like-post/wti_like_post.php' ) ) { 	// plugin WTI Like Post
			$this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'plugin-helpers/wti-like-post.php' );
		}
		if ( is_plugin_active( 'wordpress-seo/wp-seo.php' ) ) {
			$this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'plugin-helpers/yoast-seo.php' );
		}
		if ( is_plugin_active( 'entry-views/entry-views.php' ) ) {
			$this->vb_include_once( $this->get_my_plugin( 'dir' ) . 'plugin-helpers/entry-views.php' );
		}

	}

	/*
	 * A wrapper for video_source->grab_videos
	 */
	private function grab_videos() {
		if ( ! $this->video_source ) {
			return;
		}
		$this->info_message( sprintf( __( 'PHP version is %s', 'autovid-profit' ), PHP_VERSION ), 'updated', 'debug' );

		$this->info_message( sprintf( __( 'Removing WordPress flood filter check', 'autovid-profit' ) ), 'updated', 'debug' );
		add_filter( 'comment_flood_filter', array( $this, 'comment_flood_filter_off' ), 10, 3 );

		$this->video_source->grab_videos();

		$this->info_message( sprintf( __( 'Restoring WordPress flood filter check', 'autovid-profit' ) ), 'updated', 'debug' );
		remove_filter( 'comment_flood_filter', array( $this, 'comment_flood_filter_off' ) );
	}

	/*
	 * Retrieve video campaign(s) from the database
	 */
	private function get_video_campaign_details( $campaignID="", $orderby="" ) {
                global $wpdb;
		$wpdb->suppress_errors( false );
		$wpdb->show_errors();
                $table_name = $wpdb->prefix . $this->get_my_plugin( 'prefix' ) . 'campaign';
                $sql = "SELECT * FROM " . $table_name;
		if ( $campaignID ) {
			$sql .= " WHERE id = " . $campaignID;
		}
		if ( $orderby ) {
			$sql .= " ORDER BY " . $orderby;
		}
		$this->info_message( sprintf( __( '%s SQL statement: %s', 'autovid-profit' ), __FUNCTION__, $sql ), 'updated', 'debug' );
                $results = $wpdb->get_results( $sql );
		foreach ( $results as $index => $result ) {
			if ( isset( $results[$index]->pMetaFields ) ) {
				$results[$index]->pMetaFields = maybe_unserialize( $results[$index]->pMetaFields );
			}
			else {
				$results[$index]->pMetaFields = array();
			}
			if ( isset( $results[$index]->cTaxTerms ) ) {
				$results[$index]->cTaxTerms = maybe_unserialize( $results[$index]->cTaxTerms );
			}
			else {
				$results[$index]->cTaxterms = array();
			}
		}
		$wpdb->hide_errors();
		$wpdb->suppress_errors( true );
		return $results;
	}

	/*
	 * Show and process show video campaign page
	 */
	public function show_video_campaign() {
		$this->get_options_to_vars();
		$this->display_msgs = 1;
		if ( isset( $_GET['importID'] ) ) {	
		// user clicked import now link on show video campaign summary page
			$campaignID = $_GET['importID'];
			$results = $this->get_video_campaign_details( $campaignID );
			if ( $results ) {
				$args = (array)$results[0];
				$this->create_video_source( $args );
				$this->grab_videos();
			}
		}
		if ( isset( $_POST['mass_import'] ) ) {
			// mass delete selections
			foreach ($_POST['select'] as $s) {
				$results = $this->get_video_campaign_details( $s );
				if ( $results ) {
					$args = (array)$results[0];
					$this->create_video_source( $args );
					$this->grab_videos();
				}
			}
		}
		else if ( isset( $_POST['mass_delete'] ) ) {
			// mass delete selections
			foreach ($_POST['select'] as $s) {
				$this->video_campaign_db_delete( $s );
			}
		}
		else if ( isset( $_POST['mass_edit'] ) ) {
			// mass edit selections
			$args_edit = $this->get_video_campaign_form();
			// only merge items with filled out values
			foreach ( $args_edit as $key => $value ) {
				if ( is_array( $args_edit[$key] ) && empty( $args_edit[$key] ) ) {
					unset ( $args_edit[$key] );
				}
				else if ( empty( $args_edit[$key] ) && strlen($args_edit[$key]) <= 0 ) { 
					unset ( $args_edit[$key] );
				}
			}
			$this->info_message( sprintf( __( 'Mass Edit, form args detected: %s', 'autovid-profit' ), htmlentities( print_r( $args_edit,true ) ) ), 'updated', 'debug' );
			foreach ($_POST['select'] as $s) {
				$results = $this->get_video_campaign_details( $s );
				if ( ! $results ) {
					continue;	// should never happen
				}
				$args_db = (array)$results[0];
				$args = array_merge( $args_db, $args_edit );
				$this->video_campaign_db_edit( $args );
				if ( isset( $args['pProcessRetro'] ) && $args['pProcessRetro'] ) {
					$this->process_retroactively( $args );
				}
			}
		}
		else if ( isset( $_GET['delID'] ) ) {
			// delete button was pressed
			$this->video_campaign_db_delete( $_GET['delID'] );
		}
		include( sprintf( "%s/templates/show-video-campaign.php", $this->get_my_plugin('dir') ) );
		$this->display_msgs = 0;
	}

	/*
	 * Delete a video campaign from the database
	 */
	private function video_campaign_db_delete( $delID ) {
	        global $wpdb;
		$wpdb->suppress_errors( false );
		$wpdb->show_errors();
        	$table_name = $wpdb->prefix . $this->get_my_plugin( 'prefix' ) . 'campaign';
		$result = $wpdb->delete(
			$table_name,
			array( 'ID' => $delID ),
			array( '%d' )
			);
		if ( FALSE !== $result ) {
			$this->info_message( sprintf( __( 'Video campaign %s deleted successfully.', 'autovid-profit' ), $delID ), 'updated', 'critical' );
		}
		else {
			$this->info_message( sprintf( __( 'Error: Unable to delete video campaign %s from database - %s', 'autovid-profit' ), $delID, $wpdb->print_error() ) );
		}
		$wpdb->hide_errors();
		$wpdb->suppress_errors( true );
	}

	/*
	 * Save an edited video campaign to the database
	 */
	private function video_campaign_db_edit( $args ) {
	        global $wpdb;
		$wpdb->suppress_errors( false );
		$wpdb->show_errors();
		$campaignID = $args['id'];
		$table_name = $wpdb->prefix . $this->get_my_plugin( 'prefix' ) . 'campaign';
		$data =	array(
				'videoSource'		=>	$args['videoSource'],
				'campaignName'		=>	$args['campaignName'],
				'campaignStatus'		=>	$args['campaignStatus'],
				'qNumVideos'		=>	$args['qNumVideos'],
				'qKeyphrase'		=>	$args['qKeyphrase'],
				'qAssocType'		=>	$args['qAssocType'],
				'qAssoc'		=>	$args['qAssoc'],
				'qPublishedAfter'	=>	$args['qPublishedAfter'],
				'qPublishedBefore'	=>	$args['qPublishedBefore'],
				'qOrderBy'		=>	$args['qOrderBy'],
				'qOrderDirection'	=>	$args['qOrderDirection'],
				'qCategory'		=>	$args['qCategory'],
				'qDuration'		=>	$args['qDuration'],
				'qDefinition'		=>	$args['qDefinition'],
				'qImageImport'		=>	$args['qImageImport'],
				'qSyndication'		=>	$args['qSyndication'],
				'qRegionCode'		=>	$args['qRegionCode'],
				'qLanguage'		=>	$args['qLanguage'],
				'qSafeSearch'		=>	$args['qSafeSearch'],
				'qNumComments'		=>	$args['qNumComments'], 
				'qCommentOrderBy'	=>	$args['qCommentOrderBy'], 
				'qCommentSearchTerms'	=>	$args['qCommentSearchTerms'], 
				'qCommentReplies'	=>	$args['qCommentReplies'], 
				'qExtraParams'		=>	$args['qExtraParams'], 
				'qExtraParams2'		=>	$args['qExtraParams2'], 
				'pTranslateTo'		=>	$args['pTranslateTo'],
				'pSkipTitle'		=>	$args['pSkipTitle'],
				'pStrictTitle'		=>	$args['pStrictTitle'],
				'pNegateTitle'		=>	$args['pNegateTitle'],
				'pMinViews'		=>	$args['pMinViews'],
				'pCutoffDate'		=>	$args['pCutoffDate'],
				'pDurationMin'		=>	$args['pDurationMin'],
				'pDurationMax'		=>	$args['pDurationMax'],
				'pNoThumbnails'		=>	$args['pNoThumbnails'],
				'pLimitTitleChars'	=>	$args['pLimitTitleChars'],
				'pLimitTitleWord'	=>	$args['pLimitTitleWord'],
				'pRemoveTitleChars'	=>	$args['pRemoveTitleChars'],
				'pReplacePhrases'	=>	$args['pReplacePhrases'],
				'pLimitDescChars'	=>	$args['pLimitDescChars'],
				'pLimitDescWord'	=>	$args['pLimitDescWord'],
				'pSentenceRemove'	=>	$args['pSentenceRemove'],
				'pSentenceRemovePhrase'	=>	$args['pSentenceRemovePhrase'],
				'pRemoveDescUrls'	=>	$args['pRemoveDescUrls'],
				'pRemoveCommUrls'	=>	$args['pRemoveCommUrls'],
				'pLinkifyTypes'		=>	$args['pLinkifyTypes'],
				'pLinkifyDescUrls'	=>	$args['pLinkifyDescUrls'],
				'pLinkifyCommUrls'	=>	$args['pLinkifyCommUrls'],
				'pSpinTitle'		=>	$args['pSpinTitle'],
				'pSpinDesc'		=>	$args['pSpinDesc'],
				'pSpinner'		=>	$args['pSpinner'],
				'pSpinnerPost'		=>	$args['pSpinnerPost'],
				'pSpinLength'		=>	$args['pSpinLength'],
				'pTitleTemplate'	=>	$args['pTitleTemplate'],
				'pPostTemplate'		=>	$args['pPostTemplate'],
				'pImageTemplate'	=>	$args['pImageTemplate'],
				'pMetaFields'		=>	serialize( $args['pMetaFields'] ),
				'pCreateExcerpt'	=>	$args['pCreateExcerpt'],
				'pExcerptLength'	=>	$args['pExcerptLength'],
				'pGrabTags'		=>	$args['pGrabTags'],
				'pMinTagLength'		=>	$args['pMinTagLength'],
				'pIgnoreTags'		=>	$args['pIgnoreTags'],
				'pAddTags'		=>	$args['pAddTags'],
				'cUser'			=>	$args['cUser'],
				'cPostType'		=>	$args['cPostType'],
				'cPostStatus'		=>	$args['cPostStatus'],
				'cPostFormat'		=>	$args['cPostFormat'],
				'cCategories'		=>	$args['cCategories'],
				'cTaxTerms'		=>	serialize( $args['cTaxTerms'] ),
				'cAddSourceCategory'	=>	$args['cAddSourceCategory'],
				'cUsePublishedDate'	=>	$args['cUsePublishedDate']
				);
		$this->info_message( sprintf( __( 'Updating data for campaign %s into database table %s - %s', 'autovid-profit' ), $campaignID, $table_name, htmlentities( print_r( $data, true ) ) ), 'updated', 'debug' );
		$result = $wpdb->update(
			$table_name,
			$data,
			array( 'ID' => $campaignID ),
			array(
				'%s',	// videoSource
				'%s',	// campaignName
				'%s',	// campaignStatus
				'%d',	// qNumVideos
				'%s',	// qKeyphrase
				'%s',	// qAssocType
				'%s',	// qAssoc
				'%s',	// qPublishedAfter
				'%s',	// qPublishedBefore
				'%s',	// qOrderBy
				'%s',	// qOrderDirection
				'%s',	// qCategory
				'%s',	// qDuration
				'%s',	// qDefinition
				'%d',	// qImageImport
				'%s',	// qSyndication
				'%s',	// qRegionCode
				'%s',	// qLanguage
				'%s',	// qSafeSearch
				'%d',	// qNumComments
				'%s',	// qCommentOrderBy
				'%s',	// qCommentSearchTerms
				'%d',	// qCommentReplies
				'%s',	// qExtraParams
				'%s',	// qExtraParams2
				'%s',	// pTranslateTo
				'%d',	// pSkipTitle
				'%s',	// pStrictTitle
				'%s',	// pNegateTitle
				'%d',	// pMinViews
				'%s',	// pCutoffDate
				'%s',	// pDurationMin
				'%s',	// pDurationMax
				'%d',	// pNoThumbnails
				'%d',	// pLimitTitleChars
				'%s',	// pLimitTitleWord
				'%s',	// pRemoveTitleChars
				'%s',	// pReplacePhrases
				'%d',	// pLimitDescChars
				'%s',	// pLimitDescWord
				'%d',	// pSentenceRemove
				'%s',	// pSentenceRemovePhrase
				'%d',	// pRemoveDescUrls
				'%d',	// pRemoveCommUrls
				'%s',	// pLinkifyTypes
				'%d',	// pLinkifyDescUrls
				'%d',	// pLinkifyCommUrls
				'%d',	// pSpinTitle
				'%d',	// pSpinDesc
				'%s',	// pSpinner
				'%s',	// pSpinnerPost
				'%d',	// pSpinLength
				'%s',	// pTitleTemplate
				'%s',	// pPostTemplate
				'%s',	// pImageTemplate
				'%s',	// pMetaFields
				'%d',	// pCreateExcerpt
				'%d',	// pExcerptLength
				'%d',	// pGrabTags
				'%d',	// pMinTagLength
				'%s',	// pIgnoreTags
				'%s',	// pAddTags
				'%s',	// cUser
				'%s',	// cPostType
				'%s',	// cPostStatus
				'%s',	// cPostFormat
				'%s',	// cCategories
				'%s',	// cTaxTerms
				'%d',	// cAddSourceCategory
				'%d'	// cUsePublishedDate
				),
			array( '%d' ) 
			);
		if ( false !== $result ) {
			$this->info_message( sprintf( __( 'Video campaign %s edited successfully.', 'autovid-profit' ), $campaignID ), 'updated', 'critical' );
		}
		else if ( false === $result ) { // result will be 0 if data is the same, so don't report that as an error.
			$this->info_message( sprintf( __( 'Error: Unable to edit video campaign %s in database - %s', 'autovid-profit' ), $campaignID, $wpdb->print_error() ) );
		}
		$wpdb->hide_errors();
		$wpdb->suppress_errors( true );
	}

	/*
	 * Add a new video campaign to the database
	 */
	private function video_campaign_db_add( $args ) {
	        global $wpdb;
		$wpdb->suppress_errors( false );
		$wpdb->show_errors();
        	$table_name = $wpdb->prefix . $this->get_my_plugin( 'prefix' ) . 'campaign';
		$data =	array(
				'videoSource'		=>	$args['videoSource'],
				'campaignName'		=>	$args['campaignName'],
				'campaignStatus'		=>	$args['campaignStatus'],
				'qNumVideos'		=>	$args['qNumVideos'],
				'qKeyphrase'		=>	$args['qKeyphrase'],
				'qAssocType'		=>	$args['qAssocType'],
				'qAssoc'		=>	$args['qAssoc'],
				'qPublishedAfter'	=>	$args['qPublishedAfter'],
				'qPublishedBefore'	=>	$args['qPublishedBefore'],
				'qOrderBy'		=>	$args['qOrderBy'],
				'qOrderDirection'	=>	$args['qOrderDirection'],
				'qCategory'		=>	$args['qCategory'],
				'qDuration'		=>	$args['qDuration'],
				'qDefinition'		=>	$args['qDefinition'],
				'qImageImport'		=>	$args['qImageImport'],
				'qSyndication'		=>	$args['qSyndication'],
				'qRegionCode'		=>	$args['qRegionCode'],
				'qLanguage'		=>	$args['qLanguage'],
				'qSafeSearch'		=>	$args['qSafeSearch'],
				'qNumComments'		=>	$args['qNumComments'], 
				'qCommentOrderBy'	=>	$args['qCommentOrderBy'], 
				'qCommentSearchTerms'	=>	$args['qCommentSearchTerms'], 
				'qCommentReplies'	=>	$args['qCommentReplies'], 
				'qExtraParams'		=>	$args['qExtraParams'], 
				'qExtraParams2'		=>	$args['qExtraParams2'], 
				'pTranslateTo'		=>	$args['pTranslateTo'],
				'pSkipTitle'		=>	$args['pSkipTitle'],
				'pStrictTitle'		=>	$args['pStrictTitle'],
				'pNegateTitle'		=>	$args['pNegateTitle'],
				'pMinViews'		=>	$args['pMinViews'],
				'pCutoffDate'		=>	$args['pCutoffDate'],
				'pDurationMin'		=>	$args['pDurationMin'],
				'pDurationMax'		=>	$args['pDurationMax'],
				'pNoThumbnails'		=>	$args['pNoThumbnails'],
				'pLimitTitleChars'	=>	$args['pLimitTitleChars'],
				'pLimitTitleWord'	=>	$args['pLimitTitleWord'],
				'pRemoveTitleChars'	=>	$args['pRemoveTitleChars'],
				'pReplacePhrases'	=>	$args['pReplacePhrases'],
				'pLimitDescChars'	=>	$args['pLimitDescChars'],
				'pLimitDescWord'	=>	$args['pLimitDescWord'],
				'pSentenceRemove'	=>	$args['pSentenceRemove'],
				'pSentenceRemovePhrase'	=>	$args['pSentenceRemovePhrase'],
				'pRemoveDescUrls'	=>	$args['pRemoveDescUrls'],
				'pRemoveCommUrls'	=>	$args['pRemoveCommUrls'],
				'pLinkifyTypes'		=>	$args['pLinkifyTypes'],
				'pLinkifyDescUrls'	=>	$args['pLinkifyDescUrls'],
				'pLinkifyCommUrls'	=>	$args['pLinkifyCommUrls'],
				'pSpinTitle'		=>	$args['pSpinTitle'],
				'pSpinDesc'		=>	$args['pSpinDesc'],
				'pSpinner'		=>	$args['pSpinner'],
				'pSpinnerPost'		=>	$args['pSpinnerPost'],
				'pSpinLength'		=>	$args['pSpinLength'],
				'pTitleTemplate'	=>	$args['pTitleTemplate'],
				'pPostTemplate'		=>	$args['pPostTemplate'],
				'pImageTemplate'	=>	$args['pImageTemplate'],
				'pMetaFields'		=>	serialize( $args['pMetaFields'] ),
				'pCreateExcerpt'	=>	$args['pCreateExcerpt'],
				'pExcerptLength'	=>	$args['pExcerptLength'],
				'pGrabTags'		=>	$args['pGrabTags'],
				'pMinTagLength'		=>	$args['pMinTagLength'],
				'pIgnoreTags'		=>	$args['pIgnoreTags'],
				'pAddTags'		=>	$args['pAddTags'],
				'cUser'			=>	$args['cUser'],
				'cPostType'		=>	$args['cPostType'],
				'cPostStatus'		=>	$args['cPostStatus'],
				'cPostFormat'		=>	$args['cPostFormat'],
				'cCategories'		=>	$args['cCategories'],
				'cTaxTerms'		=>	serialize( $args['cTaxTerms'] ),
				'cAddSourceCategory'	=>	$args['cAddSourceCategory'],
				'cUsePublishedDate'	=>	$args['cUsePublishedDate']
				);
		$this->info_message( sprintf( __( 'Inserting data into database table %s - %s', 'autovid-profit' ), $table_name, htmlentities( print_r( $data, true ) ) ), 'updated', 'debug' );
		$result = $wpdb->insert(
			$table_name,
			$data,
			array(
				'%s',	// videoSource
				'%s',	// campaignName
				'%s',	// campaignStatus
				'%d',	// qNumVideos
				'%s',	// qKeyphrase
				'%s',	// qAssocType
				'%s',	// qAssoc
				'%s',	// qPublishedAfter
				'%s',	// qPublishedBefore
				'%s',	// qOrderBy
				'%s',	// qOrderDirection
				'%s',	// qCategory
				'%s',	// qDuration
				'%s',	// qDefinition
				'%d',	// qImageImport
				'%s',	// qSyndication
				'%s',	// qRegionCode
				'%s',	// qLanguage
				'%s',	// qSafeSearch
				'%d',	// qNumComments
				'%s',	// qCommentOrderBy
				'%s',	// qCommentSearchTerms
				'%d',	// qCommentReplies
				'%s',	// qExtraParams
				'%s',	// qExtraParams2
				'%s',	// pTranslateTo
				'%d',	// pSkipTitle
				'%s',	// pStrictTitle
				'%s',	// pNegateTitle
				'%d',	// pMinViews
				'%s',	// pCutoffDate
				'%s',	// pDurationMin
				'%s',	// pDurationMax
				'%d',	// pNoThumbnails
				'%d',	// pLimitTitleChars
				'%s',	// pLimitTitleWord
				'%s',	// pRemoveTitleChars
				'%s',	// pReplacePhrases
				'%d',	// pLimitDescChars
				'%s',	// pLimitDescWord
				'%d',	// pSentenceRemove
				'%s',	// pSentenceRemovePhrase
				'%d',	// pRemoveDescUrls
				'%d',	// pRemoveCommUrls
				'%s',	// pLinkifyTypes
				'%d',	// pLinkifyDescUrls
				'%d',	// pLinkifyCommUrls
				'%d',	// pSpinTitle
				'%d',	// pSpinDesc
				'%s',	// pSpinner
				'%s',	// pSpinnerPost
				'%d',	// pSpinLength
				'%s',	// pTitleTemplate
				'%s',	// pPostTemplate
				'%s',	// pImageTemplate
				'%s',	// pMetaFields
				'%d',	// pCreateExcerpt
				'%d',	// pExcerptLength
				'%d',	// pGrabTags
				'%d',	// pMinTagLength
				'%s',	// pIgnoreTags
				'%s',	// pAddTags
				'%s',	// cUser
				'%s',	// cPostType
				'%s',	// cPostStatus
				'%s',	// cPostFormat
				'%s',	// cCategories
				'%s',	// cTaxTerms
				'%d',	// cAddSourceCategory
				'%d'	// cUsePublishedDate
				)
			);
		if ( FALSE !== $result ) {
                       	$this->info_message( __( 'Video campaign added successfully.', 'autovid-profit' ), 'updated', 'critical' );
		}
		else {
			$this->info_message( sprintf( __( 'Error: Unable to insert video campaign in database - %s', 'autovid-profit' ), $wpdb->print_error() ) );
		}
		$wpdb->hide_errors();
		$wpdb->suppress_errors( true );
	}

	/*
	 * When plugins are loaded:
	 * Create Log and Scheduler objects
	 * Initialize the Plugin Update Checker
	 * Check to see if database structure needs to be updated
	 * Load the i18n file for translations
	 */
	public function plugin_init() { 
		#SINGLETON $this->scheduler = Autovid_Profit_Scheduler::getInstance();
		$this->scheduler = new Autovid_Profit_Scheduler();
		$this->scheduler->init( $this->plugin );

		$this->check_for_custom_includes();
		if( ! current_user_can( 'manage_options' ) ) {
			return;
		}
		if ( get_option( $this->get_my_plugin( 'prefix' ) . base64_decode( 'c3RhdHVz' ), null ) !== FALSE ) {
			// use a custom update checker for new releases
			$url = "http://www.superblogme.com/wp-update-server/?action=get_metadata&slug=";
			$license = get_option( $this->get_my_plugin( 'prefix' ) . 'license' );
			if ( stripos( $license, "-beta" ) !== FALSE ) {
				// look for beta versions instead
				$url = "http://www.superblogme.com/test-update-server/?action=get_metadata&slug=";
			}
			if ( $this->vb_include_once( 'plugin-updates/plugin-update-checker.php' ) ) {
				$this->VBPUpdateChecker = PucFactory::buildUpdateChecker(
					$url . $this->get_my_plugin( 'slug' ) ,
					$this->get_my_plugin( 'file' ),
					$this->get_my_plugin( 'slug' ) 	
				);
				$this->VBPUpdateChecker->addQueryArgFilter( array( $this, 'PUC_callback' ) );
			}
		}

		// check for plugin update which requires separate check for any db changes
		if ( get_option( $this->get_my_plugin( 'prefix' ) . 'db_version' ) != $this->get_my_plugin( 'db_version' ) ) { // database has changed
			$this->db_install();
		}

		// load i18n file
		$plugin_dir = basename( $this->get_my_plugin( 'dir' ) ) . '/languages';
		load_plugin_textdomain( 'autovid-profit', false, $plugin_dir );
	}

	/*
	 * Will create custom tables or alter the existing tables if needed
	 */
	protected function db_install()  {

		$this->save_apiKeys(); // ensure api keys are set when plugin updated from pre v2.4.0 - activation hook not called :(

	        global $wpdb;

        	if ( ! $this->vb_include_once( ABSPATH . 'wp-admin/includes/upgrade.php' ) ) {
			return 0;
		}

		$wpdb->suppress_errors( false );
		$wpdb->show_errors();
        	$table_name = $wpdb->prefix . $this->get_my_plugin( 'prefix' ) . 'campaign';

        	$charset_collate = '';	// Set default charset and collation 

        	if ( ! empty( $wpdb->charset ) ) {
                	$charset_collate = "DEFAULT CHARACTER SET {$wpdb->charset}";
		}
        	if ( ! empty( $wpdb->collate ) )  {
                	$charset_collate .= " COLLATE {$wpdb->collate}";
		}

		/// use camelCode for sql variables
        	$sql = "CREATE TABLE $table_name (
	        id smallint unsigned NOT NULL AUTO_INCREMENT,
       		time timestamp DEFAULT CURRENT_TIMESTAMP NOT NULL,
		videoSource VARCHAR(32) NOT NULL,
		campaignName VARCHAR(128),
		campaignStatus VARCHAR(16) DEFAULT 'Active',
		qNumVideos smallint unsigned NOT NULL,
		qKeyphrase VARCHAR(256),
		qAssocType VARCHAR(128),
		qAssoc VARCHAR(128),
		qPublishedAfter VARCHAR(32),
		qPublishedBefore VARCHAR(32),
		qOrderBy VARCHAR(16),
		qOrderDirection VARCHAR(8),
		qCategory VARCHAR(256) NOT NULL,
		qDuration VARCHAR(16) NOT NULL,
		qDefinition VARCHAR(16) NOT NULL,
		qImageImport tinyint DEFAULT TRUE,
		qSyndication VARCHAR(8),
		qRegionCode VARCHAR(8),
		qLanguage VARCHAR(8),
		qSafeSearch VARCHAR(16),
		qNumComments smallint NOT NULL,
		qCommentOrderBy VARCHAR(16),
		qCommentSearchTerms VARCHAR(256),
		qCommentReplies tinyint DEFAULT TRUE,
		qExtraParams VARCHAR(256),
		qExtraParams2 VARCHAR(256),
		pTranslateTo VARCHAR(8),
		pSkipTitle tinyint,
		pStrictTitle VARCHAR(256),
		pNegateTitle VARCHAR(256),
		pMinViews mediumint unsigned,
		pCutoffDate VARCHAR(32),
		pDurationMin VARCHAR(4),
		pDurationMax VARCHAR(4),
		pNoThumbnails tinyint,
		pLimitTitleChars smallint unsigned,
		pLimitTitleWord VARCHAR(32),
		pRemoveTitleChars VARCHAR(32),
		pReplacePhrases text,
		pLimitDescChars smallint unsigned,
		pLimitDescWord VARCHAR(128),
		pSentenceRemove tinyint,
		pSentenceRemovePhrase VARCHAR(256),
		pRemoveDescUrls tinyint,
		pRemoveCommUrls tinyint,
		pLinkifyTypes VARCHAR(16),
		pLinkifyDescUrls tinyint,
		pLinkifyCommUrls tinyint,
		pSpinTitle tinyint,
		pSpinDesc tinyint,
		pSpinner VARCHAR(32) NOT NULL,
		pSpinnerPost VARCHAR(32) NOT NULL,
		pSpinLength smallint unsigned,
		pTitleTemplate VARCHAR(256),
		pPostTemplate text NOT NULL,
		pImageTemplate VARCHAR(256),
		pMetaFields text,
		pCreateExcerpt tinyint,
		pExcerptLength smallint unsigned,
		pGrabTags tinyint,
		pMinTagLength tinyint,
		pIgnoreTags VARCHAR(256),
		pAddTags VARCHAR(256),
		cUser VARCHAR(64) NOT NULL,
		cPostType VARCHAR(32) NOT NULL,
		cPostStatus VARCHAR(16) NOT NULL,
		cPostFormat VARCHAR(16) NOT NULL,
		cCategories VARCHAR(256) NOT NULL,
		cTaxTerms text,
		cAddSourceCategory tinyint,
		cUsePublishedDate tinyint,
        	UNIQUE KEY id (id)
        	) $charset_collate;";

        	dbDelta( $sql );

        	update_option( $this->get_my_plugin( 'prefix' ) . 'db_version', $this->get_my_plugin( 'db_version' ) );
		if ( $this->logger ) {
			$this->logger->write_to_log( sprintf( __( '%s table verified - db version %s', 'autovid-profit' ), $table_name, $this->get_my_plugin( 'db_version' ) ) );
		}
		$wpdb->hide_errors();
		$wpdb->suppress_errors( true );
	}

	/*
	 * Replace template tags with the video info
	 */
  
  	// modified by ryan yang
	private function expand_template( $template, $videoInfo, $spintax = true ) {
		$template = str_ireplace( "%VideoAssociation%", $videoInfo['association'], $template);
		if ( isset( $videoInfo['channelID'] ) ) {	// YouTube only
			$template = str_ireplace( "%VideoChannelId%", $videoInfo['channelID'], $template);
		}
		$template = str_ireplace( "%VideoDescription%", $videoInfo['desc'], $template);
		// support for durations in ISO 8601 format
		if ( strpos( $template, '%VideoDuration8601%' ) !== false && preg_match( "/^([0-9]+):([0-9]+)(:[0-9]+)?$/", $videoInfo['duration'], $matches ) ) {
			// DateTime is rather picky... ensure video durations are in right format
			$cnt = count( $matches );
			if ( $cnt == 3 ) {	// turn mm:ss into hh:mm:ss if needed
				$videoInfo['duration'] = sprintf( "00:%02d:%02d", $matches[1], $matches[2] );
			}
			else {
				$videoInfo['duration'] = sprintf( "%02d:%02d:%02d", $matches[1], $matches[2], $matches[3] );
			}
			try {
				$vduration = new DateTime( $videoInfo['duration'] );
			} catch ( Exception $e ) {
    				$this->info_message( __FUNCTION__ . " DateTime from Duration8601 - " . $e->getMessage() );
			}
			if ( $vduration ) {
				$duration8601 = "P" . $vduration->format('\TH\Hi\Ms\S');
				$template = str_ireplace( "%VideoDuration8601%", $duration8601, $template);
			}
		}
		$template = str_ireplace( "%VideoDuration%", $videoInfo['duration'], $template);
		$template = str_ireplace( "%VideoSource%", $videoInfo['videoSource'], $template);
		$template = str_ireplace( "%VideoTitle%", $videoInfo['title'], $template);
		$template = str_ireplace( "%VideoID%", $videoInfo['videoID'], $template);
		$template = str_ireplace( "%VideoUrl%", $videoInfo['url'], $template);
		$template = str_ireplace( "%VideoImage%", $videoInfo['img'], $template);
		$template = str_ireplace( "%VideoEmbed%", $videoInfo['videoEmbed'], $template);

		$template = str_ireplace( "%VideoViews%", $videoInfo['viewCount'], $template);
		$template = str_ireplace( "%VideoLikes%", $videoInfo['likeCount'], $template);
		$template = str_ireplace( "%VideoDislikes%", $videoInfo['dislikeCount'], $template);
		$template = str_ireplace( "%VideoFavorites%", $videoInfo['favoriteCount'], $template);
		$template = str_ireplace( "%VideoPublish%", $videoInfo['publishedAt'], $template);
		// support for publishedAt in ISO 8601 format
		if ( strpos( $template, '%VideoPublish8601%' ) !== false && ! empty( $videoInfo['publishedAt'] ) ) {
			try {
				$date = new DateTime( $videoInfo['publishedAt'] );
			} catch ( Exception $e ) {
    				$this->info_message( __FUNCTION__ . " Datetime from Publish8601 - " . $e->getMessage() );
			}
			$videoInfo['publishedAt8601'] = isset( $date ) ? $date->format( 'c' ) : '';
			$template = str_ireplace( "%VideoPublish8601%", $videoInfo['publishedAt8601'], $template);
		}

		if ( isset( $videoInfo['file'] ) ) { // not supported on all video sites
			$template = str_ireplace( "%VideoFile%", $videoInfo['file'], $template);
		}

    		// modified by ryan yang
    		if ( true === $spintax ) {
      			$template = $this->spintax( $template );
    		}
		
		return $template;
	}


	public function extend_execution_time() {
		// first, check our script execution time
		if ( ! ini_get( 'safe_mode' ) ) {	// safe mode off, we can extend script time as needed
			if ( set_time_limit( 30 ) ) { // extend max time by 30 sec
				$this->info_message( sprintf( __( 'Extending PHP max script time by 30 seconds.', 'autovid-profit' ) ), 'updated', 'debug' );
			}
			else {
				$this->info_message( sprintf( __( 'Unable to extend PHP max script time. Probably denied by server configuration.', 'autovid-profit' ) ) );
			}
		}

	}

	/*
	 * Process the video info depending on video campaign settings
	 */
	public function process_query_results( $args, &$videoInfo ) {
		$this->info_message( sprintf( __( '%s - Begin - %s', 'autovid-profit' ), __FUNCTION__, htmlentities( print_r($videoInfo, true) ) ), 'updated', 'debug' );

		$this->extend_execution_time();

		// some APIs don't supply these fields so set default 0 to avoid warnings
		$videoInfo['likeCount'] = isset( $videoInfo['likeCount'] ) ? $videoInfo['likeCount'] : 0;
		$videoInfo['dislikeCount'] = isset( $videoInfo['dislikeCount'] ) ? $videoInfo['dislikeCount'] : 0;
		$videoInfo['favoriteCount'] = isset( $videoInfo['favoriteCount'] ) ? $videoInfo['favoriteCount'] : 0;
		$videoInfo['viewCount'] = isset( $videoInfo['viewCount'] ) ? $videoInfo['viewCount'] : 0;

		$updateID = isset( $videoInfo['postID'] ) ? $videoInfo['postID'] : 0;

		if ( ! $updateID ) { // for new videos, check skip rules first

		if ( $this->post_already_exists( $videoInfo, 'blogster' ) ) {
			return $this->info_message( sprintf( __( '%s video [%s] already imported. Skipping.', 'autovid-profit' ), $videoInfo['videoSource'], $videoInfo['title'] ), 'updated', 'video_skip' );
		}
		else if ( $this->post_already_exists( $videoInfo, 'mvb' ) ) {
			return $this->info_message( sprintf( __( '%s video [%s] already imported by MVB. Skipping.', 'autovid-profit' ), $videoInfo['videoSource'], $videoInfo['title'] ), 'updated', 'video_skip' );
		}
		else if ( $this->post_already_exists( $videoInfo, 'cbc' ) ) {
			return $this->info_message( sprintf( __( '%s video [%s] already imported by YouTube Video to Post. Skipping.', 'autovid-profit' ), $videoInfo['videoSource'], $videoInfo['title'] ), 'updated', 'video_skip' );
		}
		else if ( $this->post_already_exists( $videoInfo, 'filter' ) ) {
			return $this->info_message( sprintf( __( '%s video [%s] already imported by [user defined filter]. Skipping.', 'autovid-profit' ), $videoInfo['videoSource'],  $videoInfo['title'] ), 'updated', 'video_skip' );
		}
		else if ( $this->video_on_blacklist( $videoInfo['videoID'] ) ) {
			return $this->info_message( sprintf( __( '%s video ID [%s] is on the blacklist. Skipping.', 'autovid-profit' ), $videoInfo['videoSource'], $videoInfo['videoID'] ), 'updated', 'video_skip' );
		}

		if ( $args['pStrictTitle'] ) {
			$stricts = explode( ",", $args['pStrictTitle'] );
			foreach ( $stricts as $keyphrase ) {
				$keyphrase = trim( $keyphrase );
				$found = stripos( $videoInfo['title'], $keyphrase );
				if ( FALSE !== $found ) {
					break;
				}
				$found = stripos( $videoInfo['desc'], $keyphrase );
				if ( FALSE !== $found ) {
					break;
				}
				$found = stripos( $videoInfo['association'], $keyphrase );
				if ( FALSE !== $found ) {
					break;
				}
			}
			if ( FALSE === $found ) {
				return $this->info_message( sprintf( __( 'None of the keyphrases [%s] was found in the video content for [%s]. Skipping.', 'autovid-profit' ), $args['pStrictTitle'], $videoInfo['title'] ), 'updated', 'video_skip' );
			}
		}

		if ( $args['pNegateTitle'] ) {
			$stricts = explode( ",", $args['pNegateTitle'] );
			foreach ( $stricts as $keyphrase ) {
				$keyphrase = trim( $keyphrase );
				$found = stripos( $videoInfo['title'], $keyphrase );
				if ( FALSE !== $found ) {
					return $this->info_message( sprintf( __( 'Keyphrase [%s] was found in the title for [%s]. Skipping.', 'autovid-profit' ), $keyphrase, $videoInfo['title'] ), 'updated', 'video_skip' );
				}
				$found = stripos( $videoInfo['desc'], $keyphrase );
				if ( FALSE !== $found ) {
					return $this->info_message( sprintf( __( 'Keyphrase [%s] was found in the description for [%s]. Skipping.', 'autovid-profit' ), $keyphrase, $videoInfo['title'] ), 'updated', 'video_skip' );
				}
				$found = stripos( $videoInfo['association'], $keyphrase );
				if ( FALSE !== $found ) {
					return $this->info_message( sprintf( __( 'Keyphrase [%s] was found in the association for [%s]. Skipping.', 'autovid-profit' ), $keyphrase, $videoInfo['title'] ), 'updated', 'video_skip' );
				}
			}
		}

		if ( $args['pMinViews'] && (int) $videoInfo['viewCount'] < $args['pMinViews'] ) {
			return $this->info_message( sprintf( __( '%s video [%s] has %d views, less than the required %d minimum views. Skipping.', 'autovid-profit' ), $videoInfo['videoSource'], $videoInfo['title'], $videoInfo['viewCount'], $args['pMinViews'] ), 'updated', 'video_skip' );
		}

		if ( $args['pCutoffDate'] ) {
			$viddate = new DateTime( $videoInfo['publishedAt'] );
			try {
				$cutoff = new DateTime( $args['pCutoffDate'] );
			} catch ( Exception $e ) {
    				$this->info_message( __FUNCTION__ . " DateTime from pCutoffDate - " . $e->getMessage() );
			}
			if ( ! empty( $cutoff ) && $cutoff > $viddate ) {
				return $this->info_message( sprintf( __( '%s video [%s] date %s older than %s (%s). Skipping.', 'autovid-profit' ), $videoInfo['videoSource'], $videoInfo['title'], $viddate->format( 'Y-m-d H:i:s' ), $args['pCutoffDate'], $cutoff->format( 'Y-m-d H:i:s' ) ), 'updated', 'video_skip' );
			}
		}

		if ( ! empty( $videoInfo['duration'] ) && ( ! empty( $args['pDurationMin'] ) || ! empty( $args['pDurationMax'] ) ) ) {
			$vMin = $args['pDurationMin'];
			$vMax = $args['pDurationMax'];
			sscanf($videoInfo['duration'], "%d:%d:%d", $hours, $minutes, $seconds);
			$dSecs = isset($seconds) ? $hours * 3600 + $minutes * 60 + $seconds : $hours * 60 + $minutes;
			$vidLT = $vMin * 60;
			$vidGT = $vMax * 60;
			if ( $vidLT && $dSecs < $vidLT ) {
				return $this->info_message( sprintf( __( '%s video [%s] duration %s less than %s minutes. Skipping.', 'autovid-profit' ), $videoInfo['videoSource'], $videoInfo['title'], $videoInfo['duration'], $vMin ), 'updated', 'video_skip' );
			}
			else if ( $vidGT && $dSecs > $vidGT ) {
				return $this->info_message( sprintf( __( '%s video [%s] duration %s greater than %s minutes. Skipping.', 'autovid-profit' ), $videoInfo['videoSource'], $videoInfo['title'], $videoInfo['duration'], $vMax ), 'updated', 'video_skip' );
			}
		}



		if ( $args['pNoThumbnails'] == true && empty( $videoInfo['img'] ) ) {
			return $this->info_message( sprintf( __( '%s video [%s] did not have any valid thumbnails. Skipping.', 'autovid-profit' ), $videoInfo['videoSource'], $videoInfo['title'] ), 'updated', 'video_skip' );
		}

		} // end skip rules

		if ( $args['pTranslateTo'] ) {
		  $apiKey = get_option( $this->get_my_plugin( 'prefix' ) . 'gtranslate_key' );
		  if ( empty( $apiKey ) ) {
			$this->info_message( sprintf( __( 'Unable to Google translate - missing apiKey', 'autovid-profit' ) ) );
		  }
		  if ( ! empty( $apiKey ) && ! empty( $videoInfo['title'] ) ) {
			$data = $this->googleTranslate( $videoInfo['title'], $args['pTranslateTo'], $apiKey );
			if ( isset( $data->data->translations[0]->translatedText ) ) {
				$translate = $data->data->translations[0]->translatedText;
				$videoInfo['title'] = $translate;
				$this->info_message( sprintf( __( '%s video title %s has been Google (%s) translated to %s.', 'autovid-profit' ), $videoInfo['videoSource'], $videoInfo['orig_title'], $args['pTranslateTo'], $translate ), 'updated', 'video_import' );
			}
			else if ( isset( $data->error->errors[0]->reason ) ) {
				$this->info_message( sprintf( __( 'Unable to Google translate video title to %s. Reason: %s ', 'autovid-profit' ), $args['pTranslateTo'], $data->error->errors[0]->reason ) );
			}
			else {
				$this->info_message( sprintf( __( 'Unable to Google translate video title to %s. Reason unknown', 'autovid-profit' ), $args['pTranslateTo'] ) );
			}
		  }
		  if ( ! empty( $apiKey ) && ! empty( $videoInfo['desc'] ) ) {
			$data = $this->googleTranslate( $videoInfo['desc'], $args['pTranslateTo'], $apiKey );
			if ( isset( $data->data->translations[0]->translatedText ) ) {
				$translate = $data->data->translations[0]->translatedText;
				$videoInfo['desc'] = $translate;
				$this->info_message( sprintf( __( '%s video description has been Google (%s) translated to %s.', 'autovid-profit' ), $videoInfo['videoSource'], $args['pTranslateTo'], htmlentities( $translate ) ), 'updated', 'video_import' );
			}
			else if ( isset( $data->error->errors[0]->reason ) ) {
				$this->info_message( sprintf( __( 'Unable to Google translate video description to %s. Reason: %s ', 'autovid-profit' ), $args['pTranslateTo'], $data->error->errors[0]->reason ) );
			}
			else {
				$this->info_message( sprintf( __( 'Unable to translate video description to %s. Reason unknown.', 'autovid-profit' ), $args['pTranslateTo'] ) );
			}
		  }
		}

		// check duration field to make sure it's a format DateTime will recognize, like mm:ss or hh:mm:ss
		// xVideos uses format 124:21 instead of 02:04:21
		if ( preg_match( "/^([0-9]+):([0-9]+)(:[0-9]+)?$/", $videoInfo['duration'], $matches ) ) {
			$cnt = count( $matches );
			if ( $cnt == 3 && $matches[1] > 60 ) { // mm:ss with mm > 60
				$redo = $matches[1] * 60 + $matches[2];
				$v = $videoInfo['duration'];
				$videoInfo['duration'] = sprintf( '%02d:%02d:%02d', ($redo/3600),($redo/60%60), $redo%60 );
				$this->info_message( sprintf( __( 'Note: converting video duration from %s to %s', 'autovid-profit' ), $v, $videoInfo['duration'] ), 'updated', 'debug' );
			}
		}

		if ( TRUE == $args['pRemoveDescUrls'] ) {
			$videoInfo['desc'] = $this->remove_urls( $videoInfo['desc'] );
		}
		if ( $args['pLimitTitleChars'] && ( strlen( $videoInfo['title'] ) > $args['pLimitTitleChars'] ) ) {
			$last_space = strrpos(substr($videoInfo['title'], 0, $args['pLimitTitleChars']), ' ');
			$videoInfo['title'] = substr($videoInfo['title'], 0, $last_space);
		}
		if ( $args['pLimitTitleWord'] ) {
			$found = stripos( $videoInfo['title'], $args['pLimitTitleWord'] );
			if ( $found !== FALSE ) {
				$videoInfo['title'] = substr( $videoInfo['title'], 0, $found );
			}
		}

		if ( $args['pRemoveTitleChars'] ) {
			$str = $args['pRemoveTitleChars'];
/* MARK needed? 
*&^%$#@!-",'~    is fine but
*&^%$#@!-",'~    	gives strange results
			$str = iconv( "UTF-8", "ISO-8859-1//TRANSLIT", $str );
*/
			$str = preg_quote( $str );
			$newtitle = preg_replace("![" . $str . "]!", '', $videoInfo['title'] );
			$newtitle = trim( $newtitle );
			$newtitle = preg_replace( '/\s+/', ' ', $newtitle );
			$videoInfo['title'] = $newtitle;
		}

		// break on the word the limit falls on
		if ( $args['pLimitDescChars'] && ( strlen( $videoInfo['desc'] ) > $args['pLimitDescChars'] ) ) {
			$last_space = strrpos(substr($videoInfo['desc'], 0, $args['pLimitDescChars']), ' ');
			$videoInfo['desc'] = substr($videoInfo['desc'], 0, $last_space);
		}
		// the phrase to end at IS NOT included.
		if ( $args['pLimitDescWord'] ) {
			$found = stripos( $videoInfo['desc'], $args['pLimitDescWord'] );
			if ( $found !== FALSE ) {
				// to include phrase:
				// $found += strlen( $args['pLimitDescWord'] );
				$videoInfo['desc'] = substr( $videoInfo['desc'], 0, $found);
			}
		}

		if ( $args['pSentenceRemove'] || $args['pSentenceRemovePhrase'] ) {
			$re = '/# Split sentences on whitespace between them.
    				(?<=                # Begin positive lookbehind.
      				[.!?]               # Either an end of sentence punct
    				| [.!?][\'"]        # or end of sentence punct and quote.
				| \n                # or a line break,
    				)                   # End positive lookbehind.
    				(?<!                # Begin negative lookbehind.
      				Mr\.                # Skip either "Mr."
    				| Mrs\.             # or "Mrs.",
    				| Ms\.              # or "Ms.",
    				| Jr\.              # or "Jr.",
    				| Dr\.              # or "Dr.",
    				| Prof\.            # or "Prof.",
    				| Sr\.              # or "Sr.",
				| \s\w\.            # an initial like in "First M. Last"
				| \s\w\.\w\.        # double initial like in "F.M. Last"
				| \s\w.\w\.\w\.     # triple initial like in "F.M.L."
    				)                   # End negative lookbehind.
    				(\s+)               # Split on whitespace between sentences.
    				/mix';

			$sentences = preg_split( $re, $videoInfo['desc'], -1, PREG_SPLIT_DELIM_CAPTURE );
			$remove_phrases = explode( ',', $args['pSentenceRemovePhrase'] );
			$remove_phrases = array_map( 'trim', $remove_phrases);
			for ( $i=$j=0; $i < count( $sentences ); ++$i ) {
				$check = trim( $sentences[$i] );
				if ( empty( $check ) ) {
					continue;
				}
				$j++;
				$this->info_message( sprintf( __( 'Sentence %d check: %s', 'autovid-profit' ), $j, $check ), 'updated', 'debug' );
				if ( $j <= $args['pSentenceRemove'] ) {
					$this->info_message( sprintf( __( 'pSentenceRemove <= %d is true. Sentence removed.', 'autovid-profit' ), $args['pSentenceRemove'] ), 'updated', 'debug' );
					$sentences[$i] = "";
					continue;
				}
				foreach( $remove_phrases as $rp ) {
					if ( stripos( $sentences[$i], $rp ) !== FALSE ) {
						$this->info_message( sprintf( __( 'pRemovePhrase for [%s] is true. Sentence removed.', 'autovid-profit' ), $rp ), 'updated', 'debug' );
						$sentences[$i] = " ";
						break;
					}
				}
			}
			$videoInfo['desc'] = implode( $sentences );
		}

		if ( $args['pReplacePhrases'] != null ) {
			// replace phrases with spintax which will be processed below in expand_template()
			$rules = explode( "\n", $args['pReplacePhrases'] );
			foreach ( $rules as $rule ) {
				$this->info_message( sprintf( __( 'applying rule: %s', 'autovid-profit' ), $rule ), 'updated', 'debug' );
				if ( $rule == null ) {
					continue;
				}
				if ( strpos( $rule, '::') === FALSE ) {
					$this->info_message( sprintf( __( "Error with replace phrase rule: [%s] missing the '::' marker. Rule Skipped.", 'autovid-profit' ), $rule ) );
					continue;
				}
				// let's be very very careful...
				$spinrule = explode( "::", $rule );
				// first check video title
				$result = preg_replace( '~(?<!\pL)' . preg_quote( $spinrule[0] ) . '(?!\pL)~u', $spinrule[1], $videoInfo['title'] );
				if ( $result ) {
					$videoInfo['title'] = $result;
				}
				// check video desc
				$result = preg_replace( '~(?<!\pL)' . preg_quote( $spinrule[0] ) . '(?!\pL)~u', $spinrule[1], $videoInfo['desc'] );
				if ( $result ) {
					$videoInfo['desc'] = $result;
				}
				// check title template
				$result = preg_replace( '~(?<!\pL)' . preg_quote( $spinrule[0] ) . '(?!\pL)~u', $spinrule[1], $args['pTitleTemplate'] );
				if ( $result ) {
					$args['pTitleTemplate'] = $result;
				}
				// check post template
				$result = preg_replace( '~(?<!\pL)' . preg_quote( $spinrule[0] ) . '(?!\pL)~u', $spinrule[1], $args['pPostTemplate'] );
				if ( $result ) {
					$args['pPostTemplate'] = $result;
				}
			}
		}

		if ( TRUE == $args['pLinkifyDescUrls'] ) {
			$videoInfo['desc'] = $this->linkify_filtered( $videoInfo['desc'], $args['pLinkifyTypes'] );
		}


		if ( ! empty ( $args['pSpinner'] ) ) {
			$this->spin_text( $args, $videoInfo['title'], $videoInfo['desc'] );
		}

		$videoInfo['post_title'] = $this->expand_template( $args['pTitleTemplate'], $videoInfo );

		if ( $args['pSkipTitle'] ) {
			if ( $this->post_title_already_exists( $videoInfo['post_title'] ) ) {
				return $this->info_message( sprintf( __( 'New video post title [%s] already exists for another post. Skipping.', 'autovid-profit' ), $videoInfo['title'] ), 'updated', 'video_skip' );
			}
			else {
				$this->info_message( sprintf( __( '%s video title [%s] is unique.', 'autovid-profit' ), $videoInfo['videoSource'], $videoInfo['title'] ), 'updated', 'video_import' );
			}
		}

		$videoInfo['post_content'] = $this->expand_template( $args['pPostTemplate'], $videoInfo );

		if ( TRUE == $args['pCreateExcerpt'] ) {
			$excerpt = $videoInfo['post_content'];
			$excerpt = strip_shortcodes( $excerpt );
			$excerpt = apply_filters( 'the_content', $excerpt );
			if ( $args['pExcerptLength'] ) {
				$videoInfo['post_excerpt'] = wp_trim_words( $excerpt, $args['pExcerptLength'] );
			}
			else {
				$videoInfo['post_excerpt'] = wp_trim_words( $excerpt );
			}
		}
		if ( isset( $videoInfo['tags'] ) && $args['pMinTagLength'] ) {
			$tags = explode( ',', $videoInfo['tags'] );
			foreach( $tags as $key => $value ) {
				if ( strlen( trim( $value ) ) < $args['pMinTagLength'] ) {
					unset( $tags[$key] );
				}
			}
			$videoInfo['tags'] = implode( ",", $tags );
		}
		if ( isset( $videoInfo['tags'] ) && $args['pIgnoreTags'] ) {
			$tags = explode( ',', $videoInfo['tags'] );
			$tags = array_map( 'trim', $tags );
			$ignore_tags = explode( ',', $args['pIgnoreTags'] );
			$ignore_tags = array_map( 'trim', $ignore_tags );
			$diff = array_udiff( $tags, $ignore_tags, 'strcasecmp' );
			$videoInfo['tags'] = implode( ',', $diff );
		}
		if ( $args['pAddTags'] ) {
			$videoInfo['tags'] .= "," . $args['pAddTags'];
		}
		if ( $args['pImageTemplate'] ) { // custom image name
			// just in case custom filename has a directory or an extension - strip it out
			$args['pImageTemplate'] = pathinfo( $args['pImageTemplate'], PATHINFO_FILENAME ); 
			$img = strtok( $videoInfo['img'] );
			// make sure it has same extension as downloaded image:
			$ext = pathinfo( $img, PATHINFO_EXTENSION); 
			if ( ! $ext ) { // Hulu, Spotify images have no extension
				$ext = "jpg";
			}
			$args['pImageTemplate'] .= "." . $ext;
			$videoInfo['custom_name'] = $this->expand_template( $args['pImageTemplate'], $videoInfo );
		}

		$videoInfo = apply_filters( 'vb_process_query', $videoInfo );

		$this->info_message( sprintf( __( '%s - End - %s', 'autovid-profit' ), __FUNCTION__, htmlentities( print_r($videoInfo, true) ) ), 'updated', 'debug' );

	return 1;
	}

	/*
	 * Create the post with the video info using the video campaign settings
	 * if postID/updateID is not 0, then it is an update 
	 */
	public function create_the_post( $args, $videoInfo ) {
		if ( empty( $videoInfo ) ) {
			return new WP_Error('videoInfo is empty - contact support');
		}
		$videoInfo['campaignID'] = $args['id'];
		$this->info_message( sprintf( __( '%s - videoInfo: %s', 'autovid-profit' ), __FUNCTION__, htmlentities( print_r( $videoInfo, TRUE ) ) ), 'updated', 'debug' );
		$this->info_message( sprintf( __( '%s - campaign details: %s', 'autovid-profit' ), __FUNCTION__, htmlentities( print_r( $args, TRUE ) ) ), 'updated', 'debug' );
		$updateID = isset( $videoInfo['postID'] ) ? $videoInfo['postID'] : 0;
		$user = $args['cUser'];
		if ( $args['cUser'] == "(random)" ) {
			$users = $this->get_users();
			shuffle( $users );
			$user = $users[0]->ID;
		}
		if ( empty( $args['cPostStatus'] ) ) {
			$args['cPostStatus'] = 'draft';
		}
		if ( empty( $args['cPostType'] ) ) {
			$args['cPostType'] = 'post';
		}
		if ( empty( $args['cPostFormat'] ) ) {
			$args['cPostFormat'] = 'standard';
		}


		// don't publish on post creation - wait for featured image to be set, then publish
		$post_status = ( $args['cPostStatus'] == 'publish' ) ? 'draft' : $args['cPostStatus'];

		// is there a source category we have to add as well?
		$args['cCategories'] .= isset( $videoInfo['sourceCategory'] ) && ! empty( $videoInfo['sourceCategory'] ) ? ',' . $videoInfo['sourceCategory'] : null;

		$tags = apply_filters( 'vb_save_tags', $videoInfo['tags'] );

		$postID = 0;
		kses_remove_filters();
		if ( $updateID ) {	// update an existing post - only happens when processing retroactively is checked
                	$vidpost = array(
				'ID'     	=> $updateID,
				'post_name'     => sanitize_title( $videoInfo['post_title'] ),
				'post_title'    => $videoInfo['post_title'],
				'post_content'  => $videoInfo['post_content'],
				'post_type'     => $args['cPostType'],
				'post_excerpt'  => TRUE == $args['pCreateExcerpt'] ? $videoInfo['post_excerpt'] : '',
				'post_category' => empty( $args['cCategories'] ) ? null : explode( ",", $args['cCategories'] ),
				'tags_input'    => $tags,
				'filter' => true 
			);
			$vidpost = apply_filters( 'vb_update_the_post', $vidpost, $videoInfo );
			if ( ! is_numeric( $vidpost ) && ! empty( $vidpost ) ) {
				$this->info_message( sprintf( __( '%s - wp_update_post: %s', 'autovid-profit' ), __FUNCTION__, htmlentities( print_r( $vidpost, TRUE ) ) ), 'updated', 'debug' );
				$postID = wp_update_post( $vidpost, TRUE );
			}
		}
		else {			// add a new post
			$vidpost = array(
				'post_name'     => sanitize_title( $videoInfo['post_title'] ),
				'post_title'    => $videoInfo['post_title'],
				'post_content'  => $videoInfo['post_content'],
				'post_status'   => $post_status,
				'post_author' 	=> $user,
				'post_type'     => $args['cPostType'],
				'post_excerpt'  => TRUE == $args['pCreateExcerpt'] ? $videoInfo['post_excerpt'] : '',
				'post_date'	=> TRUE == $args['cUsePublishedDate'] ? $videoInfo['publishedAt'] : current_time( 'mysql' ),
				'post_date_gmt'	=> TRUE == $args['cUsePublishedDate'] ? get_gmt_from_date( $videoInfo['publishedAt'] ) : get_gmt_from_date( current_time( 'mysql' ) ),
				'post_category' => empty( $args['cCategories'] ) ? null : explode( ",", $args['cCategories'] ),
				'tags_input'    => $tags,
				'filter' => true 
				);
			$vidpost = apply_filters('vb_create_the_post', $vidpost, $videoInfo );

			// if above filter returns the data array, VB will create the post and all attachment data
			// if above filter returns a post ID, VB will not create the post but still attach data
			// if above filter returns null, VB will not create the post or attachment data
			if ( ! is_numeric( $vidpost ) && ! empty( $vidpost ) ) {
				$this->info_message( sprintf( __( '%s - wp_insert_post: %s', 'autovid-profit' ), __FUNCTION__, htmlentities( print_r( $vidpost, TRUE ) ) ), 'updated', 'debug' );
				$postID = wp_insert_post( $vidpost, TRUE );
				// check if post needs to be spinned:
				if ( ! empty ( $args['pSpinnerPost'] ) ) {
					$this->spin_post( $args, $postID );
				}
			}
		}

		kses_init_filters();

		if ( is_numeric( $vidpost ) ) {
			// external code created the post through a filter 
			// and returned the post id for VB to attach meta, thumbnail, comments, etc
			$this->info_message( sprintf( __( '%s - vb filter returned: %s', 'autovid-profit' ), __FUNCTION__, htmlentities( print_r( $vidpost, TRUE ) ) ), 'updated', 'debug' );
			$postID = $vidpost;
		}
		else if ( empty( $vidpost ) ) { 
			// external code created the post through a filter and they don't want further processing
			$this->info_message( sprintf( __( '%s - vb filter returned null. Skipping.', 'autovid-profit' ), __FUNCTION__, htmlentities( print_r( $vidpost, TRUE ) ) ), 'updated', 'debug' );
			return new WP_Error('skip_post_creation');
		}

		$this->info_message( sprintf( __( '%s - update/insert finished successfully, postID %d', 'autovid-profit' ), __FUNCTION__, $postID ), 'updated', 'debug' );


		if ( is_wp_error( $postID ) ) {
			if ( $updateID ) {
				$this->info_message( sprintf( __( 'Error %s: wp_update_post returned - %s', 'autovid-profit' ), $postID->get_error_code(), $postID->get_error_message() ) );
			}
			else {
				$this->info_message( sprintf( __( 'Error %s: wp_insert_post returned - %s', 'autovid-profit' ), $postID->get_error_code(), $postID->get_error_message() ) );
			}
			return $postID;
		}

		if ( ! $updateID ) { // only add_post_meta for Autovid Profit data if inserting a new post
			$this->info_message( sprintf( __( '%s - adding post meta to postID %d', 'autovid-profit' ), __FUNCTION__, $postID ), 'updated', 'debug' );
			add_post_meta( $postID, 'VideoSource', $videoInfo['videoSource'] );
			add_post_meta( $postID, 'VideoID', $videoInfo['videoID'] );
			add_post_meta( $postID, 'VideoDuration', $videoInfo['duration'] );
			add_post_meta( $postID, 'VideoLikes', $videoInfo['likeCount'] );
			add_post_meta( $postID, 'VideoDislikes', $videoInfo['dislikeCount'] );
			add_post_meta( $postID, 'VideoFavorites', $videoInfo['favoriteCount'] );

			add_post_meta( $postID, 'VideoCampaign', $args['id'] );
			add_post_meta( $postID, 'VideoOrigTitle', $videoInfo['orig_title'] );
			add_post_meta( $postID, 'VideoOrigDesc', $videoInfo['orig_desc'] );
			add_post_meta( $postID, 'VideoAssoc', $videoInfo['association'] );
			add_post_meta( $postID, 'VideoChannelId', isset( $videoInfo['channelID'] ) ? $videoInfo['channelID'] : '' );
			add_post_meta( $postID, 'VideoImage', $videoInfo['img'] );
			add_post_meta( $postID, 'VideoUrl', $videoInfo['url'] );
			add_post_meta( $postID, 'VideoEmbed', $videoInfo['videoEmbed'] );
			add_post_meta( $postID, 'VideoPublish', $videoInfo['publishedAt'] );

			if ( isset( $videoInfo['file'] ) ) { // direct link to video file. Not all sites support.
				add_post_meta( $postID, 'VideoFile', $videoInfo['file'] );
			}

			// use compatible name with WP-PostViews. easy.
			add_post_meta( $postID, 'views', $videoInfo['viewCount'], true ) || update_post_meta( $postID, 'views', $videoInfo['viewCount'] );

			// make compatible with WP-PostRatings. a little more difficult
			if ( defined( 'WP_POSTRATINGS_VERSION' ) ) {
				$postratings_max = intval( get_option( 'postratings_max' ) );
			}
			else {
				$postratings_max = 5; //default
			}
			$post_ratings_users = $post_ratings_score = $post_ratings_average = 0;

			if ( isset( $videoInfo['likeCount'] ) ) { // convert stats to WP-PostRatings
				$post_ratings_users = $videoInfo['likeCount'] + $videoInfo['dislikeCount'];
				$post_ratings_score = $videoInfo['likeCount'] * $postratings_max;
				$post_ratings_average = $post_ratings_users ? round($post_ratings_score/$post_ratings_users, 2) : 0;
			}

			add_post_meta( $postID, 'ratings_users', $post_ratings_users, true ) || update_post_meta( $postID, 'ratings_users', $post_ratings_users );
			add_post_meta( $postID, 'ratings_score', $post_ratings_score, true ) || update_post_meta( $postID, 'ratings_score', $post_ratings_score );
			add_post_meta( $postID, 'ratings_average', $post_ratings_average, true ) || update_post_meta( $postID, 'ratings_average', $post_ratings_average );

			// now we can attach taxonomy terms:
			if ( ! empty( $args['cTaxTerms'] ) && is_array( $args['cTaxTerms'] ) ) {
				foreach ( $args['cTaxTerms'] as $taxterm ) {
					// have to gather all terms first then add
					$parts = explode( ':', $taxterm ); // in format 'taxonomy:term:slug'
					$cTaxTerms[$parts[0]][] = $parts[2];
				}
				foreach ( $cTaxTerms as $tax => $terms ) {
					$t = implode( ',', $terms );
					$this->info_message( sprintf( __( 'Attaching taxonomy [%s] terms [%s] to post id %s', 'autovid-profit' ), $tax, $t, $postID ), 'updated', 'debug' );
					wp_set_object_terms( $postID, $terms, $tax );
				}
			}
		}
		else {	// updated an existing post - Autovid Profit meta data already set when post created
		}

		if ( $this->is_debug_set() ) {
			$doublecheck = get_post( $postID );
			$this->info_message( sprintf( __( '%s - post id %s now has: %s', 'autovid-profit' ), __FUNCTION__, $postID, htmlentities( print_r( $doublecheck,true ) ) ), 'updated', 'debug' );
		}

		// add or update any CUSTOM meta fields:
		if ( isset( $args['pMetaFields'] ) && is_array( $args['pMetaFields'] ) ) {
			foreach ( $args['pMetaFields'] as $metakey => $metavalue ) {

				// modified by ryan yang
				$metavalue = $this->expand_template( $metavalue, $videoInfo, false );
				if ( json_decode( $metavalue ) == true ) {
					$metavalue = json_decode( $metavalue, true );
				}

				add_post_meta( $postID, $metakey, $metavalue, true ) || update_post_meta( $postID, $metakey, $metavalue );
				$this->info_message( sprintf( __( 'Attaching custom field [%s] with value [%s] to post id %s', 'autovid-profit' ), $metakey, htmlentities( print_r( $metavalue,true ) ), $postID ), 'updated', 'debug' );

			}
		}

		// finally, set the post format
		if ( $args['cPostFormat'] ) {
			$this->info_message( sprintf( __( 'Setting post format to [%s]', 'autovid-profit' ), $args['cPostFormat'] ), 'updated', 'debug' );
			set_post_format( $postID, $args['cPostFormat'] );
		}

		apply_filters('vb_create_the_post_finished', $postID, $videoInfo );

		return $postID;
	}

	/*
	 * This function is called if video campaign is set to publish posts.
	 * We can't publish when we insert the post because filters would fire before thumbnail/comments were attached.
	 * So we need this function to update the post to publish with video publish date change (if specified).
	 */
	public function publish_the_post( $postID, $args, $videoInfo ) {
		$vidpost = array(
			'ID'		=> $postID,
			'post_status'	=> 'publish',
			'edit_date'	=> true,
			'post_date'     => TRUE == $args['cUsePublishedDate'] ? $videoInfo['publishedAt'] : current_time( 'mysql' ),
		);
		$this->info_message( sprintf( __( '%s - vidpost: %s', 'autovid-profit' ), __FUNCTION__, htmlentities( print_r( $vidpost, TRUE ) ) ), 'updated', 'debug' );
		kses_remove_filters();
		$postID = wp_update_post( $vidpost, TRUE );
		kses_init_filters();
		if ( is_wp_error( $postID ) ) {
			$this->info_message( sprintf( __( 'Error %s: wp_update_post returned - %s', 'autovid-profit' ), $postID->get_error_code(), $postID->get_error_message() ) );
		}
		else {
			$this->info_message( sprintf( __( 'Post: [%s] published successfully with post id <a target="_blank" href="%s">%s</a>.', 'autovid-profit' ), $videoInfo['post_title'], get_post_permalink( $postID ), $postID ), 'updated', 'video_import' );
		}
	}


	/*
	 * Expands post template with the thumbnail if %VideoImageLocal% tag is used
	 * Have to do this AFTER post was created so thumb could be fetched and attached.
	 */
	public function process_the_thumbnail ( $postID, $thumbID, $videoInfo ) {
		$this->info_message( sprintf( __( '%s for postID %s, thumbID %s, videoInfo: %s', 'autovid-profit' ), __FUNCTION__, $postID, $thumbID, htmlentities( print_r( $videoInfo, TRUE ) ) ), 'updated', 'debug' );
		if ( ! $thumbID ) { // no thumb to process
			return $postID;
		}
		// if %VideoImageLocal% tag is in post template we need to update_post we just created.
		$thumburl = wp_get_attachment_url( $thumbID );
		$videoInfo['post_content'] = str_ireplace( '%VideoImageLocal%', $thumburl, $videoInfo['post_content'] );
		if ( isset( $videoInfo['post_excerpt'] ) ) {
			$videoInfo['post_excerpt'] = str_ireplace( '%VideoImageLocal%', $thumburl, $videoInfo['post_excerpt'] );
		}
		$vidpost = array(
			'post_content'  => $videoInfo['post_content'],
			'post_excerpt'  => isset( $videoInfo['post_excerpt'] ) ? $videoInfo['post_excerpt'] : '',
			'ID'		=> $postID
			);
		kses_remove_filters();
		$postID = wp_update_post( $vidpost, TRUE );
		kses_init_filters();
		if ( is_wp_error( $postID ) ) {
			$this->info_message( sprintf( __( 'Error %s: wp_update_post returned - %s', 'autovid-profit' ), $postID->get_error_code(), $postID->get_error_message() ) );
		}
		return $postID;
	}

	/*
	 * Takes an ordered array of video image urls from the API in cascading order.
	 * Returns the first verified video image url that returns a 200 response code.
	 * ( This is used because sometimes YouTube specifies an image url that does not exist. )
	 */
	public function verify_thumbnails( $imgs ) {
		$this->info_message( sprintf( __( '%s: %s', 'autovid-profit' ), __FUNCTION__, print_r($imgs,true) ), 'updated', 'debug' );
		// in case user wants to save all possible imgs somewhere:
		$imgs = apply_filters( 'vb_verify_thumbnails', $imgs );
		foreach ( $imgs as $img ) {

			if ( false !== strpos( $img, 'video_converting' ) ) {
				$this->info_message( sprintf( __( '%s: %s skipped', 'autovid-profit' ), __FUNCTION__, $img ), 'updated', 'debug' );
				continue;
			}

			$response = wp_remote_head( $img );
			if ( wp_remote_retrieve_response_code( $response ) == 200 ) {
				$cl = wp_remote_retrieve_header( $response, 'content-length' );
				if ( $cl === '' ) {
					// Hulu, Spotify, etc doesn't supply a content-length :( 
					$this->info_message( sprintf( __( '%s: %s no content length returned (assumed okay). : %s', 'autovid-profit' ), __FUNCTION__, $img, print_r($response,true) ), 'updated', 'debug' );
					return $img; 
				}
				else if ( $cl > 0 ) return $img; // a non-0 byte response means image has SOMETHING at least
				else $this->info_message( sprintf( __( '%s: %s CONTENT LENGTH is 0, response: %s', 'autovid-profit' ), __FUNCTION__, $img, print_r($response,true) ), 'error', 'debug' );
			}
			else $this->info_message( sprintf( __( '%s: %s did not return 200 OK: %s', 'autovid-profit' ), __FUNCTION__, $img, print_r($response,true) ), 'error', 'debug' );
		}
		$this->info_message( sprintf( __( '%s: none of these urls has valid thumbnails: %s', 'autovid-profit' ), __FUNCTION__, print_r($imgs,true) ), 'error', 'debug' );
		return null;
	}


	/*
	 * Fetches the thumbnail from the video site using wp_remote_get()
	 * Saves thumbnail in the media library using wp_upload_bits()
	 * Sets the featured image with set_post_thumbnail()
	 */
	public function grab_thumbnail2( $postID, $videoImage ) {
		$img = $videoImage['img'] ? strtok( $videoImage['img'], '?' ) : null;

		$post_title = $videoImage['post_title'] ? $videoImage['post_title'] : null;
		$custom_name = isset( $videoImage['custom_name'] ) ? $videoImage['custom_name'] : null;
		$this->info_message( sprintf( __( '%s fetching image: [%s] for %s, post id %s', 'autovid-profit' ), __FUNCTION__, htmlentities( $img ), $post_title, $postID ), 'updated', 'debug' );
		if ( ! $postID || ! $post_title || ! $img ) {
			return 0;
		}
		if ( has_post_thumbnail( $postID ) ) { // sanity check
			$this->info_message( sprintf( __( '%s: post %s already has thumbnail.', 'autovid-profit' ), __FUNCTION__, $postID ), 'updated', 'debug' );
		}

		$ext = pathinfo( $img, PATHINFO_EXTENSION );
		if ( ! $ext ) {	// handles dynamic images for example Hulu, Spotify - pain in the ass
			$ext = "jpg";
			$unsafe = TRUE;
			add_filter('http_request_reject_unsafe_urls', '__return_false');
		}

		$response = wp_remote_get( $img );
    		if ( is_wp_error( $response ) ) {	// this usually means the given thumb was not found on remote site
			return $this->info_message( sprintf( __( 'WP error %s in %s for wp_remote_get() of %s - %s', 'autovid-profit' ), $response->get_error_code(), __FUNCTION__, $img, $response->get_error_message() ) );
		}
		$img_bits = $response['body'];

		$filename = $custom_name ? $custom_name : 'vbp-' . $postID . '-' . $post_title . '.' . $ext;
		$filename = sanitize_file_name( $filename );
		// remove any unsafe characters:
		$filename = preg_replace( "/[^a-zA-Z0-9_\-.]/", '', $filename );

		$this->info_message( sprintf( __( '%s: about to upload bits for %s', 'autovid-profit' ), __FUNCTION__, $filename ), 'updated', 'debug' );
		$upload = wp_upload_bits( $filename, null, $img_bits );

		if ( isset( $unsafe ) ) {
			add_filter('http_request_reject_unsafe_urls', '__return_true');
		}

		if ( $upload['error'] ) {
			return $this->info_message( sprintf( __( 'wp_upload_bits error for %s: %s', 'autovid-profit' ), $filename, $upload['error'] ) );
		}

		$filetype = wp_check_filetype( basename( $upload['file'] ), null );

		$upload = apply_filters( 'wp_handle_upload', array(
			'file' => $upload['file'],
			'url'  => $upload['url'],
			'type' => $filetype['type']
		), 'sideload' );


		$attachment = array(
			'post_mime_type'        => $upload['type'],
			'post_title'            => get_the_title( $postID ),
			'post_content'          => '',
			'post_status'           => 'inherit'
		);

		$this->info_message( sprintf( __( '%s: about to insert attachments %s.', 'autovid-profit' ), __FUNCTION__, print_r( $attachment,true ) ), 'updated', 'debug' );

		$attach_id = wp_insert_attachment( $attachment, $upload['file'], $postID );

		// Make sure that this file is included, as wp_generate_attachment_metadata() depends on it.
		require_once( ABSPATH . 'wp-admin/includes/image.php' );

		// Generate the metadata for the attachment, and update the database record.
		$attach_data = wp_generate_attachment_metadata( $attach_id, $upload['file'] );
		wp_update_attachment_metadata( $attach_id, $attach_data );

		$this->info_message( sprintf( __( '%s: about to set_post_thumbnail for thumbID %s.', 'autovid-profit' ), __FUNCTION__, $attach_id ), 'updated', 'debug' );
		$result = set_post_thumbnail( $postID, $attach_id );
		if ( $result ) {
			$this->info_message( sprintf( __( 'Image [%s] added to media library for post id %s.', 'autovid-profit' ), $filename, $postID ), 'updated', 'debug' );
		}
		else {
			$this->info_message( sprintf( __( 'set_post_thumbnail error adding image [%s] to media library for post id %s.', 'autovid-profit' ), $filename, $postID ) );
		}
		return $attach_id;
	}

	/*
	 * Fetches the thumbnail from the video site using download_url()
	 * Saves thumbnail in the media library using media_handle_sideload()
	 * Sets the featured image with set_post_thumbnail()
	 */
	public function grab_thumbnail( $postID, $videoImage ) {
		$memthumb = false;
		$memthumb = apply_filters( 'vb_alternate_thumb_fetch', $memthumb );
		if ( $memthumb ) {
			return $this->grab_thumbnail2( $postID, $videoImage ); 
		}
		$img = $videoImage['img'] ? strtok( $videoImage['img'], '?' ) : null;

		$post_title = $videoImage['post_title'] ? $videoImage['post_title'] : null;
		$custom_name = isset( $videoImage['custom_name'] ) ? $videoImage['custom_name'] : null;
		$this->info_message( sprintf( __( '%s fetching image: [%s] for %s, post id %s', 'autovid-profit' ), __FUNCTION__, htmlentities( $img ), $post_title, $postID ), 'updated', 'debug' );
		if ( ! $postID || ! $post_title || ! $img ) {
			return 0;
		}
		if ( has_post_thumbnail( $postID ) ) { // sanity check
			$this->info_message( sprintf( __( '%s: post %s already has thumbnail.', 'autovid-profit' ), __FUNCTION__, $postID ), 'updated', 'debug' );
		}
		$this->info_message( sprintf( __( '%s: checking for required functions.', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'debug' );
		if ( ! function_exists( 'download_url' ) || ! function_exists( 'media_handle_sideload' ) )  {
			if ( ! $this->vb_include_once( ABSPATH . 'wp-admin' . '/includes/image.php' ) ) {
				return 0;
			}
			if ( ! $this->vb_include_once( ABSPATH . 'wp-admin' . '/includes/file.php' ) ) {
				return 0;
			}
			if ( ! $this->vb_include_once( ABSPATH . 'wp-admin' . '/includes/media.php' ) ) {
				return 0;
			}
		}
		$ext = pathinfo( $img, PATHINFO_EXTENSION );
		if ( ! $ext ) {	// handles dynamic images for example Hulu, Spotify - pain in the ass
			$ext = "jpg";
			$unsafe = TRUE;
			add_filter('http_request_reject_unsafe_urls', '__return_false');
		}
		$this->info_message( sprintf( __( '%s: about to download img %s', 'autovid-profit' ), __FUNCTION__, $img ), 'updated', 'debug' );
    		$tmp = download_url( $img );
    		if ( is_wp_error( $tmp ) ) {	// this usually means the given thumb was not found on remote site
			return $this->info_message( sprintf( __( 'WP error %s in %s for download_url() of %s - %s', 'autovid-profit' ), $tmp->get_error_code(), __FUNCTION__, $img, $tmp->get_error_message() ) );
		}
		if ( ! file_exists( $tmp ) ) {
			return $this->info_message( sprintf( __( 'tmp file %s does not exist! check that WordPress can create a file in that directory...', 'autovid-profit' ), $tmp ) );
		}
		$filename = $custom_name ? $custom_name : 'vbp-' . $postID . '-' . $post_title . '.' . $ext;
		$filename = sanitize_file_name( $filename );
		// remove any unsafe characters:
		$filename = preg_replace( "/[^a-zA-Z0-9_\-.]/", '', $filename );
    		$file_array = array(
			'name' => $filename,
			'tmp_name' => $tmp
		);

		$this->info_message( sprintf( __( '%s: about to handle sideload of %s', 'autovid-profit' ), __FUNCTION__, print_r( $file_array,true ) ), 'updated', 'debug' );
    		$thumbID = media_handle_sideload( $file_array, $postID );
    		if ( is_wp_error( $thumbID ) )  {
			@unlink( $tmp );
			return $this->info_message( sprintf( __( 'WP error %s in %s for media_handle_sideload() of %s to %s - %s', 'autovid-profit' ), $thumbID->get_error_code(), __FUNCTION__, $img, $tmp, $thumbID->get_error_message() ) );
		}

		if ( isset( $unsafe ) ) {
			add_filter('http_request_reject_unsafe_urls', '__return_true');
		}

		$this->info_message( sprintf( __( '%s: about to set_post_thumbnail for thumbID %s.', 'autovid-profit' ), __FUNCTION__, $thumbID ), 'updated', 'debug' );
		$mediaID = set_post_thumbnail( $postID, $thumbID );
		if ( ! $mediaID ) {
			@unlink( $tmp );
			return $this->info_message( sprintf( __( 'Error: set_post_thumbnail (%s, %s) returned FALSE ', 'autovid-profit' ), $postID, $thumbID ) );
		}
		if ( file_exists( $tmp ) ) {
			@unlink( $tmp );	
		}
		$this->info_message( sprintf( __( 'Image [%s] added to media library for post id %s.', 'autovid-profit' ), $filename, $postID ), 'updated', 'debug' );
		return $thumbID;
	}

	/*
	 * Get a list of posts for video site
	 */
	public function get_posts_by_site( $videoSite = 'YouTube', $offset = 0 ) {
		$this->info_message( sprintf( __( '%s(%s)', 'autovid-profit' ), __FUNCTION__, $videoSite ), 'updated', 'debug' );
		$chunksize = apply_filters( 'vb_posts_chunksize', 1000 );
		$query_args = array(
			'posts_per_page' 	=> $chunksize,
			'offset'		=> $offset,
			'post_type' 		=> 'any',
			'post_status' 		=> array( 'pending', 'draft', 'publish', 'private' ),
			'meta_query' => array(
				'relation'		=> 'OR',
				array(
					'key' 		=> 'VideoSource',
					'value'		=> $videoSite,
				),
				array( // mvb compatability check:
					'key' 		=> 'mvb_vid_source',
					'value' 	=> strtolower( $videoSite ),
					'compare'	=> 'IN'
				),
			)
		);
		$query_args = apply_filters( 'vb_get_posts_by_site', $query_args );
		wp_suspend_cache_addition(true);
		$posts = get_posts( $query_args );
		$this->info_message( sprintf( __( 'Found %d videos, offset %d.', 'autovid-profit' ), count($posts), $offset ), 'updated', 'debug' );
		return $posts;
	}

	/*
	 * See if specific post title already exists
	 */
	public function post_title_already_exists( $title ) {
		global $wpdb;
		$title = sanitize_title( $title );
		$dbstring = "SELECT ID,post_title,post_status FROM $wpdb->posts WHERE post_title = '{$title}'";
		$dbstring = apply_filters( 'vb_post_title_already_exists', $dbstring, $title );
		$this->info_message( sprintf( __( '%s - %s', 'autovid-profit' ), __FUNCTION__, $dbstring ), 'updated', 'debug' );
		$query = $wpdb->get_results( $dbstring );
		if ( ! empty( $query ) ) {
			$this->info_message( sprintf( __( '%s - Found duplicate title [%s] already in %s post id: <a target="_blank" href="%s">%s</a>. %s', 'autovid-profit' ), __FUNCTION__, $title, $query[0]->post_status, get_edit_post_link( $query[0]->ID ), $query[0]->ID, print_r($query,true) ), 'updated', 'debug' );
			return 1;
		}
		$this->info_message( sprintf( __( '%s - title [%s] was not found in any existing posts.', 'autovid-profit' ), __FUNCTION__, $title ), 'updated', 'debug' );
		return 0;
	}

	/*
	 * See if specific video is on the blacklist
	 * should use video ID
	 */
	public function video_on_blacklist( $videoID ) {
		$blacklist = get_option( $this->get_my_plugin( 'prefix' ) . 'video_blacklist', array() );
		foreach ( $blacklist as $bl ) {
			if ( stripos( $bl, $videoID ) !== FALSE ) {
				return 1;
			}
		}
		return 0;
	}


	/*
	 * See if specific video already exists
	 * Autovid Profit - Checks using meta 'VideoSource' and the unique 'VideoID'
	 * My PHP Video Blog - Checks using meta 'mvb_vid_source' and the unique 'mvb_vid_code'
	 */
	public function post_already_exists( $videoInfo, $support = 'blogster' ) {
		$post_types = $this->get_post_types( '', 'names' );
		$query_args = array(
		 	'post_type' => $post_types,
		 	'post_status' => array( 'draft', 'future', 'pending', 'private', 'publish', 'trash' ),
		 	'posts_per_page' => 1,
		);
		if ( 'mvb' == $support ) { // mvb compatability query check:
			if ( $videoInfo['videoSource'] != 'YouTube' 
				&& $videoInfo['videoSource'] != 'DailyMotion'
				&& $videoInfo['videoSource'] != 'Hulu'
				&& $videoInfo['videoSource'] != 'PornHub'
				&& $videoInfo['videoSource'] != 'xVideos'
				&& $videoInfo['videoSource'] != 'RedTube'
			) {
				return 0;
			}
			$query_args['meta_query'] = array(
				array(
					'key' => 'mvb_vid_source',
					'value' => strtolower( $videoInfo['videoSource'] ),
					'meta_compare'	=> 'IN'
				),
				array(
					'key' => 'mvb_vid_code',
					'value' => $videoInfo['videoID']
				)
			);
		}
		else if ( 'cbc' == $support ) { // YouTube Video to Post
			$query_args['meta_query'] = array(
				array(
					'key' => '__cbc_video_id',
					'value' => $videoInfo['videoID']
				)
			);
		}
		else if ( 'filter' == $support ) { // user can define his own args for query check:
			$query_args = apply_filters( 'vb_post_already_exists', $query_args );
			if ( empty( $query_args['meta_query'] ) ) { // user did not set a meta_query so abort
				return 0;
			}
		}
		else { // Autovid Profit default query check:
			$query_args['meta_query'] = array(
				array(
					'key' => 'VideoSource',
					'value' => $videoInfo['videoSource'],
				),
				array(
					'key' => 'VideoID',
					'value' => $videoInfo['videoID']
				)
			);
		}
		$this->info_message( sprintf( __( 'Post already exists() query: %s', 'autovid-profit' ), print_r($query_args,true) ), 'updated', 'debug' );
 		$query = get_posts( $query_args );
		wp_reset_postdata();
		if ( ! empty( $query ) ) {
			$this->info_message( sprintf( __( 'Found duplicate videoID [%s] already in %s post id: <a target="_blank" href="%s">%s</a>.', 'autovid-profit' ), $videoInfo['videoID'], $query[0]->post_status, get_edit_post_link( $query[0]->ID ), $query[0]->ID ), 'updated', 'debug' );
			return 1;
		}
		return 0;
	}

	/*
	 * Add our own arguments onto the PUC Checker
	 */
	public function PUC_callback( $queryArgs ) {
		$queryArgs['license'] = get_option( $this->get_my_plugin( 'prefix' ) . 'license' );
		$queryArgs['code'] = get_option( $this->get_my_plugin( 'prefix' ) . 'purchase_code' );
		return $queryArgs;
	}

	/*
	 * TODO: Removes images associated with Trashed posts created by VBP
	 */
	private function remove_images_on_trashed_posts() {
		$this->info_message( sprintf( __( '%s: Start', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'utility_funcs' );
		$getMore = true;
		$offset = 0;
		$chunksize = apply_filters( 'vb_posts_chunksize', 1000 );
		$query_args = array(
			'meta_key' => 'VideoID',
			'post_type' => 'any',
			'post_status' => array( 'trash' ),
			'posts_per_page' => $chunksize,
			);
		$query_args = apply_filters( 'vb_remove_images_on_trashed_posts', $query_args );
		wp_suspend_cache_addition(true);

		$total = 0;
		while ( $getMore ) {
			$posts = get_posts( $query_args );
			$num_results = count( $posts );
			foreach ( $posts as $post ) {
	
				$thumbID = get_post_thumbnail_id( $post->ID );
				if ( ! $thumbID ) {
					continue;
				}
				wp_delete_attachment( $thumbID );
				delete_post_thumbnail( $post );
				$total++;
			}
			$offset += $chunksize;
			$query_args['offset'] = $offset;
			if ( $num_results < $chunksize ) $getMore = false;
			$this->extend_execution_time();
		}
		if ( $total ) {
			$this->info_message( sprintf( __( 'Removed images from %s posts found in Trash', 'autovid-profit' ), $total ), 'updated', 'critical' );
		}
		$this->info_message( sprintf( __( '%s: End', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'utility_funcs' );
	}

	/*
	 * Show and process show video campaign page
	 */
	public function show_utility_functions() {
		$this->determine_video_modules();
		$this->get_options_to_vars();
		$this->messages['utility_funcs'] = true; // since we're in admin, show util msgs no matter what
		$this->display_msgs = 1;
		$tools = array();

		$tools['broken_embeds'] = isset( $_POST['tool_broken_embeds'] ) ? $_POST['tool_broken_embeds'] : 'no';
		$tools['regenerate_thumbnails'] = isset( $_POST['tool_regenerate_thumbnails'] ) ? $_POST['tool_regenerate_thumbnails'] : 'no';
		$tools['remove_images'] = isset( $_POST['tool_remove_images'] ) ? $_POST['tool_remove_images'] : 'no';

		if ( isset( $_POST['save_values'] ) ) {
			update_option( $this->get_my_plugin( 'prefix' ) . 'tools', $tools);
			$this->info_message( __( 'Settings saved.', 'autovid-profit' ), 'updated', 'critical' );
		}
		else if ( isset( $_POST['launch_tools'] ) ) {
			if ( $tools['broken_embeds'] == 'yes' ) {
				$this->info_message( __( 'checking for broken embeds Start', 'autovid-profit' ), 'updated', 'utility_funcs' );
				foreach ( $this->modules as $module ) {
					$this->create_video_source( array( 'videoSource' => $module ) );
					if ( $this->video_source ) {
						$this->video_source->check_for_deleted_videos();
					}
				}
				$this->info_message( __( 'checking for broken embeds End', 'autovid-profit' ), 'updated', 'utility_funcs' );
			}
			if ( $tools['regenerate_thumbnails'] == 'yes' ) {
				$this->regenerate_thumbnails();
			}
			if ( $tools['remove_images'] == 'yes' ) {
				$this->remove_images_on_trashed_posts();
			}
		}

		do_action( 'custom_utility_function' );

		include( sprintf( "%s/templates/show-utility-functions.php", $this->get_my_plugin('dir') ) );
		$this->display_msgs = 0;
	}

	/**
	 * Checks posts to see if they have post thumbnails
	 * If not, try fetching again with VideoImage meta tag
	 */
	public function regenerate_thumbnails( $post_id = 0 ) {
		$this->info_message( sprintf( __( '%s: Start', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'utility_funcs' );
		$getMore = true;
		$offset = 0;
		$chunksize = apply_filters( 'vb_posts_chunksize', 1000 );
		$query_args = array(
			'meta_key' => 'VideoImage',
			'post_type' => 'any',
			'post_status' => array( 'pending', 'draft', 'publish', 'private' ),
			'posts_per_page' => $chunksize,
			);
		if ( $post_id ) { // TODO: regenerate thumb check for particular post_id
			$query_args['p'] = $post_id;
		}
		$query_args = apply_filters( 'vb_regenerate_thumbnails', $query_args );
		wp_suspend_cache_addition(true);

		$total = 0;
		while ( $getMore ) {
			$posts = get_posts( $query_args );
			$num_results = count( $posts );
			foreach ( $posts as $post ) {
				if ( has_post_thumbnail( $post->ID ) ) {
					continue;
				}
				$videoID = get_post_meta( $post->ID, 'VideoID', TRUE );
				$img = get_post_meta( $post->ID, 'VideoImage', TRUE );
				if ( empty( $img ) ) continue;
				$videoImage = array (
					'img'		=> $img,
					'post_title'	=> $post->post_title
				);
				$thumbID = $this->grab_thumbnail( $post->ID, $videoImage );
				if ( $thumbID ) {
					$this->info_message( sprintf( __( 'Fetched missing thumb %s for post ID <a target="_blank" href="%s">%s</a>, video ID: %s', 'autovid-profit' ), get_permalink( $post->ID ), $img, $post->ID, $videoID ), 'updated', 'utility_funcs' );
					$total++;
				}
				else {
					$this->info_message( sprintf( __( 'Failure fetching missing thumb %s for post ID <a target="_blank" href="%s">%s</a>, video ID: %s', 'autovid-profit' ), $img, get_permalink( $post->ID ), $post->ID, $videoID ), 'error', 'critical' );
				}
			}
			$offset += $chunksize;
			$query_args['offset'] = $offset;
			if ( $num_results < $chunksize ) $getMore = false;
			$this->extend_execution_time();
		}
		if ( $total ) {
			$this->info_message( sprintf( __( '%s missing thumbs fetched', 'autovid-profit' ), $total ), 'updated', 'critical' );
		}
		$this->info_message( sprintf( __( '%s: End', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'utility_funcs' );
	}
	
	/**
	 * Checks post before it is deleted to see if video should be added to video blacklist
	 */
	public function add_to_blacklist( $post_id ) {
		$blacklist_auto = get_option( $this->get_my_plugin( 'prefix' ) . 'blacklist_auto' );
		if ( FALSE == $blacklist_auto ) {
			return $post_id;
		}
		$videoID = get_post_meta( $post_id, 'VideoID', TRUE );

		if ( empty( $videoID ) ) {
			return $post_id;
		}

		$blacklist = get_option( $this->get_my_plugin( 'prefix' ) . 'video_blacklist' );

		if ( in_array( $videoID, $blacklist ) ) {
			return $post_id;
		}

		$blacklist[] = $videoID;

		update_option( $this->get_my_plugin( 'prefix' ) . 'video_blacklist', $blacklist);

		return $post_id;
	}


    } // END class Autovid_Profit_Engine
} // END if ( ! class_exists( 'Autovid_Profit_Engine' ) )

////////////////////////////////////////////////////////////////////////////////////////////

 
