<?php

/*
 * Prevent Direct Access
 */
defined( 'ABSPATH' ) or die( "Sorry, Can not access page directly." );

/**
 * Class for handling the log file
 */
if ( ! class_exists( 'Autovid_Profit_Log' ) ) {
    class Autovid_Profit_Log {

	private $plugin = null;
	private $logfile = null;

	protected static $instance = null;

	public function __construct() { }

	public static function getInstance() {
		if ( !isset( self::$instance ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/*
	 * Register actions and filters when object created
	 */
	public function init( $params = array() ) {
		$this->plugin = $params;
		$this->logfile = get_option( $this->get_my_plugin('prefix') . 'logfile' );
		if ( ! $this->logfile ) {
			$this->logfile = '/tmp/logfile.log';	// catchall if settings are boinked
		}
	}

	/*
	 * Wrapper to get the plugin params used to create this class object
	 */
	public function get_my_plugin( $key ) {
		return isset( $this->plugin[$key] ) ? $this->plugin[$key] : null;
	}

	/**
	 * Clear log file if 'Clear Log' pressed.
	 * Load the log-file template to display logfile.
	 */
	public function show_logfile() { 
		if ( isset( $_POST['save_values'] ) )  {
			$prefix = $this->get_my_plugin( 'prefix' );
			$logfile = isset( $_POST['autovid_profit_logfile'] ) ? trim( stripslashes( $_POST['autovid_profit_logfile'] ) ) : '';
			if (empty ( $logfile ) ) {
				$logfile = $this->get_my_plugin( 'dir' ) . 'logfile.log';
			}
			update_option( $prefix . 'logfile', $logfile );
			$messages = array(
				'video_skip'		=> isset( $_POST['messages_video_skip'] ) ? true : false,
				'video_import'		=> isset( $_POST['messages_video_import'] ) ? true : false,
				'utility_funcs'		=> isset( $_POST['messages_utility_funcs'] ) ? true : false,
				'debug'			=> isset( $_POST['messages_debug'] ) ? true : false
			);
			update_option( $prefix . 'messages', $messages );
			echo "<div id='message' class='updated'><p>" . __( 'Settings saved.', 'autovid-profit' ) . "</p></div>";
		}
		else if ( isset( $_POST['clear_log'] ) )  {
			$this->clear_the_log();
		}

		if ( ! file_exists( $this->logfile ) ) {
			$this->write_to_log( 'Import videos to create a logfile.' );
		}
		// load the last 1 MB of the file in case it is very big - don't want script to run out of memory
		$filesize = filesize( $this->logfile );
		$msg = file_get_contents( $this->logfile, false, null, $filesize - 1000000, 1000000);
		if ( ! $msg ) {
			echo "<div id='message' class='error'><p>" . __( 'Unable to open Log File for reading! Does it exist?', 'autovid-profit' ) . "</p></div>";
		}
		include( sprintf( "%s/templates/log-file.php", $this->get_my_plugin('dir') ) );
	}

	/**
	 * Writes a string to the log file
	 */
	public function write_to_log( $msg ) {
        	$fh = @fopen( $this->logfile, "a" );
        	if ( ! $fh ) {
			return 0;
		}
        	fwrite( $fh, date( 'Y-m-d H:i:s',current_time("timestamp") ) . " - " . $msg . "\n" );
        	fclose( $fh );
		return 1;
	}

	/**
	 * Clears the log file
	 */
	public function clear_the_log() {
                $fh = @fopen( $this->logfile, "w" );
                if ( ! $fh ) {
			echo "<div id='message' class='error'><p>" . __( 'Unable to open Log File for writing! Check permissions.', 'autovid-profit' ) . "</p></div>";
		}
                else {
                        @fwrite( $fh, date( 'Y-m-d H:i:s',current_time("timestamp") ) . " <strong><font color=\"#FF0000\">Log file cleared</font></strong>\n" );
                        @fclose( $fh );
                }
		return 1;
	}

    } // END class Autovid_Profit_Log
} // END if ( ! class_exists( 'Autovid_Profit_Log' ) )
?>
