<?php

/* 
 * Support for https://wordpress.org/plugins/contus-video-gallery/
 * Used by Premium themes like Snaptube http://themeforest.net/item/snaptube-premium-video-wordpress-theme/8026657
 *
 * Autovid Profit will automatically include this file if APPTHA_VGALLERY_BASEDIR is defined (by the Contus plugin).
 *
 * The action 'vb_create_the_post' will be called right before Autovid Profit creates the WordPress post.
 * The function below will have Contus Video Gallery create its custom WordPress post instead.
 * It will also add any tags into the Contus custom tag table.
 * It will also add the video to the Contus custom playlist table, creating the playlist record if necessary.
 *
 * Then it will return the custom post ID so that Autovid Profit knows the custom post was created.
 * Autovid Profit will then import and attach any video comments to the post as well as the custom meta fields.
 *
 * Contus supports YouTube and DailyMotion
 */

// =============================================================================================================

if ( ! function_exists( 'add_to_contus_video_gallery' ) ) :

add_action('vb_create_the_post', 'add_to_contus_video_gallery', 10, 2);

// Stop Autovid Profit from creating a post, have WordPress Video Gallery do it instead with their custom setup
function add_to_contus_video_gallery( $vidpost, $videoInfo ) {
	global $wpdb;

	$args = array( 
		'name' 		=> 'Contus support', 
		'prefix' 	=> 'autovid_profit_', 
		'dir' 		=> WP_PLUGIN_DIR . '/autovid-profit'
	);
	$vbp = new Autovid_Profit_Core( $args );
	if ( !$vbp ) {
		// this should never happen
		error_log( __FUNCTION__, "VBP: Could not create Autovid_Profit_Core instance.");
		return $vidpost;
	}
	if( current_user_can( 'manage_options' ) ) {
		// display msgs to screen 
		$vbp->display_msgs = TRUE;
	}

	$file =  APPTHA_VGALLERY_BASEDIR .  '/admin/models/video.php';

	if ( ! file_exists( $file ) ) {
		$vbp->info_message( sprintf( __( '%s file %s was not found.', 'autovid-profit' ), __FUNCTION__, $file ) );
		return $vidpost;	// Contus WordPress Video Gallery plugin must exist and be activated
	}
	include_once( $file );

	$file =  APPTHA_VGALLERY_BASEDIR .  '/admin/models/playlist.php';
	if ( ! file_exists( $file ) ) {
		$vbp->info_message( sprintf( __( '%s file %s was not found.', 'autovid-profit' ), __FUNCTION__, $file ) );
		return $vidpost;	// Contus WordPress Video Gallery plugin must exist and be activated
	}
	include_once( $file );

	// translate Autovid Profit data to Contus custom WordPress Video Gallery data

	$ordering       = $wpdb->get_var( 'SELECT count( ordering ) FROM ' . $wpdb->prefix . 'hdflvvideoshare' );

	$videoData = array(
		'name'			=> $videoInfo['post_title'],
		'description'		=> $videoInfo['desc'],
		'embedcode'		=> $videoInfo['videoEmbed'],
		'file'			=> addslashes( trim( $videoInfo['url'] ) ),
		'file_type'		=> 1,
		'member_id'		=> $vidpost['post_author'],
		'duration'		=> ! empty ( $videoInfo['duration'] ) ? $videoInfo['duration'] : '0:00',
		'hdfile'		=> '',
		'streamer_path'		=> '',
		'islive'		=> 0,
		'image'			=> $videoInfo['img'],
		'opimage'		=> $videoInfo['img'],
		'srtfile1'		=> '',
		'srtfile2'		=> '',
		'subtitle_lang1'	=> '',
		'subtitle_lang2'	=> '',
		'link'			=> $videoInfo['url'],
		'featured'		=> 0,
		'hitcount' 		=> $videoInfo['viewCount'],
		'ratecount'		=> round($videoInfo['likeCount']/5),
		'rate' 			=> $videoInfo['likeCount']-$videoInfo['dislikeCount'],
		'download'		=> 0,
		'postrollads'		=> 0,
		'midrollads'		=> 0,
		'imaad'			=> 0,
		'prerollads'		=> 0,
		'publish'		=> 1,
		'google_adsense'	=> 0,
		'google_adsense_value'	=> 0,
		'amazon_buckets'	=> 0,
		'post_date'		=> $vidpost['post_date'],
		'ordering'		=> $ordering,
		'slug'			=> ''
	);

	// user can use this hook to determine if a video should be 'featured'
	$videoData = apply_filters( 'vb_contus_data', $videoData, $videoInfo );

	$vbp->info_message( sprintf( __( '%s Inserting videoData: %s', 'autovid-profit' ), __FUNCTION__, htmlentities( print_r($videoData,true) ) ), 'updated', 'debug' );

	// insert video in post_table
	$vm = new VideoModel();
	if ( ! $vm ) {
		$vbp->info_message( sprintf( __( '%s Unable to create new VideoModel.', 'autovid-profit' ), __FUNCTION__ ) );
		return $vidpost;
	}
	$vm->member_id = $vidpost['post_author']; # damn you Contus!
	$contusID = $vm->insert_video( $videoData );
	if ( ! $contusID ) {
		$vbp->info_message( sprintf( __( '%s VideoModel->insert_video returned 0.', 'autovid-profit' ), __FUNCTION__ ) );
		return $vidpost;
	}

	$vbp->info_message( sprintf( __( '%s Received contusID: %s', 'autovid-profit' ), __FUNCTION__, $contusID ), 'updated', 'debug' );

	// let's get the postID attached to the Contus video we just inserted.
	$postID = $wpdb->get_var( 'SELECT slug FROM ' . $wpdb->prefix . 'hdflvvideoshare WHERE vid ='.$contusID );
	if ( ! $postID ) {
		$vbp->info_message( sprintf( __( '%s Unable to find matching postID in %shdflvvideoshare WHERE vid = %s', 'autovid-profit' ), __FUNCTION__, $wpdb->prefix, $contusID ) );
		return $vidpost;
	}

	/// PTA EDIT 150915
	$confirmID = $wpdb->get_var( 'SELECT ID FROM ' . $wpdb->prefix . 'posts order by ID desc LIMIT 1' );
	if ( $postID != $confirmID ) {
		$vbp->info_message( sprintf( __( '%s Error 150915 Contus postID %s does not match last postID %s! Resetting to last postID...', 'autovid-profit' ), __FUNCTION__, $postID, $confirmID ) );
		$guid = get_site_url() . '/?post_type=videogallery&#038;p=' . $confirmID;
		$wpdb->update( $wpdb->prefix . 'posts', array( 'guid' => $guid ), array( 'ID' => $confirmID ) );
		$wpdb->update( $wpdb->prefix . 'hdflvvideoshare', array( 'slug' => $confirmID ), array( 'vid' => $last_insert_video_id ) );
	}


	$vbp->info_message( sprintf( __( '%s Received slug/postID: %s', 'autovid-profit' ), __FUNCTION__, $postID ), 'updated', 'debug' );

	// Contus also uses its own tags:
	if ( ! empty( $videoInfo['tags'] ) ) {
		$tags_name = $videoInfo['tags'];
		// Contus doesn't handle commas without spaces after. let's be safe about it.
		if ( FALSE === strpos( $tags_name, ', ' ) ) {
			$tags_name = str_replace( ",", ", ", $tags_name );
		}

                $strip_tags_name     = strtolower( stripslashes( $tags_name ) );
                $ambersand_tags_name = preg_replace( '/[&:\s]+/i', '-', $strip_tags_name );
                $spl_tags_name           = preg_replace( '/[#!@$%^.,:;\/&*(  ){}\"\'\[\]<>|?]+/i', '', $ambersand_tags_name );
                $seo_tags_name           = preg_replace( '/---|--+/i', '-', $spl_tags_name );
		$s = 'INSERT INTO ' . $wpdb->prefix . 'hdflvvideoshare_tags ( media_id,tags_name,seo_name ) VALUES ( '.$contusID.', "'.$tags_name.'", "'.$seo_tags_name.'" )';
		$vbp->info_message( sprintf( __( '%s Tags: %s', 'autovid-profit' ), __FUNCTION__, $s ), 'updated', 'debug' );
		$result = $wpdb->query( $s );
		if ( FALSE === $result ) {
			$vbp->info_message( sprintf( __( '%s Unable to inserts tags into %shdflvvideoshare_tags', 'autovid-profit' ), __FUNCTION__, $wpdb->prefix ) );
		}
	}


	// set specific Contus categories
	$cats = $vidpost['post_category'];

	if ( empty( $cats ) ) {
		$catOne = $wpdb->get_results( "SELECT pid FROM " . $wpdb->prefix . "hdflvvideoshare_playlist WHERE is_publish=1 LIMIT 1" );
		$cats = $catOne[0]->pid;
		$vbp->info_message( sprintf( __( '%s - Contus category was not selected in video campaign %s! Defaulting to Contus category ID %s.', 'autovid-profit' ), __FUNCTION__, $videoInfo['campaignID'], $cats ) );
	}

	if ( ! is_array( $cats ) ) { 
		$cats = explode( ",", $cats );
	}

	foreach ( $cats as $video_playlist ) {
		if ( empty( $video_playlist ) ) {
			continue;
		}
		$s = "INSERT INTO " . $wpdb->prefix . "hdflvvideoshare_med2play (media_id, playlist_id, porder, sorder)
			VALUES ('" . $contusID . "', '" . $video_playlist . "', '0', '0')";
		$vbp->info_message( sprintf( __( '%s Category: %s', 'autovid-profit' ), __FUNCTION__, $s ), 'updated', 'debug' );
		$result = $wpdb->query( $s );
		if ( FALSE === $result ) {
			$vbp->info_message( sprintf( __( '%s Unable to insert category %s into %shdflvvideoshare_med2play', 'autovid-profit' ), __FUNCTION__, $video_playlist, $wpdb->prefix ) );
		}
	}


	// return postID so Autovid Profit will add meta fields and comments! 
	return $postID;
}

// =============================================================================================================

add_action('vb_create_category', 'create_contus_category', 10, 1);

// when Autovid Profit wants to create the category imported from the video, create Contus category instead of WP
function create_contus_category( $playlistName ) {
	global $wpdb;

	$file =  APPTHA_VGALLERY_BASEDIR .  '/admin/models/playlist.php';
	if ( ! file_exists( $file ) ) {
		#$vbp->info_message( sprintf( __( '%s file %s was not found.', 'autovid-profit' ), __FUNCTION__, $file ) );
		error_log( sprintf( __( '%s file %s was not found.', 'autovid-profit' ), __FUNCTION__, $file ) );
		return $vidpost;	// Contus WordPress Video Gallery plugin must exist and be activated
	}
	include_once( $file );

	$video_playlist = $wpdb->get_var( "SELECT pid FROM " . $wpdb->prefix . "hdflvvideoshare_playlist WHERE playlist_name = '" . $playlistName . "'" );

	// only create if it doesn't exist already
	if ( ! $video_playlist ) { 
                $playlsitData = array(
                	'playlist_name'         	=> $playlistName,
                	'playlist_slugname' 		=> sanitize_title( $playlistName ),
                	'is_publish'            	=> 1
                );
                $ordering = $wpdb->get_var( "SELECT MAX(playlist_order) FROM ".$wpdb->prefix."hdflvvideoshare_playlist" );
                $playlsitData['playlist_order'] = $ordering + 1;

		$plm = new PlaylistModel();
		$video_playlist = $plm->insert_playlist( $playlsitData );
	}

	// return playlist number so VBP won't create WP category
	return $video_playlist;
}

// =============================================================================================================

add_action('vb_get_categories', 'get_contus_categories', 10, 1);

// If no Contus cat set, access post with http://www.domain.com/wp-admin/edit.php?post_type=videogallery
// when Autovid Profit wants to get the WP categories, return Contus categories instead
function get_contus_categories( $categories ) {
	global $wpdb;

	$cats = $wpdb->get_results( "SELECT pid, playlist_name FROM " . $wpdb->prefix . "hdflvvideoshare_playlist WHERE is_publish=1" );

	$categories = array();

	foreach ( $cats as $cat ) {
		$obj = new stdClass();
		$obj->term_id = $cat->pid;
		$obj->name = $cat->playlist_name;
		$categories[] = $obj;
	}
	return $categories;
}
// =============================================================================================================

add_action('vb_delete_video', 'delete_from_contus_video_gallery', 10, 1);

// When Autovid Profit flags a post for trash, delete it from Video Gallery custom tables first
function delete_from_contus_video_gallery( $postID ) {
	global $wpdb;

	$args = array( 
		'name' 		=> 'Contus support', 
		'prefix' 	=> 'autovid_profit_', 
		'dir' 		=> WP_PLUGIN_DIR . '/autovid-profit-pro'
	);
	$vbp = new Autovid_Profit_Core( $args );
	if ( !$vbp ) {
		// this should never happen
		error_log( __FUNCTION__, "VBP: Could not create Autovid_Profit_Core instance.");
		return $vidpost;
	}
	if( current_user_can( 'manage_options' ) ) {
		// display msgs to screen 
		$vbp->display_msgs = TRUE;
	}

	$file =  APPTHA_VGALLERY_BASEDIR .  '/admin/models/video.php';

	if ( ! file_exists( $file ) ) {
		$vbp->info_message( sprintf( __( '%s File %s was not found.', 'autovid-profit' ), __FUNCTION__, $file ) );
		return 0;	// Contus WordPress Video Gallery plugin must exist and be activated
	}
	include_once( $file );

	// Get Video Gallery ID 
	$video_id = $wpdb->get_var( "SELECT vid FROM " . $wpdb->prefix . "hdflvvideoshare WHERE slug = '" . $postID . "'" ); 

	if ( empty ( $video_id ) ) {
		// could not find...
		$vbp->info_message( sprintf( __( '%s could not find postID %s in %shdflvvideoshare.', 'autovid-profit' ), __FUNCTION__, $postID, $wpdb->prefix ) );
		return 0;
	}

	$vbp->info_message( sprintf( __( '%s got video_id %s from postID %s in %shdflvvideoshare.', 'autovid-profit' ), __FUNCTION__, $video_id, $postID, $wpdb->prefix ), 'updated', 'debug' );

	// delete video in post_table
	$vm = new VideoModel();
	if ( ! $vm ) {
		$vbp->info_message( sprintf( __( '%s Unable to create new VideoModel.', 'autovid-profit' ), __FUNCTION__ ) );
		return 0;
	}
	$deleteflag = $vm->video_delete( $video_id );

	if ( FALSE === $deleteflag ) {
		$vbp->info_message( sprintf( __( '%s VideoModel->video_delete returned FALSE.', 'autovid-profit' ), __FUNCTION__ ) );
	}
	else {
		$vbp->info_message( sprintf( __( '%s VideoModel->video_delete returned %s.', 'autovid-profit' ), __FUNCTION__, $deleteflag ), 'updated', 'debug' );
	}

	// Contus handles the custom delete, so return 0 so Autovid Profit doesn't call wp_trash_post
	return 0;
}


/* Support for plugin Contus WordPress Video Gallery
 *
 * This function hook will set the videogallery post type and reset the description.
 *
 */

// shows changes on make video campaign form:
add_action('vb_make_campaign', 'vb_insert_contus_changes', 10, 1);

// ensures these fields are used at run time (in case of theme change or similar):
add_action('vb_video_args', 'vb_insert_contus_changes', 10, 1);

function vb_insert_contus_changes( $args ) {
	$args['pPostTemplate'] = '%VideoDescription%';
        $args['cPostType'] = 'videogallery';
        return $args;
}

endif;
?>
