<?php

/*
 * Prevent Direct Access
 */
defined( 'ABSPATH' ) or die( "Sorry, Can not access page directly." );

/**
 * Class for Autovid Profit MixCloud support
 * https://www.mixcloud.com/developers/
 */
if ( ! class_exists( 'Autovid_Profit_MixCloud' )) {
    class Autovid_Profit_MixCloud {

	private $vbp = null;			// will point to main AutoVid_Profit instance
        private $query_fields = array();        // options for current query
        private $num = 0;        		// Internal counter
        private $comment_num = 0;       	// Internal counter

	/**
	 * Create MixCloud video source
	 * Point back to Autovid Profit object to use common functions
	 * Save the query fields for easy access
	 */
	public function __construct( $vbp, $query_fields ) {
		$this->vbp = $vbp;
		$this->query_fields = $query_fields;
	}

        /**
         * Make the query and check for errors.
         */
        private function queryApi( $url )
        {
                $this->vbp->info_message( __FUNCTION__ . ': ' . htmlentities( $url ), 'updated', 'debug' );
		$response = wp_remote_get( $url );
                if ( is_wp_error( $response ) ) {
			return $this->vbp->info_message( sprintf( __( 'WP error %s in %s(%s) - %s ', 'autovid-profit' ), $response->get_error_code(), __FUNCTION__, $url, $response->get_error_message() ) );
                }
                $data = json_decode( wp_remote_retrieve_body( $response ) );

                if ( isset( $data->errors ) ) {
                        return $this->vbp->info_message( sprintf( __( '%s(%s) - MixCloud API returned %s', 'autovid-profit' ), __FUNCTION__, $url, $data->errors[0]->error_message ) );
                }
                if ( empty( $data ) )  {
                        $this->vbp->info_message( sprintf( __( '%s(%s) - MixCloud Error: no results found for query', 'autovid-profit' ), __FUNCTION__, $url ) );
                }
                return $data;
        }


	private function grab_comments_batch( $url, $postID ) {
		$totalComments = $this->query_fields['qNumComments'];

		$result = $this->queryApi( $url );

		$comments = isset( $result->data ) ? $result->data : $result;

		if ( empty( $comments ) ) {
			return null;
		}

		foreach ( $comments as $comment ) {
			if ( $this->comment_num < 50 ) {
				$this->vbp->info_message( sprintf (__( 'COMMENT: %s', 'autovid-profit' ), htmlentities( print_r($comment,TRUE) ) ), 'updated', 'debug' );
			}
			$author = isset( $comment->user->username ) ? trim( $comment->user->username ) : '';
			$author_url = isset( $comment->user->url ) ? trim( $comment->user->url ) : '';
			$content = isset( $comment->comment ) ? trim( $comment->comment ) : '';
			if ( empty( $author ) || empty( $content ) ) {
				continue;
			}
			$comment_date = current_time( 'Y-m-d H:i:s' );
			if ( TRUE == $this->query_fields['cUsePublishedDate'] ) {
				$commentAt = isset( $comment->submit_date ) ? $comment->submit_date : $comment_date;
				$comment_date = $this->vbp->getDateTime( $commentAt, $videoInfo['url'] );
			}
			if ( TRUE == $this->query_fields['pRemoveCommUrls'] ) {
				$content = $this->vbp->remove_urls( $content );
			}
			if ( TRUE == $this->query_fields['pLinkifyCommUrls'] ) {
				$content = $this->vbp->linkify_filtered( $content );
			}
			$commentdata = array(
				'comment_post_ID'       => $postID,
				'comment_author'        => $author,
				'comment_author_email'  => '',
				'comment_author_url'    => $author_url,
				'comment_content'       => $content,
				'comment_type'  	=> '',
				'comment_date'          => $comment_date,
				'comment_approved'      => 1
				);
			$commentdata = apply_filters( 'mixcloud_comment', $commentdata);
			if ( empty( $commentdata ) ) {
				continue;
			}
			$comment_id = wp_insert_comment( $commentdata );
			if ( $comment_id ) {
				$this->comment_num++;
			}
			if ( $totalComments > 0 && $this->comment_num >= $totalComments ) {
				return null;
			}
		}
		return isset( $result->next_href ) ? $result->next_href : null;
	}


	/**
	 * Get MixCloud comments on a track
	 */
	private function grab_comments( $url, $postID ) {
                $cm = get_option( 'comment_moderation' );
                $cn = get_option( 'comment_notification' );
                // temporarily turn off manual approval and email notification
                if ( 1 == $cm ) {
                        update_option( 'comment_moderation', FALSE );
                }
                if ( 1 == $cn ) {
                        update_option( 'comment_notification', FALSE );
		}

		$url = str_replace("www", "api", $url."comments/");

		$this->comment_num = 0;

		$url = apply_filters( 'mixcloud_comments_query', $url, $this->query_fields );

		while ( $url ) {
			$url = $this->grab_comments_batch( $url, $postID );
		}

		$this->vbp->info_message( sprintf( __( '%s comments imported successfully to post id %s.', 'autovid-profit' ), $this->comment_num, $postID ), 'updated', 'video_import' );

                if ( 1 == $cm ) {
                        update_option( 'comment_moderation', $cm );
                }
                if ( 1 == $cn ) {
                        update_option( 'comment_notification', $cn );
                }
	}

        /*
         * Save the video info as a WordPress post
         */
	private function save_mixcloud_entry( $videoInfo ) {

		$postID = $this->vbp->save_the_video( $this->query_fields, $videoInfo );

		if ( ! $postID ) return;

		if ( ! empty( $this->query_fields['qNumComments'] ) && $videoInfo['commentCount'] ) {
			$this->grab_comments( $videoInfo['url'], $postID );
		}

		$this->vbp->publish_the_video( $postID, $this->query_fields, $videoInfo );
	}

        /**
         * Find the best match thumbnail to what the user requested. 
	 * If it is not defined this code will cascade to the closest match.
         */
	private function get_best_thumbnail( $track ) {
		$img = array();
		$match = false;

		if ( ! $match && isset( $track->pictures->small ) ) $img[] = $track->pictures->small;
		if ( $this->query_fields['qExtraParams2'] == 'small' ) $match = true;

		if ( ! $match && isset( $track->pictures->thumbnail ) ) $img[] = $track->pictures->thumbnail;
		if ( $this->query_fields['qExtraParams2'] == 'thumbnail' ) $match = true;

		if ( ! $match && isset( $track->pictures->medium_mobile ) ) $img[] = $track->pictures->medium_mobile;
		if ( $this->query_fields['qExtraParams2'] == 'medium_mobile' ) $match = true;

		if ( ! $match && isset( $track->pictures->medium ) ) $img[] = $track->pictures->medium;
		if ( $this->query_fields['qExtraParams2'] == 'medium' ) $match = true;

		if ( ! $match && isset( $track->pictures->large ) ) $img[] = $track->pictures->large;
		if ( $this->query_fields['qExtraParams2'] == 'large' ) $match = true;

		if ( ! $match && isset( $track->pictures->{'320wx320h'} ) ) $img[] = $track->pictures->{'320wx320h'};
		if ( $this->query_fields['qExtraParams2'] == '320wx320h' ) $match = true;

		if ( ! $match && isset( $track->pictures->extra_large ) ) $img[] = $track->pictures->extra_large;
		if ( $this->query_fields['qExtraParams2'] == 'extra_large' ) $match = true;

		if ( ! $match && isset( $track->pictures->{'640wx640h'} ) ) $img[] = $track->pictures->{'640wx640h'};
		if ( $this->query_fields['qExtraParams2'] == '640wx640h' ) $match = true;

		if ( ! $match && isset( $track->pictures->{'768wx768h'} ) ) $img[] = $track->pictures->{'768wx768h'};
		if ( $this->query_fields['qExtraParams2'] == '768wx768h' ) $match = true;

		if ( ! $match && isset( $track->pictures->{'1024wx1024h'} ) ) $img[] = $track->pictures->{'1024wx1024h'};
		if ( $this->query_fields['qExtraParams2'] == '1024wx1024h' ) $match = true;

		$img = array_reverse( $img );

		$image = $this->vbp->verify_thumbnails( $img );

		return $image;
	}

        /**
         * Extract video details into our generic videoInfo array
         */
        private function get_video_info( $track ) {
		$videoInfo = array();

		$videoInfo['orig_title'] = $videoInfo['title'] = isset( $track->name ) ? $track->name : '';
		$videoInfo['orig_desc'] = $videoInfo['desc'] = isset( $track->description ) ? $track->description : '';
		$videoInfo['association'] = isset( $track->user->username ) ? $track->user->username : '';
		$videoInfo['videoID'] = isset( $track->slug ) ? $track->slug : '';
		$videoInfo['url'] = isset( $track->url ) ? $track->url : '';

		$this->vbp->info_message( sprintf( __( 'Checking MixCloud track: [%s] at %s', 'autovid-profit' ), $videoInfo['title'], $videoInfo['url'] ), 'updated', 'video_import' );

		$videoInfo['duration'] = isset( $track->audio_length ) ? $track->audio_length : 0;
		$videoInfo['duration'] = $this->vbp->seconds_to_time( $videoInfo['duration'] );
		$videoInfo['viewCount'] = isset( $track->play_count ) ? $track->play_count : '';
		$videoInfo['likeCount'] = $videoInfo['favoriteCount'] = isset( $track->favorite_count ) ? $track->favorite_count : '';
		$videoInfo['dislikeCount'] = 0;
		$videoInfo['commentCount'] = isset( $track->comment_count ) ? $track->comment_count : '';
		$videoInfo['img'] = $this->get_best_thumbnail( $track );


		// Make a separate call to MixCloud oembed for embed code 
		$url = "https://www.mixcloud.com/oembed?url=" . htmlentities( $videoInfo['url'] ) . "&format=json";
		$videoInfo['videoEmbed'] = $this->vbp->queryoEmbed( $url );

		$publishedAt = isset( $track->created_time ) ? $track->created_time : null;
		$date = $this->vbp->getDateTime( $publishedAt, $videoInfo['url'] );
		$videoInfo['publishedAt'] = $date->format( 'Y-m-d H:i:s' );
		$videoInfo['publishedAt8601'] = isset( $date ) ? $date->format( 'c' ) : '';

		$videoInfo['tags'] = '';
		if ( TRUE == $this->query_fields['pGrabTags'] && isset( $track->tags ) ) {
			$tags = array();
			foreach ( $track->tags as $tag ) {
				$tags[] = $tag->name;
			}
			$videoInfo['tags'] = implode( ",", $tags );
		}

		$videoInfo['videoSource'] = 'MixCloud';

		$videoInfo = apply_filters( 'get_video_info', $videoInfo );

		$this->vbp->info_message( sprintf( __( '%s: %s', 'autovid-profit' ), __FUNCTION__, htmlentities( print_r($videoInfo,true) ) ), 'updated', 'debug' );

		return $videoInfo;
	}


	public function grab_videos_batch( $url ) {
		$maxItems = $this->query_fields['qNumVideos'];

		$result = $this->queryApi( $url );
		$tracks = isset( $result->data ) ? $result->data : $result;

		if ( empty( $tracks ) ) {
			return $this->vbp->info_message( sprintf (__( '%s: no results returned for %s', 'autovid-profit' ), __FUNCTION__, $url ), 'error', 'critical' );
		}

		if ( ! is_array( $tracks ) ) {
			// specific track is a standard object, not an array, so make array of the stdobj
			$stdObj = $tracks;
			$tracks = array( $stdObj );
		}

		foreach ( $tracks as $track ) {

			$videoInfo = $this->get_video_info( $track );

			$this->save_mixcloud_entry( $videoInfo );

			$this->num++;
			if ( $this->num >= $maxItems ) {
				return null;
			}
		}
		return isset( $result->paging->next ) ? $result->paging->next : null;
	}

	public function grab_videos() {
		$maxItems = $this->query_fields['qNumVideos'];
		$keyphrase = urlencode( $this->query_fields['qKeyphrase'] );
		$assocType = $this->query_fields['qAssocType'];
		$assoc =  $this->query_fields['qAssoc'];	# by id or url 

		if ( empty( $keyphrase ) && ( empty( $assoc ) || empty( $assocType ) ) ) {
			return $this->vbp->info_message( __( 'Error: No MixCloud keyphrase or association not set properly.', 'autovid-profit' ), 'error', 'critical' );
		}

		$url = $assoc;
		if ( stripos( $assoc, 'http://' ) !== FALSE || stripos( $assoc, 'https://' ) !== FALSE ) {
			$url = str_replace( "www", "api", $assoc );
		}
		$url = rtrim( $url, '/' ) . '/';

		if ( $keyphrase && ! $assoc ) {
			$url = "http://api.mixcloud.com/search/?type=cloudcast&q=" . $keyphrase;
			# limit is 100 per query. use it?
			#$url = "http://api.mixcloud.com/search/?type=cloudcast&limit=" . $maxItems . "&q=" . $keyphrase;
		}
		else if ( $assocType == 'user' ) {
			$url .= 'cloudcasts/';
		}
		else if ( $assocType == 'playlist' ) {
			$url .= 'cloudcasts/';
		}
		else if ( $assocType == 'show' ) {
		}

		$this->num = 0;

		$url = apply_filters( 'mixcloud_search_query', $url, $this->query_fields );

		while ( $url ) {
			$url = $this->grab_videos_batch( $url );
		}
	}

	/**
	 * MixCloud 
	 */
	public function check_for_deleted_videos() {
		$this->vbp->info_message( sprintf( __( 'MixCloud %s Start', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'utility_funcs' );


		$getMore = true;
		$offset = 0;
		$chunksize = apply_filters( 'vb_posts_chunksize', 1000 );

		while ( $getMore ) {
		$posts = $this->vbp->get_posts_by_site( 'MixCloud', $offset );
		$num_results = count( $posts );
		foreach ( $posts as $post ) {
			$VideoUrl = get_post_meta( $post->ID, 'VideoUrl', TRUE );
			$this->vbp->info_message( sprintf( __( 'Schedule Checker: checking MixCloud track url: %s ', 'autovid-profit' ), $url ), 'updated', 'utility_funcs' );
			$url = str_replace("www", "api", $VideoUrl);
			$track = $this->queryApi( $url );
			if ( isset( $track->error->message ) && $track->error->message == 'Upload does not exist.' ) {
				// are there other error messages we should check for? unknown.
				$postID = apply_filters( 'vb_delete_video', $post->ID );
				if ( ! $postID ) {
					continue;
				}
				wp_trash_post( $postID );
				$this->vbp->info_message( sprintf( __( 'Schedule Checker: MixCloud track url %s not found. Sending post %s to trash.', 'autovid-profit' ), $VideoUrl, $postID ), 'updated', 'critical' );
			}
		} // end foreach posts
			$offset += $chunksize;
			$query_args['offset'] = $offset;
			if ( $num_results < $chunksize ) $getMore = false;
			$this->vbp->extend_execution_time();
		} // end getMore
	}

    } // END class Autovid_Profit_MixCloud
} // END if ( ! class_exists( 'Autovid_Profit_MixCloud' ) )


?>
