<?php

/*
 * Prevent Direct Access
 */
defined( 'ABSPATH' ) or die( "Sorry, Can not access page directly." );

/**
 * Class for Autovid Profit RSS support
 */
if ( ! class_exists( 'Autovid_Profit_RSS' )) {
    class Autovid_Profit_RSS {

	private $vbp = null;			// will point to main AutoVid_Profit instance
        private $query_fields = array();        // options for current query

	/**
	 * Create RSS video source
	 * Point back to Autovid Profit object to use common functions
	 * Save the query fields for easy access
	 */
	public function __construct( $vbp, $query_fields ) {
		$this->vbp = $vbp;
		$this->query_fields = $query_fields;
	}

        /*
         * Save the video info as a WordPress post
         */
	private function save_rss_entry( $videoInfo ) {

		$postID = $this->vbp->save_the_video( $this->query_fields, $videoInfo );

		if ( ! $postID ) return;

		$this->vbp->publish_the_video( $postID, $this->query_fields, $videoInfo );
	}

	public function grab_videos() {
		$maxItems = $this->query_fields['qNumVideos'];
		$assoc =  esc_url_raw( strip_tags( $this->query_fields['qAssoc'] ) );
		$filter_cats = $this->query_fields['qKeyphrase'];

		if ( empty( $assoc ) ) {
			return $this->vbp->info_message( __( 'Error: No RSS campaign has been set', 'autovid-profit' ), 'error', 'video_import' );
		}

		/* let's NOT use WordPress campaign functions - problems with filter wp_campaign_options. :(
		$rss = fetch_campaign( $assoc );
		if ( is_wp_error( $rss ) ) {
			return $this->vbp->info_message( sprintf (__( 'Error %s with fetch_campaign: %s', 'autovid-profit' ), $rss->get_error_code(), $rss->get_error_message() ), 'error', 'video_import' );
		}
		*/

		// use SimplePie directly:
		require_once( ABSPATH . WPINC . '/class-campaign.php' );
		$rss = new SimplePie();
		if ( empty( $rss ) ) {
			return $this->vbp->info_message( sprintf (__( 'Error with fetch_campaign: %s', 'autovid-profit' ), $assoc ), 'error', 'video_import' );
		}
		$rss->set_campaign_url( $assoc );
		$rss->strip_htmltags( false ); #MARK make this optional?
		$rss->enable_cache( false );
		$rss->init();
		$rss->handle_content_type();
		
		$rss_items = $rss->get_items( 0, $maxItems );


		foreach ( $rss_items as $item ) {
			$videoInfo = array();

			$videoInfo['orig_title'] = $videoInfo['title'] = $item->get_title();
			$videoInfo['orig_desc'] = $videoInfo['desc'] = $item->get_content();
			$this->vbp->info_message( sprintf( __( 'Checking RSS campaign item: [%s] from %s', 'autovid-profit' ), $videoInfo['title'], $assoc ), 'updated', 'video_import' );
			$this->vbp->info_message( sprintf (__( 'RSS campaign entry: %s', 'autovid-profit' ), $videoInfo['title'] ), 'updated', 'debug' );
			$this->vbp->info_message( sprintf (__( 'RSS campaign content: %s', 'autovid-profit' ), htmlentities( $videoInfo['desc'] ) ), 'updated', 'debug' );

			#MARK remove "The post X appeared first on Y" ?

			$author = $item->get_author();
			$videoInfo['association'] = isset( $author ) ? $author->get_name() : '';
			$videoInfo['videoID'] = $item->get_id();
			$videoInfo['url'] = $item->get_link(); // default: url of campaign item
			$videoInfo['videoEmbed'] = '';

			$publishedAt = $item->get_date();
			$date = $this->vbp->getDateTime( $publishedAt, $videoInfo['url'] );
			$videoInfo['publishedAt'] = $date ? $date->format( 'Y-m-d H:i:s' ) : current_time( 'Y-m-d H:i:s' );
			$videoInfo['publishedAt8601'] = $date ? $date->format( 'c' ) : current_time( 'c' );

			$videoInfo['publishedAt8601'] = isset( $date ) ? $date->format( 'c' ) : '';

			$videoInfo['duration'] = '';
			$videoInfo['viewCount'] = $videoInfo['likeCount'] = $videoInfo['dislikeCount'] = $videoInfo['favoriteCount'] = $videoInfo['commentCount'] = 0;

			#see if enclosure exists
			foreach ( $item->get_enclosures() as $enclosure ) {

				// grab the first thumbnail declared, if any:
				if ( empty( $videoInfo['img'] ) ) {
					$videoInfo['img'] = $enclosure->get_thumbnail(); // grabs <media:thumbnail>
				}

				// check all media content, if any:
				if ( $media = $enclosure->get_link() ) { 
					$type = $enclosure->get_type();
					$medium = $enclosure->get_medium();
					$this->vbp->info_message( sprintf (__( '[%s] enclosure media: %s', 'autovid-profit' ), $videoInfo['title'], $media ), 'updated', 'debug' );
					$this->vbp->info_message( sprintf (__( '[%s] enclosure type: %s', 'autovid-profit' ), $videoInfo['title'], ! empty( $type ) ? $type : '(not found)' ), 'updated', 'debug' );
					$this->vbp->info_message( sprintf (__( '[%s] enclosure medium: %s', 'autovid-profit' ), $videoInfo['title'], ! empty( $medium ) ? $medium : '(not found)' ), 'updated', 'debug' );
					if ( ( $medium == 'image' || $type == 'image/jpeg' ) && empty( $videoInfo['img'] ) ) {
						$videoInfo['img'] = $media;
						$this->vbp->info_message( sprintf (__( '[%s] enclosure img %s:', 'autovid-profit' ), $videoInfo['title'], $videoInfo['img'] ), 'updated', 'debug' );
					}
					else if ( $medium == 'video' || FALSE !== strpos( $type, 'video') ) {
						$videoInfo['videoEmbed'] = $media;
						$videoInfo['duration'] = $this->vbp->seconds_to_time( $enclosure->get_duration() ); 
						$this->vbp->info_message( sprintf (__( '[%s] enclosure video detected: %s', 'autovid-profit' ), $videoInfo['title'], $videoInfo['videoEmbed'] ), 'updated', 'debug' );
						$this->vbp->info_message( sprintf (__( '[%s] enclosure duration converted from %s to %s', 'autovid-profit' ), $videoInfo['title'], $enclosure->get_duration(), $videoInfo['duration'] ), 'updated', 'debug' );
					}
				}
			}
			#if no enclosure thumbnail defined, try to get first image from the content
			if ( ! isset( $videoInfo['img'] ) || empty( $videoInfo['img'] ) ) {
				preg_match_all(' /<img.+src=[\'"]([^\'"]+)[\'"].*>/i', $videoInfo['desc'], $matches );
				$videoInfo['img'] = isset( $matches[1][0] ) ? $matches[1][0] : '';
			}

			$videoInfo['sourceCategory'] = $videoInfo['tags'] = '';
			$tags = array();
			$cats = $item->get_categories();
			if ( ! empty( $cats ) ) {
				foreach ( $cats as $cat ) {
					$tags[] = $cat->get_label();
				}
			}

			#filter entries by given category, if any
			#make the comparison case insensitive by making both lowercase...
			if ( ! empty ( $filter_cats ) ) {
				$this->vbp->info_message( sprintf (__( '%s: filter entries by categories %s', 'autovid-profit' ), __FUNCTION__, $filter_cats ), 'updated', 'debug' );
				if ( empty( $tags ) ) { 
					$this->vbp->info_message( sprintf (__( '%s: RSS item [%s] has no category declared.', 'autovid-profit' ), __FUNCTION__, $videoInfo['title'] ), 'updated', 'debug' );
					continue;
				}
				$lc_tags = array_map( 'strtolower', $tags );
				$cat_match = false;
				$fcats = explode( ",", strtolower( $filter_cats ) );
				foreach ( $fcats as $fcat ) {
					if ( in_array( trim( $fcat ), $lc_tags ) ) {
						$cat_match = true;
						$this->vbp->info_message( sprintf (__( '%s: RSS item [%s] matches filter.', 'autovid-profit' ), __FUNCTION__, $videoInfo['title'] ), 'updated', 'debug' );
						break;
					}
				}
				if ( false === $cat_match ) {
					$this->vbp->info_message( sprintf (__( '%s: RSS item [%s] does not match filter.', 'autovid-profit' ), __FUNCTION__, $videoInfo['title'] ), 'updated', 'debug' );
					unset( $tags );
					continue;
				}
			}

			#categories & tags are mixed in RSS campaign... save all as tags
			if ( TRUE == $this->query_fields['pGrabTags'] && isset( $tags ) ) {
				$videoInfo['tags'] = implode( ',', $tags );
			}
			unset( $tags );

			$videoInfo['videoSource'] = 'RSS';

			$videoInfo = apply_filters( 'get_video_info', $videoInfo );

			$this->vbp->info_message( sprintf (__( 'VIDEOINFO: %s', 'autovid-profit' ), htmlentities( print_r($videoInfo,TRUE) ) ), 'updated', 'debug' );
			$this->save_rss_entry( $videoInfo );
		}

	}

	/**
	 * RSS campaign are fully imported - no need to check for deletions from source site
	 */
	public function check_for_deleted_videos() {
	}

    } // END class Autovid_Profit_RSS
} // END if ( ! class_exists( 'Autovid_Profit_RSS' ) )

?>
