<?php

/*
 * Prevent Direct Access
 */
defined( 'ABSPATH' ) or die( "Sorry, Can not access page directly." );

/**
 * Class for Autovid Profit SoundCloud support
 */
if ( ! class_exists( 'Autovid_Profit_SoundCloud' )) {
    class Autovid_Profit_SoundCloud {

	private $vbp = null;			// will point to main AutoVid_Profit instance
        private $query_fields = array();        // options for current query
        private $client_id = null;        	// SoundCloud client_id for Autovid Profit
        private $num = 0;        		// Internal counter
        private $comment_num = 0;       	// Internal counter

	/**
	 * Create SoundCloud video source
	 * Point back to Autovid Profit object to use common functions
	 * Save the query fields for easy access
	 */
	public function __construct( $vbp, $query_fields, $client_id ) {
		$this->vbp = $vbp;
		$this->query_fields = $query_fields;
		$this->client_id = $client_id;
	}

        /**
         * Make the query and check for errors.
         */
        private function queryApi( $url )
        {
                $this->vbp->info_message( __FUNCTION__ . ': ' . htmlentities( $url ), 'updated', 'debug' );
		$response = wp_remote_get( $url );
                if ( is_wp_error( $response ) ) {
			return $this->vbp->info_message( sprintf( __( 'WP error %s in %s(%s) - %s ', 'autovid-profit' ), $response->get_error_code(), __FUNCTION__, $url, $response->get_error_message() ) );
                }
                $data = json_decode( wp_remote_retrieve_body( $response ) );

                if ( isset( $data->errors ) ) {
                        return $this->vbp->info_message( sprintf( __( '%s(%s) - SoundCloud API returned %s', 'autovid-profit' ), __FUNCTION__, $url, $data->errors[0]->error_message ) );
                }
                if ( empty( $data ) )  {
                        $this->vbp->info_message( sprintf( __( '%s(%s) - SoundCloud Error: no results found for query', 'autovid-profit' ), __FUNCTION__, $url ) );
                }
                return $data;
        }


	private function grab_comments_batch( $url, $postID ) {
		$totalComments = $this->query_fields['qNumComments'];

		$result = $this->queryApi( $url );

		$comments = isset( $result->collection ) ? $result->collection : $result;

		if ( empty( $comments ) ) {
			return null;
		}

		foreach ( $comments as $comment ) {
			if ( $this->comment_num < 50 ) {
				$this->vbp->info_message( sprintf (__( 'COMMENT: %s', 'autovid-profit' ), htmlentities( print_r($comment,TRUE) ) ), 'updated', 'debug' );
			}
			$author = isset( $comment->user->username ) ? trim( $comment->user->username ) : '';
			$author_url = isset( $comment->user->permalink_url ) ? trim( $comment->user->permalink_url ) : '';
			$content = isset( $comment->body ) ? trim( $comment->body ) : '';
			if ( empty( $author ) || empty( $content ) ) {
				continue;
			}
			$comment_date = current_time( 'Y-m-d H:i:s' );
			if ( TRUE == $this->query_fields['cUsePublishedDate'] ) {
				$commentAt = isset( $comment->created_at ) ? $comment->created_at : current_time( 'Y-m-d H:i:s' );
				$commentDate = $this->vbp->getDateTime( $commentAt, $url );
				$comment_date = isset( $commentDate ) ? $commentDate->format('Y-m-d H:i:s') : $commentAt;
			}
			if ( TRUE == $this->query_fields['pRemoveCommUrls'] ) {
				$content = $this->vbp->remove_urls( $content );
			}
			if ( TRUE == $this->query_fields['pLinkifyCommUrls'] ) {
				$content = $this->vbp->linkify_filtered( $content );
			}
			$commentdata = array(
				'comment_post_ID'       => $postID,
				'comment_author'        => $author,
				'comment_author_email'  => '',
				'comment_author_url'    => $author_url,
				'comment_content'       => $content,
				'comment_type'  	=> '',
				'comment_date'          => $comment_date,
				'comment_approved'      => 1
				);
			$commentdata = apply_filters( 'soundcloud_comment', $commentdata);
			if ( empty( $commentdata ) ) {
				continue;
			}
			$comment_id = wp_insert_comment( $commentdata );
			if ( $comment_id ) {
				$this->comment_num++;
			}
			if ( $totalComments > 0 && $this->comment_num >= $totalComments ) {
				return null;
			}
		}
		return isset( $result->next_href ) ? $result->next_href : null;
	}


	/**
	 * Get SoundCloud comments on a track
	 */
	private function grab_comments( $videoID, $postID ) {
                $cm = get_option( 'comment_moderation' );
                $cn = get_option( 'comment_notification' );
                // temporarily turn off manual approval and email notification
                if ( 1 == $cm ) {
                        update_option( 'comment_moderation', FALSE );
                }
                if ( 1 == $cn ) {
                        update_option( 'comment_notification', FALSE );
		}

		$url = "http://api.soundcloud.com/tracks/" . $videoID . "/comments.json?client_id=" . $this->client_id;

		$url .= "&linked_partitioning=1";

		$this->comment_num = 0;

		$url = apply_filters( 'soundcloud_comments_query', $url, $this->query_fields );

		while ( $url ) {
			$url = $this->grab_comments_batch( $url, $postID );
		}

		$this->vbp->info_message( sprintf( __( '%s comments imported successfully to post id %s.', 'autovid-profit' ), $this->comment_num, $postID ), 'updated', 'video_import' );

                if ( 1 == $cm ) {
                        update_option( 'comment_moderation', $cm );
                }
                if ( 1 == $cn ) {
                        update_option( 'comment_notification', $cn );
                }
	}

        /*
         * Save the video info as a WordPress post
         */
	private function save_soundcloud_entry( $videoInfo ) {

		$postID = $this->vbp->save_the_video( $this->query_fields, $videoInfo );

		if ( ! $postID ) return;

		if ( ! empty( $this->query_fields['qNumComments'] ) && $videoInfo['commentCount'] ) {
			$this->grab_comments( $videoInfo['videoID'], $postID );
		}

		$this->vbp->publish_the_video( $postID, $this->query_fields, $videoInfo );
	}

	private function get_best_thumbnail( $track ) {
		$img = array();

		$size = isset( $this->query_fields['qExtraParams2'] ) ? $this->query_fields['qExtraParams2'] : 't500x500';

		if ( isset( $track->user->avatar_url ) ) {
			$img[] = str_replace( 'large', $size, $track->user->avatar_url );
		}
		if ( isset( $track->artwork_url ) ) {
			$img[] = str_replace( 'large', $size, $track->artwork_url );
		}

		$img = array_reverse( $img );

		$image = $this->vbp->verify_thumbnails( $img );

		return $image;
	}

	public function grab_videos_batch( $url ) {
		$maxItems = $this->query_fields['qNumVideos'];

		$result = $this->queryApi( $url );
		$tracks = isset( $result->collection ) ? $result->collection : $result;

		if ( empty( $result ) ) {
			return $this->vbp->info_message( sprintf (__( '%s: no results returned for %s', 'autovid-profit' ), __FUNCTION__, $url ), 'error', 'critical' );
		}

		if ( ! is_array( $tracks ) ) {
			// specific track is a standard object, not an array, so make array of the stdobj
			$stdObj = $tracks;
			$tracks = array( $stdObj );
		}

		foreach ( $tracks as $track ) {

			$videoInfo = array();

			$videoInfo['orig_title'] = $videoInfo['title'] = isset( $track->title ) ? $track->title : '';
			$videoInfo['orig_desc'] = $videoInfo['desc'] = isset( $track->description ) ? $track->description : '';
			$videoInfo['association'] = isset( $track->user->username ) ? $track->user->username : '';
			$videoInfo['videoID'] = isset( $track->id ) ? $track->id : '';
			$videoInfo['url'] = isset( $track->permalink_url ) ? $track->permalink_url : '';

			$this->vbp->info_message( sprintf( __( 'Checking Soundcloud track: [%s] at %s', 'autovid-profit' ), $videoInfo['title'], $videoInfo['url'] ), 'updated', 'video_import' );

			$videoInfo['duration'] = isset( $track->duration ) ? $track->duration : 0;
			$videoInfo['duration'] = $this->vbp->seconds_to_time( $videoInfo['duration'] / 1000 );
			$videoInfo['viewCount'] = isset( $track->playback_count ) ? $track->playback_count : '';
			$videoInfo['likeCount'] = $videoInfo['favoriteCount'] = isset( $track->favoritings_count ) ? $track->favoritings_count : '';
			$videoInfo['dislikeCount'] = 0;
			$videoInfo['commentCount'] = isset( $track->comment_count ) ? $track->comment_count : '';
			$videoInfo['img'] = $this->get_best_thumbnail( $track );


			// Make a separate call to Soundcloud oembed for embed code 
			$url = "https://www.soundcloud.com/oembed?url=" . htmlentities( $videoInfo['url'] ) . "/&format=json";
			$videoInfo['videoEmbed'] = $this->vbp->queryoEmbed( $url );

			$publishedAt = isset( $track->created_at ) ? $track->created_at : null;
			$date = $this->vbp->getDateTime( $publishedAt, $videoInfo['url'] );
			$videoInfo['publishedAt'] = $date->format( 'Y-m-d H:i:s' );
			$videoInfo['publishedAt8601'] = isset( $date ) ? $date->format( 'c' ) : '';

			$videoInfo['sourceCategory'] = '';
			if ( TRUE == $this->query_fields['cAddSourceCategory'] && isset( $track->genre ) ) {
				$videoInfo['sourceCategoryName'] = $track->genre;
				$videoInfo['sourceCategory'] = $this->vbp->create_the_category( $track->genre );
			}

			$videoInfo['tags'] = '';
			if ( TRUE == $this->query_fields['pGrabTags'] && isset( $track->tag_list ) ) {
				$videoInfo['tags'] = rtrim( $track->tag_list );
				$videoInfo['tags'] = preg_replace('/(\\s)(?=(?:[^"]|"[^"]*")*$)/', ',', $videoInfo['tags'] );
				$videoInfo['tags'] = str_replace( "\"", "", $videoInfo['tags'] );
			}

			$videoInfo['videoSource'] = 'SoundCloud';

			$videoInfo = apply_filters( 'get_video_info', $videoInfo );

			$this->vbp->info_message( sprintf (__( 'VIDEOINFO: %s', 'autovid-profit' ), htmlentities( print_r($videoInfo,TRUE) ) ), 'updated', 'debug' );
			$this->save_soundcloud_entry( $videoInfo );

			$this->num++;
			if ( $this->num >= $maxItems ) {
				return null;
			}
		}
		return isset( $result->next_href ) ? $result->next_href : null;
	}

	public function grab_videos() {

		$assoc =  $this->query_fields['qAssoc'];	# group, playlist, user, or track
		$assocType = $this->query_fields['qAssocType'];
		$keyphrase = urlencode( $this->query_fields['qKeyphrase'] );
		$filters = ! empty( $this->query_fields['qExtraParams'] ) ? "&" . ltrim( $this->query_fields['qExtraParams'], "&" ) : '';

                if ( empty( $keyphrase ) && ( empty( $assoc ) || empty( $assocType ) ) ) {
                        return $this->vbp->info_message( __( 'Error: No SoundCloud keyphrase or association not set properly.', 'autovid-profit' ), 'error', 'critical' );
                }

		if ( stripos( $assoc, 'http://' ) !== FALSE || stripos( $assoc, 'https://' ) !== FALSE ) {
			// resolve SoundCloud url to get id!
			$url = "http://api.soundcloud.com/resolve.json?url=" . $assoc . "&client_id=" . $this->client_id;
			$item = $this->queryApi( $url );
			if ( empty( $item ) ) {
				return; 
			}
			$this->vbp->info_message( sprintf (__( 'SoundCloud resolved [%s] to get item id %s', 'autovid-profit' ), $assoc, $item->id), 'updated', 'debug' );
			$assoc = $item->id;
		}

		if ( $keyphrase && ! $assoc ) {
			$url = "http://api.soundcloud.com/tracks.json?client_id=" . $this->client_id;
		}
		else if ( $assocType == 'track' ) { // fetch specific track
			$url = "http://api.soundcloud.com/tracks/" . $assoc . ".json?client_id=" . $this->client_id;
		}
		else if ( $assocType == 'user' ) {
			$url = "http://api.soundcloud.com/users/" . $assoc . "/tracks.json?client_id=" . $this->client_id;
		}
		else if ( $assocType == 'playlist' ) {
			$url = "http://api.soundcloud.com/playlists/" . $assoc . "/tracks.json?client_id=" . $this->client_id;
		}
		else if ( $assocType == 'group' ) {
			$url = "http://api.soundcloud.com/groups/" . $assoc . "/tracks.json?client_id=" . $this->client_id;
		}

		if ( $keyphrase ) {
			$url .= "&q=" . $keyphrase;
		}
		if ( $filters ) {
			$url .= $filters;
		}

		$url .= "&linked_partitioning=1";

		$this->num = 0;

		$url = apply_filters( 'soundcloud_search_query', $url, $this->query_fields );

		while ( $url ) {
			$url = $this->grab_videos_batch( $url );
		}
	}

	/**
	 * SoundCloud 
	 */
	public function check_for_deleted_videos() {
		$this->vbp->info_message( sprintf( __( 'SoundCloud %s Start', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'utility_funcs' );

		$getMore = true;
		$offset = 0;
		$chunksize = apply_filters( 'vb_posts_chunksize', 1000 );

		while ( $getMore ) {
		$posts = $this->vbp->get_posts_by_site( 'SoundCloud', $offset );
		$num_results = count( $posts );
		foreach ( $posts as $post ) {
			$videoID = get_post_meta( $post->ID, 'VideoID', TRUE );
			if ( ! $videoID ) {
				continue;
			}
			$url = get_post_meta( $post->ID, 'VideoUrl', TRUE );
			$this->vbp->info_message( sprintf( __( 'Schedule Checker: checking SoundCloud track id %s, url: %s ', 'autovid-profit' ), $videoID, $url ), 'updated', 'utility_funcs' );
			$url = "http://api.soundcloud.com/tracks/" . $videoID . ".json?client_id=" . $this->client_id;
			$track = $this->queryApi( $url );
			if ( empty( $track ) ) {
				$postID = apply_filters( 'vb_delete_video', $post->ID );
				if ( ! $postID ) {
					continue;
				}
				wp_trash_post( $postID );
				$this->vbp->info_message( sprintf( __( 'Schedule Checker: SoundCloud video %s not found. Sending post %s to trash.', 'autovid-profit' ), $videoID, $postID ), 'updated', 'critical' );
			}
		} // end foreach posts
			$offset += $chunksize;
			$query_args['offset'] = $offset;
			if ( $num_results < $chunksize ) $getMore = false;
			$this->vbp->extend_execution_time();
		} // end getMore
	}

    } // END class Autovid_Profit_SoundCloud
} // END if ( ! class_exists( 'Autovid_Profit_SoundCloud' ) )


?>
